"""
A function that creates colormap legends given an input colormap.
"""

from enthought.util.numerix import arange, transpose, repeat, NewAxis

from image_plot_value import ImageData, CmapImagePlotValue
from plot_axis import PlotAxis


def ColormapLegend(colormap, orientation='vertical', margin_width = 10,\
                          margin_height = 10, grid_visible=False, \
                          tick_visible = True, **plotvalue_traits):
    """
    Returns a new PlotValue representing a colormap legend.  This can/should
    be placed inside a PlotGroup with the ImagePlotValue, and the legend
    PlotValue's .weight attribute can be used to make it narrower.
    """
    
    # value_step is the amount of value that each colormap step represents
    value_step = float(colormap.max_value - colormap.min_value) / colormap.steps
    
    # generate the X and Y indices for the array we'll use to draw the colormap
    y_index = arange(colormap.min_value, colormap.max_value + value_step/2, value_step)
    
    # Because 1xN arrays are rendered by the image renderer as thin lines,
    # we extrude the data array to form a 2xN array.
    value_grid = transpose(repeat(y_index[:, NewAxis], 2, axis=1))
    
    # The "X values" along the extruded dimension of the colormap array.
    x_index = arange(value_grid.shape[0])
    
    index_vals = (x_index, y_index)

    imagedata = ImageData(value_grid, index_vals)
    
    x_axis = PlotAxis(tick_visible=False, grid_visible=False, \
                      bound_low = index_vals[0][0], bound_high = index_vals[0][-1])
    x_axis.visible = False
    y_axis = PlotAxis(tick_visible=tick_visible, grid_visible=grid_visible, \
                      bound_low = colormap.min_value, bound_high = colormap.max_value)
    pv = CmapImagePlotValue(imagedata, colormap, axis_index = x_axis, axis = y_axis,\
                        type='image', **plotvalue_traits)
    
    pv.margin_width = margin_width
    pv.margin_height = margin_height
    
    return pv
    
