#--------------------------------------------------------------------------------
#
#  Define a 'grid' plot data source class
#
#  Written by: David C. Morrill
#
#  Date: 05/06/2003
#
#  (c) Copyright 2003 by Enthought, Inc.
#
#  Classes defined: Grid
#
#--------------------------------------------------------------------------------

#--------------------------------------------------------------------------------
#  Imports:
#--------------------------------------------------------------------------------

from enthought.chaco.plot_base import *
from enthought.chaco.plot_data import PlotData
from enthought.traits.api          import Trait, Range, TraitError

#-------------------------------------------------------------------------------
#  Check for a valid data point:
#-------------------------------------------------------------------------------

simple_numeric_types = [ IntType, FloatType ]

def is_point ( object, name, value ):
    if isinstance(value, basestring):
        try:
            value = eval( value )
        except:
            raise TraitError
    if ((type( value ) in basic_sequence_types)    and
        (len( value ) == 2)                        and
        (type( value[0] ) in simple_numeric_types) and
        (type( value[1] ) in simple_numeric_types)):
        return value
    raise TraitError

is_point.info = ('a point of the form (x,y) or [x,y], where x and y are '
                 'integers or floats')

#-------------------------------------------------------------------------------
#  'Grid' class:
#-------------------------------------------------------------------------------

count_trait = Range( 0, 50, value = 3 )

class Grid ( PlotData ):

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    p0 = Trait( ( 0.0, 0.0 ), is_point )
    p1 = Trait( ( 1.0, 0.0 ), is_point )
    p2 = Trait( ( 0.0, 1.0 ), is_point )
    p3 = Trait( ( 1.0, 1.0 ), is_point )
    n0 = count_trait
    n1 = count_trait

    #----------------------------------------------------------------------------
    #  Get the associated stock data for the specified stock symbol:
    #----------------------------------------------------------------------------

    def data ( self, source_name ):
        result = []
        n0     = self.n0
        if n0 >= 1:
            p0   = array( self.p0 )
            p1   = array( self.p1 )
            p2   = array( self.p2 )
            p3   = array( self.p3 )
            dxy0 = (p1 - p0) / float( n0 )
            dxy2 = (p3 - p2) / float( n0 )
            for i in range( n0 + 1 ):
                result.append( p0.tolist() )
                result.append( p2.tolist() )
                p0 = p0 + dxy0
                p2 = p2 + dxy2
        n1 = self.n1
        if n1 >= 1:
            p0   = array( self.p0 )
            p1   = array( self.p1 )
            p2   = array( self.p2 )
            p3   = array( self.p3 )
            dxy0 = (p2 - p0) / float( n1 )
            dxy1 = (p3 - p1) / float( n1 )
            for i in range( n1 + 1 ):
                result.append( p0.tolist() )
                result.append( p1.tolist() )
                p0 = p0 + dxy0
                p1 = p1 + dxy1
        return result

    #----------------------------------------------------------------------------
    #  Add one or more source connections to a plottable object:
    #----------------------------------------------------------------------------

    def add_to ( self, plottable, source_name ):
        return PlotData.add_to( self, plottable, source_name, type = 'segment' )
