#--------------------------------------------------------------------------------
#
#  Automatic legend generator for use with the Chaco plotting toolkit
#
#  Written by: David C. Morrill
#
#  Date: 02/28/2003
#
#  (c) Copyright 2003 by Enthought, Inc.
#
#  Classes defined: Legend
#
#--------------------------------------------------------------------------------

#--------------------------------------------------------------------------------
#  Imports:
#--------------------------------------------------------------------------------

from enthought.chaco.plot_base   import *
from enthought.chaco.plot_canvas import PlotGroup
from enthought.chaco.plot_frame  import PlotTitle
from enthought.chaco.plot_value  import PlotValue

#-------------------------------------------------------------------------------
#  'Legend' class:
#-------------------------------------------------------------------------------

class Legend:
    
   #----------------------------------------------------------------------------
   #  Initialize the object:
   #----------------------------------------------------------------------------
   
   def __init__ ( self, *items, **traits ):
       self._items  = items
       self._traits = traits
   
   #----------------------------------------------------------------------------
   #  Add a legend to a plot object:
   #----------------------------------------------------------------------------
   
   def add_to_plot ( self, plottable ):
       values = []
       if isinstance( plottable, PlotValue ):
           if plottable.label != '':
               values.append( plottable )
       else: 
           self.get_values( self._items or plottable, values )
       if len( values ) > 0:
           pg = PlotGroup( overlay         = True, 
                           title_alignment = 'left',
                           label           = 'Legend',
                           bg_color        = 0xE0E0E0 )
           pg.set( **self._traits )
           pg.add( PlotTitle( pg.label + ':' ) )
           plottable.add( pg )
           fill_color = values[0].fill_color
           trait = [ 'fill_color', 'line_color' ][ len( values ) == len(
                       filter( lambda x: x.fill_color == fill_color, values )) ]
           for value in values:
               pg.add( PlotTitle( value.label, 
                                  label = value.label + ' legend',
                                  legend_color = getattr( value, trait ) ) )
                                
   #----------------------------------------------------------------------------
   #  Add all PlotValues in a list to another list:
   #----------------------------------------------------------------------------
   
   def get_values ( self, items, values ):
       for item in items:
           if isinstance( item, PlotValue ):
               if item.label != '':
                   values.append( item )
           else:
               self.get_values( item, values )
