#-------------------------------------------------------------------------------
#  
#  Define a randomly distributed point data set
#  
#  Written by: David C. Morrill
#  
#  Date: 08/04/2003
#  
#  (c) Copyright 2003, Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from enthought.chaco.plot_base import *
from enthought.chaco.plot_data import PlotData
from enthought.traits.api          import Range
from scipy                     import stats

#-------------------------------------------------------------------------------
#  'Scatter' class:
#-------------------------------------------------------------------------------

class Scatter ( PlotData ):

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------
    
    points = Range( 100.0, 10000.0, value = 1000.0 )
    x_mean = Range(   0.0,    60.0, value =   30.0 )
    y_mean = Range( -10.0,    10.0, value =    0.0 )
    x_std  = Range(   1.0,    20.0, value =    5.0 )
    y_std  = Range(   0.1,     3.0, value =    1.0 )
   
    #----------------------------------------------------------------------------
    #  Return the data for a specified source: 
    #----------------------------------------------------------------------------
   
    def data ( self, source_name ):
        x = stats.norm( self.x_mean, self.x_std )
        y = stats.norm( self.y_mean, self.y_std )
        c = concatenate( ( x.rvs( size = self.points), y.rvs( size = self.points ) ) )
        r = reshape( c, ( 2, self.points ) )
        result = transpose( r )
        return result

    #----------------------------------------------------------------------------
    #  Add one or more source connections to a plottable object:
    #----------------------------------------------------------------------------

    def add_to ( self, plottable, source_name ):
        return PlotData.add_to( self, plottable, source_name, type = 'scatter' )
