#-------------------------------------------------------------------------------
#
#  Define a 'stock quote' example plot data source class
#
#  Written by: David C. Morrill
#
#  Date: 02/15/2003
#
#  (c) Copyright 2003 by Enthought, Inc.
#
#  Classes defined: StockQuote
#                   StockQuoteData
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

# Major package imports.
from urllib                      import urlopen
from time                        import time, localtime

# Enthought library imports.
from enthought.traits.api            import Str, Trait
from enthought.traits.ui.api         import Item, View

from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_data   import PlotData
from enthought.chaco.plot_value  import PlotValue

#-------------------------------------------------------------------------------
#  'StockQuote' class:
#-------------------------------------------------------------------------------
    
class StockQuote ( PlotData ):

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    traits_view = View(Item('symbol'))

    symbol = Trait('dell', auto_set = False, enter_set = True, event = 'update')
    
    _symbol = Str
    
    sources = ['close', 'open', 'high', 'low', 'volume', 'date']

    #---------------------------------------------------------------------------
    #  Get the associated stock data for the specified stock symbol:
    #---------------------------------------------------------------------------
       
    def data ( self, source_name ):
        if self.symbol != self._symbol: 
            begin     = time() - 24 * 60 * 60 
            yesterday = localtime( begin )
            before    = localtime( begin - (13 * 7 * 24 * 60 * 60) )
            url = ('http://table.finance.yahoo.com/table.csv?a=%d&b=%d&c=%d&'
                   'd=%d&e=%d&f=%d&s=%s&y=0&g=d&ignore=.csv') % ( 
                   (before[1] - 1),    before[2],     before[0],
                   (yesterday[1] - 1), yesterday[2],  yesterday[0], self.symbol)
            try:
                fh = urlopen( url )
                date, high, low, open, close, volume = [], [], [], [], [], []
                for line in map( lambda x: x[:-1], fh.readlines()[1:] ):
                    items = line.split( ',' )
                    date.append(   items[0] )
                    open.append(   float( items[1] ) )
                    high.append(   float( items[2] ) )
                    low.append(    float( items[3] ) )
                    close.append(  float( items[4] ) )
                    volume.append( float( items[5] ) )
                fh.close()
                self._symbol = self.symbol
                self._data   = { 
                    'date':   self.validate( date ),
                    'high':   self.validate( high ),
                    'low':    self.validate( low ),
                    'open':   self.validate( open ),
                    'close':  self.validate( close ),
                    'volume': self.validate( volume )
                }
            except:
                self._symbol = ''
                return [ 0.0 ] * 61
        return self._data[ source_name ]
        
    #---------------------------------------------------------------------------
    #  Validate a collection of data:
    #---------------------------------------------------------------------------
    
    def validate ( self, data ):
        data.reverse()
        data = data[:61]
        if len( data ) < 61:
            data.extend( [ data[-1] ] * (61 - len( data )) )
        return data
        
    #---------------------------------------------------------------------------
    #  Add one or more source connections to a plottable object:
    #---------------------------------------------------------------------------
    
    def add_to ( self, plottable, source_name ):
        if isinstance( plottable, PlotValue ):
            self.get_source( plottable, source_name or 'close' )
            return plottable
        if source_name is not None:
            source_names = [ source_name ]
        else:
            source_names = [ 'high', 'low', 'open', 'close' ]
            if isinstance( plottable, PlotCanvas ):
                plottable.plot_type = 'HLOC'
                plottable.index     = PlotValue( self.data( 'date' ) )
        for source_name in source_names:
            value = PlotValue()
            self.get_source( value, source_name )
            plottable.add( value )
        return plottable
