""" A plot interaction that returns the X,Y point of the user's mouseclick."""


# Enthought library imports.
from enthought.chaco.plot_interaction import PlotInteraction
from enthought.chaco.plot_value import PlotValue
from enthought.chaco.image_plot_value import AbstractImagePlotValue
from enthought.traits.api import Any, Float, Instance, Trait, Tuple


class ClickInteraction(PlotInteraction):
    """ A plot interaction that returns the X,Y point of the user's mouseclick.
    """

    # The PlotValue that was clicked on.
    clicked_value = Trait(None, Instance(PlotValue), Instance(AbstractImagePlotValue))
                          
    
    # The data point within the PlotValue that was clicked on.
    clicked_data_point = Trait(None, None, Tuple)
    
    # The threshold that constitutes a hit (in pixels).
    threshold = Float(5.0)

    # The last raw mouse event.
    event = Any
    
    
    ###########################################################################
    # Event handlers.
    ###########################################################################

    def normal_left_down(self, event):
        """ Called when the user presses the left mouse button. """

        self.event = event
        try:
            values = self.canvas.values_at(event.x, event.y, self.threshold)
        except:
            values = self.canvas.values_at(event.x, event.y)
        if len(values) > 0:
            self.clicked_value = values[0]
            self.clicked_data_point = self.clicked_value.map_xy(
                event.x,
                event.y                
            )
        else:
            self.clicked_value = None
            self.clicked_data_point = None
            
        return
    
    def normal_left_up(self, event):
        """ Called when the user releases the left mouse button. """
        
        self.clicked_value = None
        self.clicked_data_point = None
        
        return
            
#### EOF ######################################################################
