""" A plot interaction that highlights the plot value under the cursor. """


from enthought.chaco.plot_interaction import PlotInteraction
from enthought.chaco.plot_value import PlotValue
from enthought.traits.api import Instance, Trait, Tuple, Float

class ObserveInteraction(PlotInteraction):
    """ A plot interaction that highlights the plot value under the cursor. """

    # The plot value under the mouse cursor.
    observed_value = Instance(PlotValue)
    
    # The data point under the mouse cursor.
    observed_data_point = Trait(None, None, Tuple)
    
    # The threshold (in pixels) that constitutes "under the mouse cursor."
    threshold = Float(5.0)
    
    ###########################################################################
    # Event handlers.
    ###########################################################################

    def normal_mouse_move(self, event):
        """ Called when the mouse moves. """
        
        # Locate any PlotValues under the cursor.
        values = self.canvas.values_at(event.x, event.y, self.threshold)
        
        # If there are PlotValues under the cursor, then extract the point.
        if len(values) > 0:
            self.observed_value = values[0]
            self.observed_data_point = self.observed_value.map_xy(
                event.x,
                event.y
            )

        # Otherwise, clear the traits.
        else:
            self.observed_value = None
            self.observed_data_point = None
            
        return
            
#### EOF ######################################################################
