"An interaction that moves the bounds of the plot."

import sys

# Enthought library imports.
from enthought.chaco.plot_interaction import PlotInteraction
from enthought.chaco.plot_value import PlotValue
from enthought.chaco.image_plot_value import AbstractImagePlotValue
from enthought.traits.api import Any, Float, Instance, Trait, Tuple
from enthought.chaco.plot_base import TOP, BOTTOM, LEFT, RIGHT


class PanInteraction(PlotInteraction):
    "A plot interaction that returns the X,Y point of the user's mouseclick."

    # The amount to scale the mouse motion delta.
    mousespeed = Float(1.0)

    # The last raw mouse event.
    event = Any
    
    ###########################################################################
    # Event handlers.
    ###########################################################################

    def normal_right_down(self, event):
        """Called when the user presses the right mouse button."""
        self.event = event
        self.event_state = "panning"
        self.pointer = 'hand'
        self._original_xy = (event.x, event.y)
        return
    
    def panning_right_up(self, event):
        """ Called when the user releases the right mouse button. """
        self._original_xy = None
        self.event_state = "normal"
        self.pointer = 'arrow'
        return
    
    def panning_mouse_move(self, event):
        for value in self.canvas.values:
            if value.index_axis == 'x':
                x_axis_info = value._axis_info_index
                y_axis_info = value._axis_info
                x_axis = value.axis_index
                y_axis = value.axis
            else:
                x_axis_info = value._axis_info
                y_axis_info = value._axis_info_index
                x_axis = value.axis
                y_axis = value.axis_index

            if 'auto' in (x_axis.bound_low, x_axis.bound_high, y_axis.bound_low, y_axis.bound_high):
                x_axis.bound_low = x_axis_info.min
                x_axis.bound_high = x_axis_info.max
                y_axis.bound_low = y_axis_info.min
                y_axis.bound_high = y_axis_info.max

            xscale = value._plot_sxdx / (x_axis.bound_high - x_axis.bound_low)
            xdelta = -self.mousespeed * (event.x - self._original_xy[0]) / xscale
            yscale = value._plot_sydy / (y_axis.bound_high - y_axis.bound_low)
            ydelta = -self.mousespeed * (event.y - self._original_xy[1]) / yscale
            
            #If the origin is set other than bottom left, our movements are reversed
            #in coordinate space.  Here we correct for this.
            if self.canvas.plot_origin_ & TOP:
                ydelta = -ydelta
            if self.canvas.plot_origin_ & RIGHT:
                xdelta = -xdelta
                
            x_axis.bound_low += xdelta
            x_axis.bound_high += xdelta
            y_axis.bound_low += ydelta
            y_axis.bound_high += ydelta
        
            self._original_xy = (event.x, event.y)
        self.canvas.update()
    
#### EOF ######################################################################
