#-----------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  Author: Scott Swarts <swarts@enthought.com>
#
#-----------------------------------------------------------------------------

"""Interaction for selection a range of data.
"""

# Standard library imports.

# Major packages.
from enthought.util.numerix import transpose

# Enthought library imports
from enthought.traits.api import Dict
from enthought.enable.enable_traits import Pointer
from enthought.enable.drawing import DragBox
from enthought.enable.base import transparent_color
from enthought.chaco.plot_interaction import PlotInteraction

# Application specific imports.

# Local imports.

##############################################################################
# class 'RangeSelectInteraction'
##############################################################################

class RangeSelectInteraction(DragBox, PlotInteraction):
    """Interaction that allows zooming the graph to a selected box.
    """

    ##########################################################################
    # Traits
    ##########################################################################

    ### 'RangeSelectInteraction' interface ###################################

    # The selected data points  { PlotValue : [ (x, y) ] }
    selections = Dict

    ### 'DragBox' interface ##################################################
    
    # Pointer for the complete state.
    complete_pointer = Pointer('arrow')

    ### 'Box' interface ######################################################

    # Set the box color to transparent grey
    color = ( 0.501961, 0.501961, 0.501961, 0.25 )

    # Set a border size to get the box
    border_size = 0

    ##########################################################################
    # 'DragBox' interface
    ##########################################################################

    def normal_left_down(self, event):
        """
        Constrains the box to only change size on the index axis and then
        calls the superclass function.
        """
        x, y, dx, dy = self.canvas._plot_bounds
        if self.canvas.index_axis == 'x':
            self.y_bounds = y, dy
        else:
            self.x_bounds = x, dx
        super(RangeSelectInteraction, self).normal_left_down(event)


    def _complete_changed(self):
        """ Handle the box being completed. """

        all_selections = {}        
        for value in self.canvas.values:
            index = value.index.data
            data = value.data

            # map_screen() returns a 2xN array, but to iterate over the points
            # we need Nx2.
            points = transpose(value.map_screen(index,data))
            
            selection = [(index[i],data[i]) for (i,(px,py)) in enumerate(points) \
                                    if self._is_in( (px, py) )]

            if len(selection) > 0:
                all_selections[value] = selection
                
        self.selections = all_selections

    def _is_in(self, point):
        px, py = point
        x, y, dx, dy = self.bounds
        return x <= px and px-x <= dx and \
               y <= py and py-y <= dy



#### EOF ######################################################################
