""" A plot interaction that allows for selection of data points. """


# Major package imports.

# Enthought library imports.
from enthought.util.numerix import array, ravel, transpose
from enthought.chaco.plot_value import PlotValue
from enthought.chaco.plot_interaction import PlotInteraction
from enthought.enable.polygon import PolygonModel
from enthought.enable.drawing import DragPolygon
from enthought.model.api import PolygonFilter
from enthought.traits.api import Bool, Callable, Float, Instance, List, Property



class SelectPolygonFilter(PolygonFilter):

    # A Callable which takes the screen-space model and maps it to data space.
    mapper = Callable

    # The polygon model (in screen space).
    model = Instance(PolygonModel, ())

    # The vertices (in data space) of the polygon.
    vertices = Property
    
    ###########################################################################
    # Property accessors.
    ###########################################################################
    
    def _get_vertices(self):
        """ Property accessor for vertices trait. """
        if self.mapper is not None:
            return self.mapper(self.model)

        else:
            return None

class SelectFilterInteraction(DragPolygon, PlotInteraction):
    """ A plot interaction that allows for selection of data points. """

    # Should the selection disappear on complete.
    reset_on_complete = Bool(True)

    # The data selection filter.
    model = Instance(PolygonModel, ())

    # The value to perform the mapping.
    value = Instance(PlotValue)

    use_winding = Bool(False)  ### FIXME : Need to unify with polygon inside_rule

    #### Trait overrides. #####################################################

    #### Polygon interface. ####
    
    # Background color of the selection polygon - transparent blue.
    background_color = (0.0, 0.0, 1.0, 0.2)

    # The attached filters which listen on this interaction.
    filters = List

    ###########################################################################
    # Public 'SelectFilterInteraction' interface. 
    ###########################################################################

    def add_filter(self, filter):
        """ Add a filter to the interested filters. """

        filter.model = self.model
        filter.mapper = self._map_model
        self.filters.append(filter)

        return

    ###########################################################################
    # Protected 'SelectFilterInteraction' interface.
    ###########################################################################

    def _map_model(self, model):
        """ Map the model from screen space to data space. """

        if self.value is not None:
            mapped_points = [ self.value.map_xy(point[0]-self.x, point[1]+self.y) \
                              for point in model.points ]
            vertices = [point[:2] for point in mapped_points]
            return vertices
        else:
            return None

    ###########################################################################
    # Trait event handlers.
    ###########################################################################
    
    def _complete_changed(self):
        """ Handle the polygon being completed. 
        
        Here we traverse the values locating points within the drawn polygon,
        storing them in the selections dictionary.
        """

        for filter in self.filters:
            filter.updated = True
        
        if self.reset_on_complete:
            self.reset()
        
        return

    def _use_winding_changed(self):

        if self.use_winding:
            self.inside_rule = 'winding'
        else:
            self.inside_rule = 'oddeven'

#### EOF ######################################################################

