""" A plot interaction that allows for selection of data points. """


# Major package imports.
from enthought.util.scipyx         import compress, nonzero, transpose

# Enthought library imports.
from enthought.chaco.plot_interaction import PlotInteraction
from enthought.enable.drawing import DragPolygon
from enthought.kiva.agg import points_in_polygon
from enthought.traits.api import Bool, Dict


class SelectInteraction(DragPolygon, PlotInteraction):
    """ A plot interaction that allows for selection of data points. """

    # Background color of the selection polygon - transparent blue.
    background_color = (0.0, 0.0, 1.0, 0.2)

    # The selected data points  { PlotValue : [ (x, y) ] }
    selections = Dict
    selection_indexes = Dict

    # Should the selection disappear on complete.
    reset_on_complete = Bool(True)
    
    ###########################################################################
    # Trait event handlers.
    ###########################################################################
    
    def _complete_changed(self):
        """ Handle the polygon being completed. 
        
        Here we traverse the values locating points within the drawn polygon,
        storing them in the selections dictionary.
        """

        all_selections = {}     
        all_indexes  = {}
        for value in self.canvas.values:
            if value.index is None:
                continue
            index = value.index.data
            data = value.data

            # map_screen() returns a 2xN array, but to iterate over the points
            # we need Nx2.
            points = transpose(value.map_screen(index,data))
            points[:,:1] += self.x
            points[:,1:] -= self.y
            
            selected_points = points_in_polygon(points[:,:2], self.points,
                                                self.inside_rule == 'winding')
            selected_indices = compress(selected_points, index)
            selected_data = compress(data, index)
            selection = zip(selected_indices, selected_data)
            indexes = nonzero(selected_points)

            if len(selection) > 0:
                all_selections[value] = selection
                all_indexes[value] = indexes
                
        self.selections = all_selections
        self.selection_indexes = all_indexes

        if self.reset_on_complete:
            self.reset()
        
        return

#### EOF ######################################################################

