#-----------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  Author: Scott Swarts <swarts@enthought.com>
#
#-----------------------------------------------------------------------------

"""
"""

# Standard library imports.

# Major packages.

# Enthought library imports
from enthought.traits.api import *
from enthought.chaco.plot_interaction import PlotInteraction
from enthought.enable import Line

# Application specific imports.

# Local imports.

##############################################################################
# class 'SliceInteraction'
##############################################################################

class SliceInteraction(Line, PlotInteraction):
    """
    """

    ##########################################################################
    # Traits
    ##########################################################################

    ### 'Line' interface #####################################################

    line_dash = (4.0, 2.0)

    ### 'SliceInteraction' interface #########################################

    # A map from plot_value to (index_value, value) at the slice line.  This
    # will be set on completion
    values = Dict

    ##########################################################################
    # 'object' interface
    ##########################################################################

    ##########################################################################
    # 'Component' interface
    ##########################################################################

    def normal_mouse_move(self, event):
        self._move_line(event)

    def normal_left_down(self, event):
        self._move_line(event)

        # Find the values
        values = {}
        if self.canvas.index_axis == 'x':
            for value in self.canvas.values:
                if len(value.data) > 1:
                    x, y, v = value.map_xy(event.x, event.y)
                    values[value] = (x, v)
        else:
            for value in self.canvas.values:
                if len(value.data) > 1:
                    x, y, v = value.map_xy(event.x, event.y)
                    values[value] = (y, v)
        self.values = values

        self.event_state = "complete"

    ##########################################################################
    # 'SliceInteraction' interface
    ##########################################################################

    def _move_line(self, event):
        bx, by, bdx, bdy = self.canvas._plot_bounds
        if self.canvas.index_axis == 'x':
            self.points = [(event.x, by), (event.x, by+bdy)]
        else:
            self.points = [(bx, event.y), (bx+bdx, event.y)]
        self.redraw()

    
    
    



#### EOF ######################################################################
