""" Style sheet for plot values. """

from enthought.enable.enable_traits import ComponentMinSize
from enthought.enable.traits.api import RGBAColor
from enthought.kiva.traits.kiva_font_trait import KivaFont
from enthought.traits.api import HasStrictTraits, Range, Float, \
                             String, Int


from plot_frame_style import PlotFrameStyle
from plot_traits import BorderSize, BorderStyle, FillStyle, LineStyle
from plot_traits import LineWeight, Margin, Origin, OutlineWeight, PlotType
from plot_traits import RelativeSize, Symbol, Position


class PlotStyle(PlotFrameStyle):
    """ Style sheet for plot values. """

    # The color of annotations and labels on data points
    plot_annotation_color = RGBAColor('black')

    # The relative position of a plot annotation
    plot_annotation_position = Position('bottom center')

    # Number of pixels of padding between the annotation labels and the plot
    plot_annotation_padding = Float(15)

    # The font to use for the annotation text
    plot_annotation_font = KivaFont('modern')

    # The size of the annotation text
    plot_annotation_size = Int(14)

    # For bar charts, the width of a bar.
    plot_bar_width = Range(10.0, 150.0, value=67.0)

    # For bar charts, the width of a group of bars.
    plot_bin_width = Range(50.0, 100.0, value=80.0)

    # The plot background color.
    plot_bg_color = RGBAColor('white')

    # The border color.
    plot_border_color = RGBAColor('black')

    # The thickness of plot borders.
    plot_border_size = BorderSize(1)

    # The style of plot borders.
    plot_border_style = BorderStyle

    # For line type plots, the color of the line.
    plot_line_color = RGBAColor('blue')

    # For line type plots, the style of the line (e.g. dash).
    plot_line_style = LineStyle

    # For line type plots, the thickness of the line.
    plot_line_weight = LineWeight

    # For plots with markers, the fill color of the marker.
    plot_fill_color = RGBAColor('yellow')

    # For plots with markers, the fill style of the marker.
    plot_fill_style = FillStyle

    # The minimum height of a contained plot.
    plot_min_height = ComponentMinSize(128)

    # The minimum width of a contained plot.
    plot_min_width = ComponentMinSize(128)

    # The corner of the plotting area used as the axis origin.
    plot_origin = Origin

    # For plots with markers, the outline color of the marker.
    plot_outline_color = RGBAColor('black')

    # For plots with markers, the thickness of the outline.
    plot_outline_weight = OutlineWeight(1)

    # The padding inside any contained plot.
    plot_padding = Margin(4)

    # The padding inside any contained plot above and below the content.
    plot_padding_height = Margin(4)

    # The padding inside any contained plot left and right of the content.
    plot_padding_width = Margin(4)

    # For plots with markers, the size of the marker.
    plot_size = RelativeSize('medium')

    # For plots with markers, the symbol of the marker.
    plot_symbol = Symbol

    # The type of plot being drawn.
    plot_type = PlotType


#### EOF ######################################################################
