"""
PlotTitle class
"""

# Enthought library imports.
from enthought.traits.api import Delegate
from enthought.traits.ui.api import View

# Local imports.
from plot_label import PlotLabel
from plot_views import border_group, legend_group, margin_group, misc_group

class PlotTitle ( PlotLabel ):
    """
    A specialized subclass of PlotLabel for representing the titles of plots.
    """

    # Delegation prefix.
    __prefix__ = 'plot_title_'
    
    # The style sheet for this plot title.  Delegate up the container hierarchy
    # by default.
    style = Delegate('container', '*')

    # The location of this plot title.
    position = Delegate('style')

    traits_view = View(
        [ [ 'text{Label}$', '_', 'color', ' ', 'font', '_',
            'alignment', ' ', 'rotate{Angle}', 
            '|[Text]@' ],
          [ 'bg_color{Color}', '_', 'position', ' ', 'visible',
            '|[Title]@' ],
          '|{Main}' ],
        border_group,
        margin_group,
        legend_group,
        misc_group
    )

    def remove ( self ):
        "Remove this title from its container."
        self.container.remove( self )
        return
    
# EOF
