
from renderer import Renderer

class AnnotationRenderer(Renderer):
    """
    Base class for annotation renderers for plotvalues.  This class doesn't
    actually render annotations, but instead defines the interface and 
    methods that concrete subclasses need to implement.
    """

    supported_annotation_types = []

    # The only function subclasses have to implement.  The rest are stubs
    # so that the Renderer base class doesn't barf.
    def _do_draw(self, gc, value, ox, oy):
        raise NotImplementedError
    
    # This implementation should work for most subclasses as long as they
    # properly define 'supported_annotation_types'
    def _is_valid(self, values):
        bad_values = [v for v in values if v.annotation_type \
                            not in self.supported_annotation_types]
        if bad_values:
            return False
        else:
            return True

    def _layout(self, values, x, y, dx, dy, sx, sy):
        pass
    
    def _max_values(self):
        # returning 0 means we support unlimited number of plotvalues
        return 0
    
    def _map_xy(self, value, x, y):
        return None
    
    def _hittest(self, values, x, y, threshold=5.0):
        return []
    
# EOF
