#-------------------------------------------------------------------------------
#
#  Define the 'filled' line plot base class
#
#  Written by: David C. Morrill
#
#  Date: 12/04/2002
#
#  (c) Copyright 2002 by Enthought, Inc.
#
#  Classes defined: FillRendererBase
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from enthought.chaco.fill           import fill_areas
from enthought.chaco.plot_container import TOP, LEFT
from line                           import LineRenderer

#-------------------------------------------------------------------------------
#  'FillRendererBase' class:
#
#  This is the concrete rendering base class for filled line plots.
#-------------------------------------------------------------------------------

class FillRendererBase ( LineRenderer ):
    
    #---------------------------------------------------------------------------
    #  Return the 'fill type' to use:
    #     -1: Less than
    #      0: Not equal to
    #     +1: Greater than
    #---------------------------------------------------------------------------
    
    def _plot_type ( self ):
        return 0
     
    #---------------------------------------------------------------------------
    #  Return the maximum number of values the renderer can process:
    #---------------------------------------------------------------------------
    
    def _max_values ( self ):
        return 2
    
    #---------------------------------------------------------------------------
    #  Determine if the renderer can correctly plot a set of values:
    #---------------------------------------------------------------------------
    
    def _is_valid ( self, values ):
        return (len( values ) == 2)
     
    #---------------------------------------------------------------------------
    #  Layout a list of PlotValues:  
    #---------------------------------------------------------------------------
    
    def _layout ( self, values, x, y, dx, dy, sx, sy ):
        LineRenderer._layout( self, values, x, y, dx, dy, sx, sy )
 
        # Lay the plots out:
        plot_type = self._plot_type()
        value0    = values[0]
        value1    = values[1]
        origin    = value0.plot_origin_
        left      = origin & LEFT
        top       = origin & TOP
        larger    = (plot_type >= 0)
        smaller   = (plot_type <= 0)
        value0._fill_info = fill_info = []   ### EXPERIMENTAL ###
        flipxy  = 1 - 2 * (value0.index_axis == 'y')
        if flipxy == -1:
            left, top = top, left
        flipx = 1 - 2 * (left == 0)
        if top == 0:
            larger, smaller = smaller, larger
        xy0 = value0._xy[ ::flipx, ::flipxy ]
        xy1 = value1._xy[ ::flipx, ::flipxy ]
        if ((xy0[ 0][0] == xy1[ 0][0]) and
            (xy0[-1][0] == xy1[-1][0])):
            if larger:
                fill_info.append( ( value0,
                    map( lambda x, y = flipx, z = flipxy:
                         x[ ::y, ::z ],
                         fill_areas( xy0, xy1 ) ) ) )
            if smaller:
                fill_info.append( ( value1,
                    map( lambda x, y = flipx, z = flipxy:
                         x[ ::y, ::z ],
                         fill_areas( xy1, xy0 ) ) ) )
    
    #---------------------------------------------------------------------------
    #  Draw a list of PlotValues:
    #
    #  This draws the 'background' part of the plot (i.e. the part that should
    #  underlie the foreground part of itself and all of its siblings in the 
    #  same plot canvas).
    #---------------------------------------------------------------------------
    
    def _do_draw_canvas ( self, gc, values, ox, oy ):
        value0 = values[0]
        for value, fill_regions in value0._fill_info:
            #todo: chaco currently does not support fill style so it is
            #      being left out here:
            gc.set_fill_color( value.fill_color_ )
            gc.begin_path()
            for polygon in fill_regions:
                if ox or oy:
                    gc.lines( polygon - ( ox, oy ) )
                else:
                    gc.lines( polygon )
                gc.close_path()
            gc.fill_path()
