
from enthought.util.numerix  import UInt8, take, array, transpose


from enthought.chaco.plot_base import *
from enthought.chaco.image_plot_value import ImagePlotValue
from enthought.traits.api import Bool
from renderer                  import Renderer

from enthought.kiva.agg import GraphicsContextArray

class ImageRenderer ( Renderer ):

    draw_as_background = Bool(True)

    #------------------------------------------------------------------------
    # Renderer interface public methods
    #------------------------------------------------------------------------

    def _get_bounds_value(self, values):
        # values must be ImagePlotValue instances
        minval = min([min(val.data.y_index) for val in values if val.index_axis == 'x'] +
                 [min(val.data.x_index) for val in values if val.index_axis == 'y'])
        maxval = max([max(val.data.y_index) for val in values if val.index_axis == 'x'] +
                 [max(val.data.x_index) for val in values if val.index_axis == 'y'])
        return (minval, maxval)

    def _get_bounds_index(self, values):
        # values must be ImagePlotValue instances
        minval = min([min(val.data.x_index) for val in values if val.index_axis == 'x'] +
                 [min(val.data.y_index) for val in values if val.index_axis == 'y'])
        maxval = max([max(val.data.x_index) for val in values if val.index_axis == 'x'] +
                 [max(val.data.y_index) for val in values if val.index_axis == 'y'])
        return (minval, maxval)

    def _layout(self, values, x, y, dx, dy, sx, sy):
        for value in values:
            # Cache the bounds of the plot area and the scaling factors:
            translate = self._bounds( value, x, y, dx, dy, sx, sy )
            if ((translate is None) or (translate == ( 0, 0 )) or
                (value._xy is None)):
 
                # For ImagePlotValue, _xy stores only two screen-space points:
                # the lower left coordinate, and the upper right coordinate.
                # These are then used by _draw_canvas.
                bounds_x = take(value.data.x_index, [0,-1])
                bounds_y = take(value.data.y_index, [0,-1])
                
                value._xy = value.map_screen(bounds_x, bounds_y, xy = True)
                
                # We have to check for linear data; basically, if either
                # dimension of the data set is 1, then it has zero width in
                # screen space, and we need to offset this by giving at least
                # 1 pixel width in screen space.
                if bounds_x[0] == bounds_x[1]:
                    ll, ur = value._xy
                    ur[0] = ll[0] + 1
                if bounds_y[0] == bounds_y[1]:
                    ll, ur = value._xy
                    ur[1] = ll[1] + 1
            else:
                value._xy += translate

    #------------------------------------------------------------------------
    # Renderer interface private methods
    #------------------------------------------------------------------------
    
    def _is_valid(self, values):
        return True

    def _do_draw_canvas ( self, gc, values, ox, oy ):
        # This draws the background part of the plot.
        for image_pv in values:
            image_pv._render(gc, ox, oy)

    #------------------------------------------------------------------------
    # private helper methods
    #------------------------------------------------------------------------

    def _map_xy ( self, value, x, y ):
        """
        Map a specified (x,y) point in window space to the corresponding point
        (x',y',v') in a PlotValue's data space (v' is the PlotValue intercept
        of a line orthogonal to the index axis that passes through the point
        (x',y'):
        """

        # Modify x and y to be their fractional offset into the plot area:
        x = (float( x - value._plot_bounds[0] + value._plot_ox ) / 
            (value._plot_sxdx - 1))
        y = (float( y - value._plot_bounds[1] + value._plot_oy ) / 
            (value._plot_sydy - 1))

        # Get the x and y axis information:
        x_axis = value._axis_info_index
        y_axis = value._axis_info

        # If the index axis is vertical, flip-flop the values:
        use_y = (not x_axis.horizontal)
        if use_y:
            x_axis, y_axis = y_axis, x_axis
        
        # Scale and translate the x data:
        dxmin = x_axis.tick_min
        dymin = y_axis.tick_min

        # Get the plot origin:
        origin = value.origin_
##         origin = value.plot_origin_

        # Invert the x and y values if necessary:
        if (origin & LEFT) == 0:
            x = 1.0 - x
        if origin & TOP:
            y = 1.0 - y

        # Calculate (x',y'):
        xp = dxmin + x * (x_axis.tick_max - dxmin)
        if x_axis.log_scale:
            xp = pow(10, xp)

        yp = dymin + y * (y_axis.tick_max - dymin)
        if y_axis.log_scale:
            yp = pow(10, yp)

        # vp doesn't make sense for images
        vp = None
            
        # Return the result:
        return ( xp, yp, vp )
        
    def _hittest ( self, values, x, y, threshold = 5.0 ):
        return []
    
    def _distance_to(self, value, x, y):
        return 0.0
    
    def _get_xydata(self, value, x, y):
        return value._xy


