#-------------------------------------------------------------------------------
#
#  Define the 'invalid' plot rendering class for indicating that the user
#  selected renderer cannot legally plot the specified data
#
#  Written by: David C. Morrill
#
#  Date: 12/05/2002
#
#  (c) Copyright 2002 by Enthought, Inc.
#
#  Classes defined: InvalidRenderer
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from scatter import ScatterRenderer

#-------------------------------------------------------------------------------
#  'InvalidRenderer' class:
#
#  This is the concrete rendering class for invalid combinations of renderers
#  and data sets.
#-------------------------------------------------------------------------------

class InvalidRenderer ( ScatterRenderer ):
   
    #---------------------------------------------------------------------------
    #  Draw the plotting area:
    #---------------------------------------------------------------------------
 
    def _do_draw ( self, gc, value, ox, oy ):
        
        index1, index2, xy = self._get_plot_data( value, ox, oy )
        n                  = len( xy )
 
        # Set up to draw solid red lines of width 2:
        gc.set_stroke_color( ( 1.0, 0.0, 0.0, 1.0 ) )
        gc.set_line_width( 2 )
        gc.set_line_dash( None ) 
 
        gc.begin_path()
        for low, high in value._xy_ranges:
            high -= index1
            if high > 0:
                low = max( 0, low - index1 )
                if low >= n:
                    break
                high      = min( n, high )
                size      = 3;
                for i in range( low, high - 1 ):
                    x, y = xy[i]
                    # The following draws an x-eyed guy with his tongue out:
                    gc.move_to( x - 6, y + 6 )
                    gc.line_to( x - 2, y + 2 )
                    gc.move_to( x - 6, y + 2 )
                    gc.line_to( x - 2, y + 6 )
                    gc.move_to( x + 6, y + 6 )
                    gc.line_to( x + 2, y + 2 )
                    gc.move_to( x + 6, y + 2 )
                    gc.line_to( x + 2, y + 6 )
                    gc.move_to( x - 7, y - 2 )
                    gc.line_to( x + 7, y - 2 )
                    gc.move_to( x - 3, y - 2 )
                    gc.line_to( x - 1, y - 7 )
                    gc.line_to( x + 5, y - 7 )
                    gc.line_to( x + 3, y - 2 )
        gc.close_path()
        gc.stroke_path()
