#-------------------------------------------------------------------------------
#
#  Define the range bar plot rendering class
#
#  Written by: David C. Morrill
#
#  Date: 12/13/2002
#
#  (c) Copyright 2002 by Enthought, Inc.
#
#  Classes defined: RangeBarRenderer
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from enthought.chaco.plot_base import *
from range                     import RangeRenderer

#-------------------------------------------------------------------------------
#  'RangeBarRenderer' class:
#-------------------------------------------------------------------------------

class RangeBarRenderer ( RangeRenderer ):
   
    #---------------------------------------------------------------------------
    #  Draw a list of PlotValues:
    #
    #  This draws the 'background' part of the plot (i.e. the part that should
    #  underlie the foreground part of itself and all of its siblings in the 
    # same plot canvas).
    #---------------------------------------------------------------------------
    
    def _do_draw_canvas ( self, gc, values, ox, oy ):
        value0, value1  = values
        value0._plot_ox = value1._plot_ox = ox
        value0._plot_oy = value1._plot_oy = oy
        xy0       = value0._xyr0
        xy1       = value1._xyr1
        choose    = ((value0.data - value1.data) < 0.0)
        colors    = ( value0.fill_color_, value1.fill_color_ )
        cur_color = colors[0]
        gc.set_line_dash( None )
        gc.set_line_width( value0.outline_weight )
        gc.set_stroke_color( value0.outline_color_ )
        gc.set_fill_color( cur_color )
        gc.begin_path()
        for i in range( len( xy0 ) ):
            x0, y0 = xy0[i]
            x1, y1 = xy1[i]
            color  = colors[ choose[i] ]
            if color != cur_color:
               gc.draw_path()
               gc.set_fill_color( color )
               cur_color = color
            gc.move_to( x0 - ox, y0 - oy )
            gc.line_to( x1 - ox, y0 - oy )
            gc.line_to( x1 - ox, y1 - oy )
            gc.line_to( x0 - ox, y1 - oy )
            gc.close_path()
        gc.draw_path()
    
    #---------------------------------------------------------------------------
    #  Map a specified (x,y) point in window space to the corresponding point
    #  (x',y',v') in a PlotValue's data space (v' is the PlotValue intercept
    #  of a line orthogonal to the index axis that passes through the point
    #  (x',y'):
    #
    #  NOTE: This routine currently does not take into account 'log scaling':
    #---------------------------------------------------------------------------
 
    def _map_xy ( self, value, x, y ):
        raise NotImplementedError    
        
    #---------------------------------------------------------------------------
    #  Determine if any PlotValue's bar representation intersects a
    #  specified point:
    #---------------------------------------------------------------------------
 
    def _hittest ( self, values, x, y, threshold ):
        if self._hittest_rect( x, y, values[0]._xyr0, values[1]._xyr1, threshold ):
            return values 
        return []
