
from enthought.kiva import Font
from annotation import AnnotationRenderer


class StringAnnotationRenderer(AnnotationRenderer):
    """
    Concrete subclass of AnnotationRenderer that renders annotations of type
    "string" (which are just strings of text).
    """

    supported_annotation_types = ['string']

    # The position_offset_map defines where, relative to a datapoint's screen
    # x,y position, the annotation should go.  The characteristic distance
    # is the height of the text of the annotation.  For positions on a left,
    # top, right, or bottom edge, the annotation is placed 1 characteristic
    # distance away.  For positions in the corners, the label is placed
    # 1 unit to the side and 1 unit above or below.
    #
    # In the map, the tuple of expressions calculates the origin of the
    # text box, with x,y being the position of the datapoint, w,h being
    # the dimensions of the text box, and 'u' being a unit of characteristic
    # space.  (u can be set by the user to adjust the appearance of the labels.)
    # TODO: Having to eval() the expressions in the core drawing loop is very
    #       wasteful; a simple optimization would be to store the compiled
    #       bytecode for the expression using compile().
    position_offset_map = { 'top left' : ('x-w-u', 'y+u'),
                            'top center': ('x-w/2.0', 'y+u'),
                            'top right': ('x+u', 'y+u'),
                            'bottom left': ('x-w-h', 'y-u'),
                            'bottom center': ('x-w/2.0', 'y-u'),
                            'bottom right': ('x+u', 'y-u'),
                            'left center': ('x-w-u', 'y'),
                            'right center': ('x+u', 'y') }
    position_offset_map['center left'] = position_offset_map['left center']
    position_offset_map['center right'] = position_offset_map['right center']


    def _do_draw(self, gc, value, ox, oy):
        index1, index2, xy = self._get_plot_data(value, ox, oy)

        annotated_indices = [i for i in range(index1, index2+1) \
                                   if value.annotations.has_key(i)]

        xfunc, yfunc = self.position_offset_map[value.annotation_position]
        u = value.annotation_padding

        gc.save_state()

        gc.set_stroke_color(value.annotation_color_)
        gc.set_fill_color(value.annotation_color_)

        fn = value.annotation_font
        fn.size = value.annotation_size
        gc.set_font(fn)

        for i in annotated_indices:
            x, y = xy[i-index1]
            x_offset,y_offset,w,h = gc.get_text_extent(value.annotations[i])
            x = int(eval(xfunc) + x_offset) + 0.5
            y = int(eval(yfunc) + y_offset) + 0.5
            # FIXME: For some reason the Mac GC doesn't work for show_text_at_point.
            #gc.show_text_at_point(value.annotations[i], x, y)
            gc.set_text_position(x, y)
            gc.show_text(value.annotations[i])

        gc.restore_state()
        return

#EOF
