import unittest
import pdb

from enthought.util.numerix import *
from enthought.util.testingx import *

from enthought.chaco.colormap import LinearColormap

black_color4 = (0.0, 0.0, 0.0, 1.0)
black_color3 = black_color4[:3]
white_color4 = (1.0, 1.0, 1.0, 1.0)
white_color3 = white_color4[:3]


def image_almost_equal(ary1, ary2):
    # checks to see if two image arrays are almost equal
    allowed_diff = 0.001
    if ary1.shape != ary2.shape:
        return False
    return abs(ravel(ary1) - ravel(ary2)) < allowed_diff

def grayscale_colormap(**kw):
    if not kw.has_key('color_depth'):
        kw['color_depth'] = 'rgba'
    cm = LinearColormap(**kw)
    cm.min_color = array(black_color4)
    cm.max_color = array(white_color4)
    return cm

def fill_image(shape, color):
    tmp = list(color) * sum(shape)
    output = resize(array(tmp), shape+(len(color),))
    return output
    

class LinearColormapTestCase(unittest.TestCase):
    def check_zeros(self):
        # check the case when the input data is all zeros
        x = zeros((3,3))
        x_img = fill_image(x.shape, color=black_color4)
        cm = grayscale_colormap()
        cm.min_value = 0.0
        cm.max_value = 1.0
        assert_equal(cm.map_array(x), x_img)
    
    def check_ones(self):
        x = ones((3,3))
        x_img = fill_image(x.shape, color=white_color4)
        cm = grayscale_colormap()
        cm.min_value = 0.0
        cm.max_value = 1.0
        assert_equal(cm.map_array(x), x_img)

    def check_four_tones(self):
        cm = grayscale_colormap(min_value = 0.0, max_value = 1.0)
        cm.steps = 4
        x = array([0.25, 0.5, 0.75])
        x_img = array([ [0.0, 0.0, 0.0, 1.0],
                        [0.33333, 0.33333, 0.33333, 1.0],
                        [0.6666, 0.6666, 0.6666, 1.0] ])
        assert_equal(cm.map_array(x), x_img)

    def check_event_handlers(self):
        cm = grayscale_colormap()
        cm.min_value = 0.0
        cm.max_value = 1.0
        cm.color_depth = 'rgb'
        assert_equal( cm._min_color , array([0.0, 0.0, 0.0]))
        assert_equal( cm._max_color, array([1.0, 1.0, 1.0]))
        cm.color_depth = 'rgba'
        assert_equal( cm._min_color, array([0.0, 0.0, 0.0, 1.0]))
        assert_equal( cm._max_color, array([1.0, 1.0, 1.0, 1.0]))

def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(LinearColormapTestCase,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite(level)
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()

#EOF
