import unittest

from enthought.util.numerix import allclose, array, ravel

from enthought.chaco.linear_segmented_colormap import LinearSegmentedColormap


class LinearSegmentedColormapTestCase(unittest.TestCase):
    
    def setUp(self):
        """ Set up called before each test case. """

        _gray_data =  {'red':   ((0., 0, 0), (1., 1, 1)),
                       'green': ((0., 0, 0), (1., 1, 1)),
                       'blue':  ((0., 0, 0), (1., 1, 1))}      
                
        self.colormap = LinearSegmentedColormap.from_segment_map(_gray_data)

    def test_simple_map(self):

        a = array([0.0, 0.5, 1.0])
        b = self.colormap.map_array(a)

        expected = array([0.0, 0.5, 1.0])

        close = allclose(ravel(b[:,:1]), expected, atol=0.02)
        self.assert_(close,
            "Simple map failed.  Expected %s.  Got %s" % (expected, b[:,:1]))

        return

    def test_change_min_max(self):
        """ Test that changing min_value and max_value does not break map. """

        # Perform a dummy mapping.
        a = array([0.0, 0.5, 1.0])
        b = self.colormap.map_array(a)

        # Update the min_value.
        self.colormap.min_value = -1.0

        # Test that the map still works.
        a = array([-1.0, 0.0, 1.0])
        b = self.colormap.map_array(a)
        expected = array([0.0, 0.5, 1.0])

        close = allclose(ravel(b[:,:1]), expected, atol=0.02)
        self.assert_(close,
            "Changing min value broke map.  Expected %s.  Got %s" % (expected, b[:,:1]))

        # Update the max_value.
        self.colormap.max_value = 0.0
        # Test that the map still works.
        a = array([-1.0, -0.5, 0.0])
        b = self.colormap.map_array(a)
        expected = array([0.0, 0.5, 1.0])

        close = allclose(ravel(b[:,:1]), expected, atol=0.02)
        self.assert_(close,
            "Changing min value broke map.  Expected %s.  Got %s" % (expected, b[:,:1]))
        
        
        return

    def test_array_factory(self):
        """ Test that the array factory creates valid LSCM. """

        colors = array([[0.0,0.0,0.0], [1.0,1.0,1.0]])
        cm = LinearSegmentedColormap.from_palette_array(colors)

        a = array([0.0, 0.5, 1.0])
        b = self.colormap.map_array(a)

        expected = array([0.0, 0.5, 1.0])

        close = allclose(ravel(b[:,:1]), expected, atol=0.02)
        self.assert_(close,
            "Array facotry failed.  Expected %s.  Got %s" % (expected, b[:,:1]))

        return
        

##     def test_no_interpolation(self):
##         grayscale_colors = array([[0.0,0.0,0.0,1.0], [1.0, 1.0, 1.0, 1.0]])
##         grayscale_bins = array([0.0, 1.0])
##         grayscale_steps = array([1])
        
##         colormap = LinearSegmentedColormap(
##             grayscale_colors, grayscale_bins, grayscale_steps
##         )

##         a = array([0.0, 0.25, 0.75, 1.0])
##         b = colormap.map_array(a)
##         result = ravel(b[:,:1])
##         expected = array([0.0, 0.0, 1.0, 1.0])

##         close = allclose(result, expected, atol=0.02)
##         self.assert_(close,
##             "Map with no interpolation broken.  Expected %s.  Got %s" % (expected, result))

##     def test_value_bands(self):
        
##         grayscale_colors = array([[0.0,0.0,0.0,1.0], [1.0, 1.0, 1.0, 1.0]])
##         grayscale_bins = array([0.0, 1.0])
##         grayscale_steps = array([1])
        
##         colormap = LinearSegmentedColormap(
##             grayscale_colors, grayscale_bins, grayscale_steps
##         )

##         colormap._recalculate()

##         print '**************', colormap._color_bands, colormap._value_bands


def test():
    runner = unittest.TextTestRunner()
    runner.run(unittest.makeSuite(LinearSegmentedColormapTestCase, 'test_'))
    return runner

if __name__ == "__main__":
    test()


#### EOF ######################################################################
