""" Helper class to layout and draw titles. """

from plot_container import BOTTOM, LEFT, TOP

class TitleInfo:
    """ Helper class to layout and draw titles. """
    
    #---------------------------------------------------------------------------
    #  Initialize the object:
    #---------------------------------------------------------------------------
  
    def __init__ ( self, gc, titles = None ):
        self.dx = self.ldx = self.rdx = 0
        self.dy = self.tdy = self.bdy = 0
        self.max_dx = self.max_dy = 0
        self.titles = []
        if titles is not None:
            self.add_titles( gc, titles )
 
    #---------------------------------------------------------------------------
    #  Add a group of titles:
    #---------------------------------------------------------------------------
  
    def add_titles ( self, gc, titles ):
        self.titles += titles
        for title_item in titles:
            self.min_size( gc, title_item )
 
    #---------------------------------------------------------------------------
    #  Return the list of titles that contain the specified (x,y) point:
    #---------------------------------------------------------------------------
 
    def objects_at ( self, x, y ):
        for title_item in self.titles:
            if title_item.item._isin( x, y ):
                return [ title_item.item ]
        return []
       
    #---------------------------------------------------------------------------
    #  Add the minimum size information for a title:
    #---------------------------------------------------------------------------
        
    def min_size ( self, gc, title_item ):
        title  = title_item.item
        dx, dy = title._min_size( gc )
        title_item.dx = dx
        title_item.dy = dy
        position      = title.position_
        if position & (TOP | BOTTOM):
            self.max_dx = max( self.max_dx, dx )
            self.dy    += dy
            if position & TOP:
                self.tdy += dy
            else:
                self.bdy += dy 
        else:
            self.dx    += dx
            self.max_dy = max( self.max_dy, dy )
            if position & LEFT:
                self.ldx += dx
            else:
                self.rdx += dx
           
    #-------------------------------------------------------------------------
    #  Lay out the accumulated titles:
    #--------------------------------------------------------------------------
 
    def _layout ( self, px, py, pdx, pdy, ltx, tty, rtx, bty ):
        for title_item in self.titles:
            title    = title_item.item
            position = title.position_
            if position & TOP:
                tty -= title_item.dy
                title._layout( px, tty, pdx, title_item.dy )
            elif position & BOTTOM:
                title._layout( px, bty, pdx, title_item.dy )
                bty += title_item.dy
            elif position & LEFT:
                title._layout( ltx, py, title_item.dx, pdy )
                ltx += title_item.dx
            else:
                rtx -= title_item.dx
                title._layout( rtx, py, title_item.dx, pdy )

    #--------------------------------------------------------------------------
    #  Draw all of the titles:
    #--------------------------------------------------------------------------
 
    def _draw ( self, gc ):
        for title_item in self.titles:
            title_item.item._draw( gc )

#### EOF ######################################################################
