#-------------------------------------------------------------------------------
#
#  Define the plotting and Tkinter specific trait sheet editors
#
#  Written by: David C. Morrill
#
#  Date: 10/21/2002
#
#  (c) Copyright 2002 by Enthought, Inc.
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import Tkinter as tk
import tkFont

from enthought.traits.tktrait_sheet import *
from enthought.kiva.fonttools       import Font
from plot_base                      import *

#-------------------------------------------------------------------------------
#  Module initialization:
#-------------------------------------------------------------------------------

trait_editors( __name__ ) 

#-------------------------------------------------------------------------------
#  'PlotTraitSheetHandler' class:
#-------------------------------------------------------------------------------

class PlotTraitSheetHandler ( TraitSheetHandler ):
    
   #----------------------------------------------------------------------------
   #  Set the initial position of a trait sheet:
   #----------------------------------------------------------------------------
   
    def position ( self, trait_sheet, object ):
        return False
#       if object._frame_x is None:
#          return False
    
#       window   = object.window
#       dx       = object._frame_dx
#       dy       = object._frame_dy
#       x, y     = window.ClientToScreenXY(
#                     object._frame_x, window._flip_y( object._frame_y + dy ) )
#       sx, sy   = window.GetViewStart()
#       x       -= sx * window.scroll_incr
#       y       -= sy * window.scroll_incr
#       if object._plot_ox is not None:
#           x -= object._plot_ox
#           y -= object._plot_oy
#       sdx      = wx.wxSystemSettings_GetSystemMetric( wx.wxSYS_SCREEN_X )
#       sdy      = wx.wxSystemSettings_GetSystemMetric( wx.wxSYS_SCREEN_Y )
#       wdx, wdy = trait_sheet.GetSizeTuple()
#       px       = x - wdx - 10
#       if px < 0:
#           px2   = x + dx + 10
#           delta = sdx - (px2 + wdx)
#           if delta >= 0:
#               px = px2
#           elif px > delta:
#               px = 0
#           else:
#               px = sdx - wdx
#       py = max( 0, min( y + (dy - wdy) / 2, sdy - wdy ) )
#       trait_sheet.SetPosition( wx.wxPoint( px, py ) )
#       return True
       
    #---------------------------------------------------------------------------
    #  Notification that a trait sheet has been closed:
    #---------------------------------------------------------------------------
    
    def closed ( self, trait_sheet, object ):
        object.host._trait_sheet_for()
    
    #---------------------------------------------------------------------------
    #  Notification that a trait sheet has modified a trait of its
    #  associated object: 
    #---------------------------------------------------------------------------
    
    def changed ( self, object, trait_name, new_value, old_value, is_set ):
        object.trait_changed( trait_name, old_value, new_value )
                       
# Create the trait sheet handler:                       
trait_sheet_handler = PlotTraitSheetHandler()                       

#-------------------------------------------------------------------------------
#  'TraitEditorColor' class:
#-------------------------------------------------------------------------------

tkTraitEditorColor = TraitEditorColor
 
class TraitEditorColor ( tkTraitEditorColor ):

    #---------------------------------------------------------------------------
    #  Return the text representation of the 'trait' trait of 'object':
    #---------------------------------------------------------------------------
 
    def str ( self, object, trait_name ):
        value = getattr( object, trait_name )
        if type( value ) in basic_sequence_types:
            for name, color in standard_colors.items():
                if (abs( color[0] - value[0] ) +
                    abs( color[1] - value[1] ) +
                    abs( color[2] - value[2] ) +
                    abs( color[3] - value[3] )) < .01:
                    return name  
            return 'R:%.2g\nG:%.2g\nB:%.2g' % value[:3]
        return str( value )
 
    #---------------------------------------------------------------------------
    #  Get the Tkinter color equivalent of the object trait:
    #---------------------------------------------------------------------------
    
    def to_tk_color ( self, object, trait_name ):
        cur_color = getattr( object, trait_name + '_' )
        if cur_color is None:
            return '#FFFFFF'  #  = White
        return '#%02X%02X%02X' % ( int( cur_color[0] * 255 ),
                                   int( cur_color[1] * 255 ),
                                   int( cur_color[2] * 255 ) )
 
    #---------------------------------------------------------------------------
    #  Get the application equivalent of a Tkinter value:
    #---------------------------------------------------------------------------
    
    def from_tk_color ( self, color ):
        color = eval( '0x' + color[1:] )
        return ( ((color >> 16) & 0xFF) / 255.0,
                 ((color >>  8) & 0xFF) / 255.0,
                 (color & 0xFF) / 255.0, 
                 1.0 )
                
#-------------------------------------------------------------------------------
#  'TraitEditorFont' class:
#-------------------------------------------------------------------------------

tkTraitEditorFont = TraitEditorFont

class TraitEditorFont ( tkTraitEditorFont ):
 
    #---------------------------------------------------------------------------
    #  Return the text representation of the 'trait' trait of 'object':
    #---------------------------------------------------------------------------
 
    def str ( self, object, trait_name ):
        font      = getattr( object, trait_name )
        weight    = { BOLD:   ' Bold'   }.get( font.weight, '' )
        style     = { ITALIC: ' Italic' }.get( font.style,  '' )
        underline = [ '', ' Underline' ][ font.underline != 0 ]
        return '%s point %s%s%s%s' % (
               font.size, font.face_name, style, weight, underline )
 
    #---------------------------------------------------------------------------
    #  Return a Font object corresponding to a specified object's font trait:
    #---------------------------------------------------------------------------
    
    def to_tk_font ( self, object, trait_name ):
        font   = getattr( object, trait_name )
        weight = ( 'normal', 'bold'  )[ font.weight == BOLD ]
        slant  = ( 'roman', 'italic' )[ font.style  == ITALIC ]
        return tkFont.Font( family    = font.face_name,
                            size      = font.size,
                            weight    = weight,
                            slant     = slant,
                            underline = (font.underline != 0) )
 
    #---------------------------------------------------------------------------
    #  Get the application equivalent of a Tkinter value:
    #---------------------------------------------------------------------------
    
    def from_tk_font ( self, font ):
        return Font( font.cget( 'size' ), 
                     SWISS, 
                     ( NORMAL, BOLD   )[ font.cget( 'weight' ) == 'bold' ],
                     ( NORMAL, ITALIC )[ font.cget( 'slant' )  == 'italic' ],
                     int( font.cget( 'underline' ) ),
                     font.cget( 'family' ) )
                
