#-------------------------------------------------------------------------------
#
#  Define an Enable-backed window surrogate for containing plot components.
#
#  Written by: David C. Morrill
#
#  Date: 11/11/2003
#
#  (c) Copyright 2003 by Enthought, Inc.
#
#  Classes defined: WindowSurrogate
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from plot_base          import UPDATE, REDRAW
from plot_traits        import HORIZONTAL

#------------------------------------------------------------------------------
#  'WindowSurrogate' class:
#------------------------------------------------------------------------------

class WindowSurrogate:

    # Screen scroll increment amount:
    scroll_incr = 64

    # Width/Height of standard scrollbars:
    scrollbar_dx = scrollbar_dy = 17

    #--------------------------------------------------------------------------
    #  Initialize the object:  
    #--------------------------------------------------------------------------    
                
    def __init__ ( self, owner ):
        self.owner        = owner
        self.update_flags = UPDATE
        self._panels      = []
        
    #--------------------------------------------------------------------------
    #  Handle an object being added to an object in the window:
    #--------------------------------------------------------------------------
   
    def _add_item ( self, container, object ):
        pass

    #--------------------------------------------------------------------------
    #  Handle an object being removed from an object in the window:
    #--------------------------------------------------------------------------
   
    def _remove_item ( self, container, object ):
        pass

    #--------------------------------------------------------------------------
    #  Update the contents of the window:
    #--------------------------------------------------------------------------
   
    def update ( self, update_flags = UPDATE ):
        self.update_flags |= update_flags
        self.owner.redraw()
   
    #--------------------------------------------------------------------------
    #  Set up a scroll bar for a specified 'plottable' object:
    #--------------------------------------------------------------------------
 
    def _scrollbar_for ( self, plottable, style, scale, size, x, y, dx, dy ):
        range = int( size * scale )
        sb    = self.owner._scrollbar_for( plottable, style )
        if sb._plottable is None:
            self.init_scrollbar( sb )
            sb._plottable = plottable
            position      = 0
            if style != HORIZONTAL:
                position = range - size
        else:
            position = self.positionof_scrollbar( sb, range, size )
           
        if style == HORIZONTAL:
            if plottable._hsb_pos is not None:
                position, plottable._hsb_pos = plottable._hsb_pos, None
        elif plottable._vsb_pos is not None:
            position, plottable._vsb_pos = plottable._vsb_pos, None
               
        sb.bounds = ( x, y, dx, dy )
        self.set_scrollbar( sb, position, range, size )
        sb._used = True
        if style == HORIZONTAL:
            plottable._plot_ox = position
        else:
            plottable._plot_oy = position
 
    #--------------------------------------------------------------------------
    #  Initialize a user provided scrollbar for a specified 'plottable' object:
    #--------------------------------------------------------------------------
 
    def _init_scrollbar ( self, plottable, style, scale, size, sb ):
        range = int( size * scale )
        if not hasattr( sb, '_plottable' ):
            self.init_scrollbar( sb )
            sb._plottable = plottable
            position      = 0
            if style != HORIZONTAL:
                position = range - size
        else:
            position = self.positionof_scrollbar( sb, range, size )
        self.set_scrollbar( sb, position, range, size )
        if style == HORIZONTAL:
            plottable._plot_ox = position
        else:
            plottable._plot_oy = range - size - position

    #--------------------------------------------------------------------------
    #  Initialize a user provided scrollbar for a specified 'plottable' object:
    #--------------------------------------------------------------------------
 
    def init_scrollbar ( self, sb ):
        sb.on_trait_change( self._on_scroll, 'position' )

    #--------------------------------------------------------------------------
    #  Dispose of an existing scrollbar:
    #--------------------------------------------------------------------------
 
    def dispose_scrollbar ( self, sb ):
        sb.on_trait_change( self._on_scroll, 'position', remove = True )
 
    #--------------------------------------------------------------------------
    # Set the thumbbar position and size and the scrollbar range and page size:
    #--------------------------------------------------------------------------
    
    def set_scrollbar ( self, sb, position, range, page_size ):
        sb.range    = ( 0.0, range, page_size, self.scroll_incr )
        sb.position = position
        
    #--------------------------------------------------------------------------
    #  Get the current position of a specified scrollbar:
    #--------------------------------------------------------------------------
       
    def positionof_scrollbar ( self, sb, range, page_size ):
        return ((sb.position * (range - page_size)) / 
                (sb.high - sb.low - page_size))

    #--------------------------------------------------------------------------
    #  Handle a scroll event for a scaled plottable:
    #--------------------------------------------------------------------------
  
    def _on_scroll ( self, sb, old, position ):
        plottable = sb._plottable
        if sb.style[0] == 'h':
            if position != plottable._plot_ox:
                plottable._plot_ox = position
                plottable.update( REDRAW )
        elif position != plottable._plot_oy:
            plottable._plot_oy = position
            plottable.update( REDRAW )


#### EOF ######################################################################
