#!/usr/bin/env python
""" Plots chaco graphics in a PyFace window. """
#-------------------------------------------------------------------------------
#
#  Plot chaco graphics in a PyFace window.
#
#  Written by: Andrew D. Straw
#
#  Date: 04/18/2003
#
#  (c) Copyright 2003 by Andrew D. Straw
#  May be distributed under terms of Scipy license (BSD style).
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.


# Enthought library imports.
from enthought.util.numerix import pi
from enthought.chaco.wx.plot import *
from enthought.enable.wx     import Window
from enthought.kiva      import CompiledPath
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance

# Local imports.
from demo_base import PlotApplicationWindow


def make_well_symbol():
    "Creates a symbol representing wells on the map view"
    path = CompiledPath()
    path.scale_ctm(1.2,1.2)
    path.begin_path()
    path.move_to(0.0, 0.0)
    path.line_to(3.0, 15.0)
    path.line_to(7.0, 15.0)
    path.line_to(10.0, 0.0)
    path.move_to(2.5, 12.5)
    path.line_to(8.25, 8.75)
    path.line_to(1.0, 5.0)
    path.move_to(7.5, 12.5)
    path.line_to(1.75, 8.75)
    path.line_to(9.0, 5.0)
    path.scale_ctm(1.0/1.2, 1.0/1.2)

    return path

# Create a single instance of the well symbol compiled path
WELL_SYMBOL = make_well_symbol()

class SimplePlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """

    def _create_plot( self ):
        """ Create the plot to be displayed. """

        import enthought.util.numerix as Numeric
        x  = Numeric.arange( 101.0 )
        y1 = 10 * Numeric.sin( x * 0.3 ) + 3 * Numeric.sin( x * 0.2 ) + 15
        y2 =  8 * Numeric.cos( x * 0.3 ) + 3 * Numeric.sin( x * 0.2 ) + 15
        y3 =  2 * Numeric.sin( x * 0.5 ) + 1 * Numeric.sin( x * 0.1 )
        y4 =  8 * Numeric.cos( x * 0.2 ) + 3 * Numeric.sin( x * 0.1 )


        grid = PlotGrid(rows=2, columns=2)

        y_ax = PlotAxis( title = 'Y value' )
        x_ax = PlotAxis( title = 'X value', position='bottom' )
        label = PlotLabel(text='here', rotate=90, alignment='left')
        label_2 = PlotLabel(path=WELL_SYMBOL, path_size=(12,18))
        x_ax.markers[20.0] = label
        x_ax.markers[80.0] = label
        x_ax.markers[40.0] = label_2

        pv1  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )
        grid.add(pv1)

        y_ax = PlotAxis( title = 'Y value' )
        x_ax = PlotAxis( position='top' )
        label = PlotLabel(text='here', rotate=90, alignment='left')
        label_2 = PlotLabel(path=WELL_SYMBOL, path_size=(12,18))
        x_ax.markers[0.0] = label
        label = PlotLabel(text='or maybe here)', rotate=90)
        x_ax.markers[80.0] = label
        x_ax.markers[40.0] = label_2

        pv2  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )

        grid.add(pv2)

        y_ax = PlotAxis( title = 'Y value', position='left' )
        x_ax = PlotAxis( title = 'X value', position='top' )
        label = PlotLabel(text='here', alignment='left')
        label_2 = PlotLabel(path=WELL_SYMBOL, path_size=(12,18))
        y_ax.markers[4.0] = label
        y_ax.markers[25.0] = label
        y_ax.markers[10.0] = label_2

        pv3  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )

        grid.add(pv3)

        y_ax = PlotAxis( title = 'Y value', position='right' )
        x_ax = PlotAxis( title = 'X value', position='top', tick_visible=False, label_style='none' )
        label = PlotLabel(text='here', alignment='left')
        label_2 = PlotLabel(path=WELL_SYMBOL, path_size=(12,18))
        y_ax.markers[4.0] = label
        y_ax.markers[15.0] = label
        y_ax.markers[10.0] = label_2

        pv4  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )

        grid.add(pv4)

        return grid

#==============================================================================
#  Chaco use demonstration:
#==============================================================================

def main():
    """Demonstration of Chaco."""

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SimplePlotApplicationWindow( title = "Plot" )
    #window.plot_item = object
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()

    # Start the GUI event loop.
    gui.start_event_loop()

#===============================================================================
#  Program start-up:
#===============================================================================

if __name__ == '__main__':
    main()
