#!/usr/bin/env python
doc = \
"""
This file contains a support class that wraps up the boilerplate wx calls
that virtually all the demo programs have to use, and doesn't actually do
anything when run on its own.

Try running selection_demo.py, colormapped_plot.py, image_plot.py, or
others.
"""

#--------------------------------------------------------------------------------
#  Written by: Brandon DuRette
#  Date: 04/2005
#  May be distributed under terms of Scipy license (BSD style).
#--------------------------------------------------------------------------------

import wx
from enthought.enable.wx import Window
from enthought.pyface.api import ApplicationWindow, GUI
from enthought.chaco.plot_component import PlotComponent

class PlotApplicationWindow( ApplicationWindow ):
    "A simple PyFace application window that plots Chaco graphics."
    
    def _create_plot( self ):
        """
        Create the plot to be displayed.
        Subclasses must override this method to produce the desired plot.
        """
        raise NotImplementedError
    
    def _create_contents( self, parent ):
        """ Create the toolkit-specific content for this window. """
        
        # Create the enable-compliant PlotComponent.
        plot_component = PlotComponent( component = self._create_plot() )
        
        # Create the enable window with the enable component.
        plot_window = Window( parent, component = plot_component )
        
        return plot_window.control
    
    
    def _plot_item_changed(self):
        """ Update the window when the plot_item changes. """
        
        if self._plot_component is not None:
            self._plot_component.component = self.plot_item
        
        return


#=============================================================================== 
#  'plot_it' function:
#=============================================================================== 

def plot_it( object ):
    """ Displays a PyFace window with the Chaco graphics object. """

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = PlotApplicationWindow( title = "Plot" )
    window.plot_item = object
    window.size = (2*screen_width/3, 2*screen_height/3)
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()


if __name__ == "__main__":
    print "\n" + doc + "\n"
