#!/usr/bin/env python
#-------------------------------------------------------------------------------
#  errorbar_demo.py
#
#  Demonstration of how to do errorbar plots with Chaco.
#
#  Written by: Peter Wang
#
#  Date: 7/5/2005
#
#  (c) Copyright 2005 by Enthought
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.

# Enthought library imports.
from enthought.util.numerix  import arange, Float, cos, ones, pi, sin
from enthought.chaco.wx.plot import *
from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance
from enthought.chaco.plot_value import ErrorbarPlotValue, SymmetricErrorbarPlotValue

from demo_base import PlotApplicationWindow

class SimplePlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot( self ):
        """ Create the plot to be displayed. """
        
        x = arange( 101.0 )
        y1 = 10 * sin( x * 0.3 ) + 3 * sin( x * 0.2 ) + 15
        y2 = y1 - 10
        high_data =  3 * cos( x * 0.3 ) + sin( x * 0.2 ) + 2
        low_data = ones(len(x), Float) * 2
        
        x_ax = PlotAxis( title = 'X value' )
        y_ax = PlotAxis( title = 'Y value', selected = True )
        
        pv1 = SymmetricErrorbarPlotValue( zip(x,y1), axis_index = x_ax,axis = y_ax,
                                          line_color = 'blue',
                                          type="symmetricerrorbar" )
        pv1.error_value = low_data
        
        pv2 = ErrorbarPlotValue( zip(x,y2), axis_index = x_ax, axis=y_ax,
                                 line_color="red", type="errorbar" )
        pv2.low_value = low_data
        pv2.high_value = high_data
        
        pc = PlotCanvas( pv1, pv2 )

        return pc
    
#============================================================================== 
#  Chaco use demonstration:
#============================================================================== 

def main():
    """Demonstration of Chaco."""

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SimplePlotApplicationWindow( title = "Plot" )
    #window.plot_item = object
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#=============================================================================== 
#  Program start-up:
#=============================================================================== 

if __name__ == '__main__':
    main()
