

# Other Enthought library imports.
from enthought.enable.wx import Window
from enthought.pyface.api import ApplicationWindow, GUI
from enthought.traits.api import List

# Chaco library imports.
from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_component import PlotComponent
from enthought.chaco.plot_title import PlotTitle

from gis_parser import GISParser
from state import State
from state_data import StateData

class GISApplicationWindow(ApplicationWindow):
    """ A PyFace application window for displaying the Chaco GIS plot. """

    states = List(State)

    ###########################################################################
    # ApplicationWindow interface.
    ###########################################################################

    def _create_contents(self, parent):
        """ Create the toolkit-specific content for this window. """

        canvas = PlotCanvas()

        for state in self.states:
            state_value = StateData(state=state)
            canvas.add(state_value)
            
        canvas.add(PlotTitle('Continental United States', font='modern 14'))

        plot_component = PlotComponent(component=canvas)

        window = Window(parent, component=plot_component)

        return window.control

###############################################################################
# Program start-up.
###############################################################################

def main():
    """ Application main routine. """

    # Parse all of the state data into a 'State' objects.
    parser = GISParser()
    states = parser.parse('../data/states/statesp020')
    # Exclude states not in Continental U.S.
    exclude = ['Alaska', 'Hawaii', 'U.S. Virgin Islands', 'Puerto Rico']
    continental_states = [ state for state in states.values()
                           if state.name not in exclude ]


    for state in continental_states:
        state.downsample(1200, 600, 65, 25)

    import enthought.gotcha as gotcha
    gotcha.begin_profiling()
    # Create the GUI object (this does NOT start the event loop).
    gui = GUI()

    # Create the main window and open it.
    window = GISApplicationWindow(size=(600, 400), states=continental_states)
    window.open()

    # Start the GUI event loop.
    gui.event_loop()
    gotcha.end_profiling()

    return

if __name__ == '__main__':
    main()

#### EOF ######################################################################
