from struct import unpack

from enthought.traits.api import HasTraits

from state import State

class GISParser(HasTraits):

    ###########################################################################
    # 'GISParser' interface.
    ###########################################################################

    def parse(self, file_prefix):
        """ Creates a set of records by parsing gis files. """

        shape_file = open( file_prefix + '.shp', 'rb' )
        records = {}
        try:
            self.read_header(shape_file)
            while True:
                try:
                    self.read_record(shape_file, records)
                except:
                    break
                
        finally:
            shape_file.close()
            if len(records.keys()) > 0:
                keys = records.keys()
                keys.sort()

        states = {}
        db_file = open(file_prefix + '.csv', 'rb')
        lines = db_file.readlines()
        for line in lines[1:]:
            values = line.split(',')
            name = values[3]
            id = values[2]
            
            # Get ahold of the appropriate state, creating it if necessary.
            state = states.get(name, None)
            if state is None:
                state = State(name)
                states[name] = state

            id = int(id) - 1
            state.boundaries.append(records[str(id)])
            
        return states

    def read_header(self, fh):
        """ Read the shape file header. """
        
        header = (unpack('>i 20x i', fh.read(28)) +
                  unpack('<i i 4d 32x', fh.read(72)))

        if header[0] != 9994:
            raise ValueError('Not a shape file.')

        return header

    def read_record(self, fh, records):
        """ Read an individual record from the shape file. """

        rn, size = unpack('>2i', fh.read(8))
        type = unpack('<i', fh.read(4))[0]
        data = fh.read(2 * size - 4)

        key = '%d' % rn

        # Point:
        if type == 1:
            records[key] = unpack('<2d', data)

        # Multipoint:
        elif type == 8:
            xmin, ymin, xmax, ymax, n = unpack('<4d i', data[0:36])
            points = []
            index = 36
            for i in range(n):
                points.append(unpack('<2d', data[index:index+16]))
                index += 16
            records[key] = points

        # Arc/Polygon:
        elif (type == 3) or (type == 5):
            xmin, ymin, xmax, ymax, parts, n = unpack('<4d 2i', data[0:40])
            index   = 40 + 4 * parts
            offsets = unpack('<%di' % parts, data[40:index]) + (n,)
            points  = []
            results = []
            for i in range( n ):
                points.append( unpack( '<2d', data[ index: index + 16 ] ) )
                index += 16
            for i in range( parts ):
                results.append(points[offsets[i]: offsets[i+1]])
                records[key] = results

        # Unknown:
        else:
            raise ValueError('Unrecognized record type %d. Ignoring.' % type)

        return
            
        
#### EOF ######################################################################
