

# Major package imports.
from random import random
from types import DictType

# Enthought library imports.
from enthought.enable.traits.api import RGBAColor
from enthought.traits.api import Instance, Trait

# Chaco imports.
from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_data import PlotData
from enthought.chaco.plot_value import PlotValue

# Local imports.
from state import State


class StateData(PlotData):
    """ A PlotData wrapper around a state. """

    # The state which this StateData adapts.
    state = Instance(State)

    # The color to represent this state visually.
    color = RGBAColor


    ###########################################################################
    # Public PlotData interface.
    ###########################################################################

    def add_to(self, plottable, source_name):

        if isinstance(plottable, PlotValue):
            self.get_source(plottable, source_name or 'boundary 0,0')
            return

        if source_name is not None:
            value = PlotValue(plot_type = 'polygon',
                              fill_color=self.color,
                              line_color='black',
                              antialias=False)
            self.get_source(value, source_name)
            plottable.add(value)

        else:
            canvas = PlotCanvas(plot_type = 'polygon')
            for i, boundary in enumerate(self.state.boundaries):
                for j in range(len(boundary)):
                    value = PlotValue(fill_color=self.color,
                                      line_color='black',
                                      antialias=False)
                    value._state_data = self
                    self.get_source(value, 'boundary %d,%d' % (i,j))
                    canvas.add(value)

            plottable.add(canvas)

        return


    def data(self, source_name):
        try:
            if source_name.startswith('boundary'):
                return self._get_boundary(source_name)

            else:
                return self._lookup_key(self.state.statistics, source_name)

        except StandardError, e:
            print 'Error retrieving %s' % source_name
            return [0.0, 1.0]

    def source(self, value, source_name):
        if source_name == 'default':
            source_name = 'boundary 0,0'

        return super(StateData, self).source(value, source_name)


    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_boundary(self, source_name):

        id = source_name.split(' ')[1]
        index = int(id.split(',')[0])
        subindex = int(id.split(',')[1])
        result = self.state.boundaries[index][subindex]
        return result

    ###########################################################################
    # Trait handlers.
    ###########################################################################

    #### Static ####

    def _state_changed(self, old, new):
        """ Called when the state trait changes. """

        boundary_pattern = 'boundary %d,%d'
        if new is not None:
            boundary_names = []
            for i, boundary in enumerate(new.boundaries):
                for j in range(len(boundary)):
                    boundary_names.append(boundary_pattern % (i,j))

            self.sources = boundary_names
            self.color = (random(), random(), random(), 1.0)

        return


#### EOF ######################################################################
