

# Major package imports.
from random import random
from types import DictType

# Enthought library imports.
from enthought.enable.traits.api import RGBAColor
from enthought.traits.api import Instance, Trait

# Chaco imports.
from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_data import PlotData
from enthought.chaco.plot_value import PlotValue

# Local imports.
from state import State


class StateData(PlotData):
    """ A PlotData wrapper around a state. """

    # The state which this StateData adapts.
    state = Instance(State)

    # The color to represent this state visually.
    color = RGBAColor


    ###########################################################################
    # Public PlotData interface.
    ###########################################################################

    def add_to(self, plottable, source_name):

        if isinstance(plottable, PlotValue):
            self.get_source(plottable, source_name or 'boundary 0,0')
            return

        if source_name is not None:
            if source_name.startswith('boundary'):
                value = PlotValue(plot_type = 'polygon',
                                  fill_color=self.color,
                                  line_color='black',
                                  antialias=False)
                self.get_source(value, source_name)
                plottable.add(value)

            else:
                value = PlotValue(plot_type='line', line_color=self.color)
                self.get_source(value, source_name)
                plottable.add(value)

        else:
            canvas = PlotCanvas(plot_type = 'polygon')
            for i, boundary in enumerate(self.state.boundaries):
                for j in range(len(boundary)):
                    value = PlotValue(fill_color=self.color,
                                      line_color='black',
                                      antialias=False)
                    value._state_data = self
                    self.get_source(value, 'boundary %d,%d' % (i,j))
                    canvas.add(value)

            plottable.add(canvas)

        return


    def data(self, source_name):
        try:
            if source_name.startswith('boundary'):
                return self._get_boundary(source_name)

            else:
                return self._lookup_key(self.state.statistics, source_name)

        except StandardError, e:
            print 'Error retrieving %s' % source_name
            return [0.0, 1.0]

    def source(self, value, source_name):
        if source_name == 'default':
            source_name = 'boundary 0,0'

        return super(StateData, self).source(value, source_name)


    ###########################################################################
    # Private interface.
    ###########################################################################

    def _extract_keys(self, dictionary):
        """ Recursively extract parseable keys from the dictionary.

        The resultant keys are of the form form is <key>_<subkey>_<subsubkey>...
        """

        result = []
        for key in dictionary.keys():
            if type(dictionary[key]) is DictType:
                subkeys = self._extract_keys(dictionary[key])
                for subkey in subkeys:
                    result.append("_".join((key, subkey)))
            else:
                result.append(key)

        return result

    def _lookup_key(self, dictionary, key):
        """ Parse and lookup the value for the key in the dictionary. """

        keys = key.split('_')
        if len(keys) > 1:
            # Recursively lookup the key.
            return self._lookup_key(dictionary[keys[0]], '_'.join(keys[1:]))

        else:
            return dictionary[keys[0]]

    def _get_boundary(self, source_name):

        id = source_name.split(' ')[1]
        index = int(id.split(',')[0])
        subindex = int(id.split(',')[1])
        result = self.state.boundaries[index][subindex]
        return result

    ###########################################################################
    # Trait handlers.
    ###########################################################################

    #### Static ####

    def _state_changed(self, old, new):
        """ Called when the state trait changes. """

        boundary_pattern = 'boundary %d,%d'
        if new is not None:
            boundary_names = []
            for i, boundary in enumerate(new.boundaries):
                for j in range(len(boundary)):
                    boundary_names.append(boundary_pattern % (i,j))

            statistics = self._extract_keys(new.statistics)
            self.add_trait('source_names', Trait(boundary_names + statistics))
            self.color = (random(), random(), random(), 1.0)

        return


#### EOF ######################################################################
