
from enthought.traits.api import HasTraits


class UnemploymentParser(HasTraits):

    def parse(self, filename):
        """ Creates a multi-level map of state unemployment data.

        Format returned is:
        <State> : (['Adjusted'|'Not Adjusted'] : (<Series Name> : <Data>)
        where:
        State is the name of the state
        Series name may be one of 'date', 'labor force', 'employment',
        'unemployment', or 'unemployment rate'.
        Data is the data (as a list) of the data for that series.
        """

        
        unemployment_file = open(filename, 'r')

        result = {}
        
        while True:
            try:
                state, adjusted, data = self._read_table(unemployment_file)
                # Set the appropriate data in the map.
                state_map = result.setdefault(state, {})
                state_map[adjusted] = data

            except Exception, e:
                break

        return result

    def _read_table(self, file):
        """ Read a table from the given file. """

        self._skip(file, 2)
        seasonal = file.readline().strip()
        state = file.readline().strip()
        state = state.split(':')[1]
        self._skip(file, 5)

        # Now we should be at the real data.
        date, force, employment, unemployment, rate = [], [], [], [], []
        while True:
            line = file.readline().strip()
            
            # Check that we have not found the last line in the table.
            if len(line) == 0:
                break

            items = line.split(',')
            date.append(' '.join((items[1], items[0])))
            force.append(int(self._cleanup(items[2])))
            employment.append(int(self._cleanup(items[3])))
            unemployment.append(int(self._cleanup(items[4])))
            rate.append(float(self._cleanup(items[5])))

        self._skip(file, 4)

        result = { 'date' : date,
                   'workforce' : force,
                   'employment' : employment,
                   'unemployment' : unemployment,
                   'rate' : rate
                 }

        return state, seasonal, result

    def _skip(self, file, count):
        for i in range(count):
            
            skipped = file.readline()

        return

    def _cleanup(self, item):
        if item.endswith(')'):
            return item[:-3]
        else:
            return item

#### EOF ######################################################################
