#!/usr/bin/env python
"""
Demonstration of plotting a pre-existing image from file.
"""
#--------------------------------------------------------------------------------
#
#  Plot chaco graphics in a wxPython wxFrame window (with option to save as PDF).
#
#  Written by: Peter Wang
#  Date: 04/2005
#
#  May be distributed under terms of Scipy license (BSD style).
#
#--------------------------------------------------------------------------------

#--------------------------------------------------------------------------------
#  Imports:
#--------------------------------------------------------------------------------

import wx

from enthought.traits.api import Any, Instance
from enthought.enable.wx     import Window
from enthought.enable.base   import blue_color, transparent_color
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.util.numerix  import arange, array, concatenate, pi, resize, \
                                    sin, transpose

from enthought.chaco import PlotAxis, PlotCanvas, PlotComponent, PlotGroup, \
                            PlotValue, ImagePlotValue
from enthought.chaco.interaction import ZoomInteraction

from demo_base import PlotApplicationWindow


class ImagePlotApplicationWindow( PlotApplicationWindow ):

    ###########################################################################
    # PlotApplicationWindow interface.
    ###########################################################################
    
    def _create_plot(self):
        xbounds = (-4.0, 4.0)
        ybounds = (-4.0, 4.0)
        
        xaxis = PlotAxis(tick_visible=True, grid_visible=True, \
                         bound_low = xbounds[0], bound_high = xbounds[1])
        yaxis = PlotAxis(tick_visible=True, grid_visible=True, \
                     bound_low = ybounds[0], bound_high = ybounds[1])
        
        # create the image plot
        image_pv = ImagePlotValue((-3.5,0.5), (-0.5, 3.5), "kaylee.jpg",
                                  axis_index=xaxis, axis=yaxis)
        image_pv.alpha = 0.65
        
        image_pv2 = ImagePlotValue((-0.5, 3.5), (-3.5, 0.5), "capitol.jpg",
                                   axis_index = xaxis, axis = yaxis)
        image_pv2.alpha = 0.75
        
        # create the scatter plot
        xwidth = xbounds[1] - xbounds[0]
        ywidth = ybounds[1] - ybounds[0]
        x = arange(xbounds[0], xbounds[1], xwidth/100)
        y = sin(x*4*pi/xwidth) * ywidth/2
        scatter_data = transpose(array([x,y]))
        scatter_pv = PlotValue(scatter_data, axis_index=xaxis, axis=yaxis,
                               type="line,scatter")
        
        zoom = ZoomInteraction()
        zoom.color = (0.439216, 0.858824, 0.576471, 0.3)    # transparent aquamarine
        zoom.border_color = blue_color
        zoom.bounds = (0,0,0,0)
        
        canvas = PlotCanvas(image_pv, image_pv2, scatter_pv, axis_index=xaxis, axis=yaxis,
                            interaction = zoom)
        
        return canvas

def main():

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = ImagePlotApplicationWindow( title = "Plot" )
    #window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.size = (800,600)
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()


#=============================================================================== 
#  Program start-up:
#=============================================================================== 

if __name__ == '__main__':
    main()
