
import wx
from enthought.util.numerix import arange, sin
from enthought.enable.wx import Window
from enthought.chaco.wx.plot import PlotAxis, PlotComponent, PlotCanvas, PlotValue

from enthought.chaco.plot_interaction import PlotInteraction

class KeyboardPan(PlotInteraction):
    stepsize = 10  # number of pixels to move left or right
    accepts_focus = True
    def _key_changed(self, event):
        # event is an enable.events.KeyEvent; character is a simple char
        # or the name of a control char (see enable.wx.window.key_map).
        if event.character == "Right":
            delta = self.stepsize
        elif event.character == "Left":
            delta = -self.stepsize
        else:
            return
            
        handled_axes = []
        for plotvalue in self.canvas.values:
            x_axis = plotvalue.axis_index
            # Check for auto-scaling and turn it off.
            if 'auto' in (x_axis.bound_low, x_axis.bound_high):
                axis_info = plotvalue._axis_info_index
                x_axis.bound_low = axis_info.min
                x_axis.bound_high = axis_info.max
            
            xscale = plotvalue._plot_sxdx / (x_axis.bound_high - x_axis.bound_low)
            if x_axis not in handled_axes:
                x_axis.bound_low += delta/xscale
                x_axis.bound_high += delta/xscale
                handled_axes.append(x_axis)
        self.canvas.update()
        return


class MyFrame ( wx.Frame ):
    def __init__ ( self, *args, **kw ):
        wx.Frame.__init__( *(self,) + args, **kw )
        data = sin(arange(-5.0, 5.0, 0.3))
        
        pan = KeyboardPan()
        x_axis = PlotAxis(title="X axis")
        y_axis = PlotAxis(title="Y axis")
        plot = PlotValue(data, type="scatter,line",
                          axis_index = x_axis, axis = y_axis,
                          interaction=pan)
        plot_window = Window(self, component=PlotComponent(plot))
        
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(plot_window.control, 1, wx.EXPAND)
        self.SetSizer(sizer)
        self.SetAutoLayout( True )
        self.Show( True )


def main():
    app = wx.PySimpleApp()
    frame = MyFrame(None, size=(400,400))
    app.SetTopWindow(frame)
    app.MainLoop()

if __name__ == "__main__":
    main()
