#!/usr/bin/env python
"""Demo of mathtext labels"""

# Major package imports.

# Enthought library imports.
from enthought.util.numerix import pi
from enthought.chaco.plot_axis import PlotAxis
from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_overlay import PlotOverlay
from enthought.chaco.plot_title import PlotTitle
from enthought.chaco.plot_value import PlotValue

from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance


from demo_base import PlotApplicationWindow

class SimplePlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot( self ):
        """ Create the plot to be displayed. """
        
        import enthought.util.numerix as Numeric
        x  = Numeric.arange( 101.0 )
        y1 = 10 * Numeric.sin( x/5 ) + 3 * Numeric.sin( x/5 ) + 15
        
        x_ax = PlotAxis( title = r'$\mu = e^{i\omega}$' )
        y_ax = PlotAxis( title = 'Y value' )
        pv1  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )
        overlay = PlotOverlay(text=r"$\nabla \cdot E = 4 \pi \rho$", value=20.0, index=21.0)
        x_val     = PlotValue( x )
        pc        = PlotCanvas( pv1,
                                PlotTitle( 'Demo of MathText labels' ),
                                margin = 15 )

        return pc
    
#============================================================================== 
#  Chaco use demonstration:
#============================================================================== 

def main():
    """Demonstration of Chaco."""

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SimplePlotApplicationWindow( title = "Plot" )
    #window.plot_item = object
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#=============================================================================== 
#  Program start-up:
#=============================================================================== 

if __name__ == '__main__':
    main()
