# driver for sample plots
# determines what backend to use based on file extension.
import sys
import os
import glob

import enthought.chaco.plot_canvas
import enthought.chaco.plot_frame
import enthought.chaco.plot_base

from enthought.util.numerix import *

GCMap = {}

try:
    from enthought.kiva.svgcore2d import SVGGC
    GCMap['.svg'] = SVGGC
    GCMap['.html'] = SVGGC
except ImportError:
    print "Couldn't load SVG backend"

try:
    from enthought.kiva.pilcore2d import PILGC
    GCMap['.gif'] = PILGC
    GCMap['.png'] = PILGC
    GCMap['.jpg'] = PILGC
    GCMap['.jpeg'] = PILGC
except ImportError:
    print "Couldn't load PIL backend"

#try:
#    from enthought.kiva.pdfcore2d import GraphicsContext as PDFGC
#    GCMap['.pdf'] = PDFGC
#except ImportError:
#    print "Couldn't load PDF backend"

if len(sys.argv) == 1:
    print "Usage: %s <plot_type> <output_file>\n" % sys.argv[0]
    print "Valid plot types are:\n\t",
    print '\n\t'.join([name[len("draw_"):-3] for name in glob.glob("draw_*.py")])
    print "\nOuput file must end in one of:", ', '.join(GCMap.keys())
    raise SystemExit

plot_type = 'draw_' + sys.argv[1] + '.py'
filename = sys.argv[2]

class Canvas:
    def __init__(self, filename='', id = -1, size = (200,200)):
        extension = os.path.splitext(filename)[1]
        self.filename = filename
        self._size = size
        self.gc = GCMap[extension](size)

    def size(self):
        return self._size
        
    def save(self):
        self.gc.save(self.filename)

canvas = Canvas( filename=filename, size=(400,400))
execfile(plot_type)
canvas.save()  # Save the contents of the GraphicsContext to the file
print "Created:", filename
