# driver for sample plots
# determines what backend to use based on file extension.
import sys
import os
import glob

import enthought.chaco.plot_canvas
import enthought.chaco.plot_frame
import enthought.chaco.plot_base

extension_map = {
    '.svg': 'svgcore2d',
    '.ps': 'pscore2d',
    '.eps': 'pscore2d',
    '.epsf': 'pscore2d',
    '.png': 'pilcore2d',
    '.bmp': 'pilcore2d',
    '.gif': 'pilcore2d',
    '.jpg': 'pilcore2d',
    '.jpeg': 'pilcore2d',
    '.pdf': 'pdfcore2d',
}

def get_gc(extension):
    if extension not in extension_map:
        extensions = extension_map.keys()
        extensions.sort()
        print "\nOuput file must end in one of:\n\t", '\n\t'.join(extensions)
        sys.exit(1)
    module_name = extension_map[extension]
    try:
        # do the equivalent of "from enthought.kiva.foocore2d import GraphicsContext")
        module = __import__('enthought.kiva.'+module_name,
                            globals(),
                            locals(),
                            ['GraphicsContext'])
        return module.GraphicsContext
    except ImportError:
        print "Couldn't load %s backend" % module_name
        sys.exit(1)

def usage():
    print "Usage: %s <plot_type> <output_file>\n" % sys.argv[0]
    print "Valid plot types are:\n\t",
    print '\n\t'.join([name[len("draw_"):-3] for name in glob.glob("draw_*.py")])
    extensions = extension_map.keys()
    extensions.sort()
    print "\nOuput file must end in one of:\n\t", '\n\t'.join(extensions)
    sys.exit(0)

if len(sys.argv) != 3:
    usage()

if __name__ == '__main__':
    filename = sys.argv[2]
    extension = os.path.splitext(filename)[1]
    width = 300
    height = 300
    x = int(width * .05)
    y = int(height * .05)
    dx = width - 2*x
    dy = height - 2*y
    gc = get_gc(extension)((width, height))
    mod = __import__('draw_' + sys.argv[1])
    mod.plot.render(gc, x, y, dx, dy)
    gc.save(filename)  # Save the contents of the GraphicsContext to the file
    print "Created:", filename
