#!/usr/bin/env python
"""Demonstrates the use of the pan and zoom interactions."""

# Major package imports.

# Enthought library imports.
from enthought.util.numerix import pi
from enthought.chaco.interaction import PanZoomInteraction
from enthought.chaco import PlotAxis, PlotCanvas, PlotTitle, PlotValue
from enthought.enable.wx import Window
from enthought.pyface.api import ApplicationWindow, GUI
from enthought.traits.api import Any, Instance

from demo_base import PlotApplicationWindow

    
class PanZoomPlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot( self ):
        """Demonstration of Chaco."""

        from enthought.util.randomx import random
        data = random((2500,2))
        data2 = random((2500,2))
        
        x_ax = PlotAxis( title = 'X value' )
        y_ax = PlotAxis( title = 'Y value' )
        pv1  = PlotValue( data,
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'scatter')
        pv2  = PlotValue( data2,
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'scatter',
                          symbol     = 'circle')
        pan_zoom = PanZoomInteraction()
        pc        = PlotCanvas( pv1,
                                pv2,
                                PlotTitle( 'Chaco Interaction Demonstration' ),
                                margin = 15,
                                interaction = pan_zoom)

        return pc

    
def main():
    """ Displays a PyFace window with your Chaco graphics object. """

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = PanZoomPlotApplicationWindow( title = "Plot" )
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#============================================================================== 
#  Program start-up:
#============================================================================== 

if __name__ == '__main__':
    main()


#### EOF ######################################################################
