#!/usr/bin/env python
""" Plots chaco graphics in a PyFace window. """
#-------------------------------------------------------------------------------
#
#  Plot chaco graphics in a PyFace window.
#
#  Written by: Andrew D. Straw
#
#  Date: 04/18/2003
#
#  (c) Copyright 2003 by Andrew D. Straw
#  May be distributed under terms of Scipy license (BSD style).
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.
import wx

# Enthought library imports.
from enthought.util.numerix  import pi
from enthought.chaco.wx.plot import *
from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance

try:
    import enthought.kiva.pdfcore2d
except:
    have_pdf = False
else:
    have_pdf = True


class PlotApplicationWindow( ApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    # The chaco object to be plotted (PlotCanvas, PlotGroup, etc.)
    # plot_item = Any
    
    # The chaco PlotData source to be plotted.
    plot_data = Any

    
    # Internally wrapped version of the plot_item.
    _plot_component = Instance( PlotComponent )

    # Internal reference to the actual window.     
    _plot_window = Any
    
    def _create_contents( self, parent ):
        """ Create the toolkit-specific content for this window. """
            
        component = wx.Panel( parent, -1 )

        # Create the plot canvas object with the data source.
        canvas = PlotCanvas()  #  self.plot_data )
        value = PlotValue()
        self._plot_source = self.plot_data.get_source(value, 'high')
        canvas.add(value)
        self._plot_component = PlotComponent( component = canvas )
        self._plot_window = Window( component,
                                    component = self._plot_component )

        # The properties of the plotted data source.
        data_properties = self.plot_data.edit_traits(kind='subpanel', parent=component)
        source_properties = self._plot_source.edit_traits( kind = 'subpanel',
                                                           parent = component )
        
        # Lay out the controls.
        box = wx.BoxSizer( wx.VERTICAL )
        box.Add( self._plot_window.control, 1, wx.EXPAND )
        box.Add( data_properties.control, 0, wx.ALL )
        box.Add( source_properties.control, 0, wx.ALL )

        component.SetSizer( box )
        component.SetAutoLayout( True )
        
        #return self._plot_window.control
        return component
     

def plot_it( object ):
    """ Displays a PyFace window with your Chaco graphics object. """

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = PlotApplicationWindow( title = "Plot" )
    window.plot_data = object
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()

    # Start the GUI event loop.
    gui.start_event_loop()

#============================================================================== 
#  Chaco use demonstration:
#============================================================================== 

def main():
    """Demonstration of Chaco."""
    
    from enthought.chaco.extras.stock_quote import StockQuote
    plot_data = StockQuote()
    plot_it( plot_data )

#============================================================================== 
#  Program start-up:
#============================================================================== 

if __name__ == '__main__':
    main()
