#!/usr/bin/env python
"Demonstration of the colormapped scatterplot functionality."
#-------------------------------------------------------------------------------
#  Plot chaco graphics in a PyFace window.
#  Written by: Peter Wang
#  Date: 07/28/2005
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.

# Enthought library imports.
from enthought.util.numerix  import arange, array, cos, pi, sin, transpose
from enthought.util.randomx  import uniform
from enthought.chaco.wx.plot import *
from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance
from enthought.chaco.plot_value_cmap import ColorMappedPlotValue
from enthought.chaco.colormap import LinearColormap

from demo_base import PlotApplicationWindow

class SimplePlotApplicationWindow(PlotApplicationWindow):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot(self):
        """ Create the plot to be displayed. """
        
        numpoints = 30000
        
        points = uniform(-1.0, 1.0, (2,numpoints))
        x,y = points
        colorval = sin(x*3*pi) + sin(y*3*pi)
        
        cmap = LinearColormap(color_depth='rgba',
                              min_color=(1.0,0.0,0.0,1.0),
                              max_color=(0.0,0.0,1.0,1.0),
                              min_value=min(colorval),
                              max_value=max(colorval),
                              steps=50)
        
        x_ax = PlotAxis(title = 'X value')
        y_ax = PlotAxis(title = 'Y value')
        pv  = ColorMappedPlotValue(transpose(array([x,y,colorval])),
                          colormap   = cmap, 
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'colormappedscatter',
                          size       = 'medium',
                          symbol     = 'circle')
        
        pc = PlotCanvas(pv)

        return pc
    
#============================================================================== 
#  Chaco use demonstration:
#============================================================================== 

def main():
    """Demonstration of Chaco."""

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SimplePlotApplicationWindow(title = "Plot")
    #window.plot_item = object
    window.size = (2.0/3.0*screen_width, 2.0/3.0*screen_height)
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#=============================================================================== 
#  Program start-up:
#=============================================================================== 

if __name__ == '__main__':
    main()
