#!/usr/bin/env python
""" Plots chaco graphics in a PyFace window. """
#-------------------------------------------------------------------------------
#
#  Plot chaco graphics in a PyFace window.
#
#  Written by: Andrew D. Straw
#
#  Date: 04/18/2003
#
#  (c) Copyright 2003 by Andrew D. Straw
#  May be distributed under terms of Scipy license (BSD style).
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.

# Enthought library imports.
from enthought.util.numerix  import pi
from enthought.chaco.interaction.observe_interaction import ObserveInteraction
from enthought.chaco.interaction.select_interaction import SelectInteraction
from enthought.chaco.wx.plot import *
from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance

from demo_base import PlotApplicationWindow

class SelectionPlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot( self ):
        """Demonstration of Chaco."""

        from enthought.util.randomx  import random
        data = random((100,2))
        data2 = random((100,2))
        
        x_ax = PlotAxis( title = 'X value' )
        y_ax = PlotAxis( title = 'Y value' )
        pv1  = PlotValue( data,
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'scatter' )
        pv2  = PlotValue( data2,
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'scatter',
                          symbol     = 'circle' )
        observe_interaction = ObserveInteraction()
        observe_interaction.on_trait_change(self._do_observe, 'observed_value')
        select_interaction = SelectInteraction(reset_on_complete=False)
        select_interaction.on_trait_change(self._do_select, 'complete')
        pc        = PlotCanvas( pv1,
                                pv2,
                                PlotTitle( 'Chaco Interaction Demonstration' ),
                                margin = 15,
                                interaction = select_interaction)

        return pc

    ###########################################################################
    # Private interface. 
    ###########################################################################

    def _do_observe( self, source, trait, old, new ):
        """ Observe the plot value. """
        
        if new is not None:
            new.fill_color = 'red'
            new.update()
            
            if old is not None:
                old.fill_color = 'yellow'
                old.update()
                
        return

    def _do_select( self, source, trait, event ):

        print source.selections

        return


    
def main():
    """ Displays a PyFace window with your Chaco graphics object. """

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SelectionPlotApplicationWindow( title = "Plot" )
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#============================================================================== 
#  Program start-up:
#============================================================================== 

if __name__ == '__main__':
    main()


#### EOF ######################################################################
