#!/usr/bin/env python
""" Plots chaco graphics in a PyFace window. """
#-------------------------------------------------------------------------------
#
#  Plot chaco graphics in a PyFace window.
#
#  Written by: Andrew D. Straw
#
#  Date: 04/18/2003
#
#  (c) Copyright 2003 by Andrew D. Straw
#  May be distributed under terms of Scipy license (BSD style).
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

# Major package imports.

# Enthought library imports.
from enthought.util.numerix  import pi
from enthought.chaco.plot_axis import PlotAxis
from enthought.chaco.plot_canvas import PlotCanvas
from enthought.chaco.plot_overlay import PlotOverlay
from enthought.chaco.plot_title import PlotTitle
from enthought.chaco.plot_value import PlotValue

from enthought.enable.wx     import Window
from enthought.pyface.api        import ApplicationWindow, GUI
from enthought.traits.api        import Any, Instance

from enthought.kiva import Font, MODERN

from demo_base import PlotApplicationWindow

font = Font(family=MODERN)

class SimplePlotApplicationWindow( PlotApplicationWindow ):
    """ A simple PyFace application window that plots Chaco graphics. """
    
    def _create_plot( self ):
        """ Create the plot to be displayed. """
        
        import enthought.util.numerix as Numeric
        x  = Numeric.arange( 101.0 )
        y1 = 10 * Numeric.sin( x * 0.3 ) + 3 * Numeric.sin( x * 0.2 ) + 15
        y2 =  8 * Numeric.cos( x * 0.3 ) + 3 * Numeric.sin( x * 0.2 ) + 15
        y3 =  2 * Numeric.sin( x * 0.5 ) + 1 * Numeric.sin( x * 0.1 )
        y4 =  8 * Numeric.cos( x * 0.2 ) + 3 * Numeric.sin( x * 0.1 )
        
        x_ax = PlotAxis( title = 'X value', label_font=font)
        y_ax = PlotAxis( title = 'Y value', label_font=font )
        pv1  = PlotValue( zip( x, y1 ),
                          axis_index = x_ax,
                          axis       = y_ax,
                          type       = 'line',
                          line_color = 'blue' )
        overlay = PlotOverlay(text="Lorem Ipsum", value=20.0, index=21.0)
        pv1.add(overlay)
        pv2 = PlotValue( zip( x, y2 ),
                         axis_index = x_ax,
                         axis       = y_ax,
                         type       = 'line,scatter',
                         symbol     = 'cross',
                         line_color = 'green' )
        
        pv3       = PlotValue( y3 )
        pv4       = PlotValue( y4 )
        x_val     = PlotValue( x )
        fill_canv = PlotCanvas( pv3, pv4, type = 'rangebar', index = x_val,
                                axis_index = x_ax, axis = y_ax )
        pc        = PlotCanvas( pv1, pv2, fill_canv,
                    PlotTitle( 'Simple wxPython-based Chaco Demonstration', font=font ),
                                margin = 15,
                                )

        return pc
    
#============================================================================== 
#  Chaco use demonstration:
#============================================================================== 

def main():
    """Demonstration of Chaco."""

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Screen size:
    screen_width = gui.system_metrics.screen_width or 1024
    screen_height = gui.system_metrics.screen_height or 768

    # Create and open the main window.
    window = SimplePlotApplicationWindow( title = "Plot" )
    #window.plot_item = object
    window.size = ( 2 * screen_width / 3, 2 * screen_height / 3 )
    window.open()
    
    # Start the GUI event loop.
    gui.start_event_loop()

#=============================================================================== 
#  Program start-up:
#=============================================================================== 

if __name__ == '__main__':
    main()
