#-------------------------------------------------------------------------------
#  
#  Define a class for parsing text files containing nested dictionaries
#
#  Written by: David C. Morrill
#
#  Date: 09/03/2002
#
#  Copyright (c) Enthought, Inc. 2002
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  'TraitObject' class:
#-------------------------------------------------------------------------------

class TraitObject:
    
    #---------------------------------------------------------------------------
    #  Initialize an object:
    #---------------------------------------------------------------------------
    
    def __init__ ( self, fn_or_str = None ):
        self.name     = ''
        self.traits   = {}
        self.contents = []
        self.filename = None
        if fn_or_str is not None:
            self.parse( fn_or_str )
        
    #---------------------------------------------------------------------------
    #  Parse a file or string:
    #---------------------------------------------------------------------------
    
    def parse ( self, fn_or_str ):
        text = fn_or_str.split( '\n' )
        if len( text ) <= 1:
            fh = None
            try:
                fh   = open( fn_or_str, 'r' )
                text = fh.readlines()
                self.filename = fn_or_str
            finally:
                if fh is not None:
                    fh.close()
        self._parse( text, 0, -1 )
       
    #---------------------------------------------------------------------------
    #  Parse
    #---------------------------------------------------------------------------
    
    def _parse ( self, text, line, indent ):
        while True:
            if line >= len( text ):
                return line
            ctext = text[ line ].rstrip()
            stext = ctext.lstrip()
            if (stext == '') or (stext[:1] == '#'):
                line += 1
                continue
            cindent = len( ctext ) - len( stext )
            if cindent <= indent:
                return line
            special = False
            do_eval = True
            words   = stext.split( '=', 1 )
            if len( words ) <= 1:
                do_eval = False
                special = (stext[:1] == ':')
                if special:
                    stext = stext[1:]
                words = stext.split( ':', 1 )
            if len( words ) <= 1:
                raise TraitObjectError( "Missing ':' or '='", ctext )
            name  = words[0].strip()
            value = words[1].strip()
            if special:
                if (name != 'traits') or (name != 'contents'):
                    raise TraitObjectError( 
                             "Expected keyword 'traits' or 'contents'", ctext )
                if value == '':
                    raise TraitObjectError( 
                       "The 'traits' or 'contents' keyword must be followed by "
                       "[filename]['['object']']", ctext )
                parts    = value.split( '[', 1 )
                filename = parts[0].strip()
                if len( parts ) == 1:
                    object = TraitObject( filename )
                else:
                    object_name = parts[1].strip()
                    if object_name[-1:] == ']':
                        object_name = object_name[:-1].strip()
                    if filename == '':
                        filename = self.filename
                    container_object = TraitObject( filename )
                    for object in container_object.contents:
                        if object.name == object_name:
                            break
                    else:
                        raise TraitError( 
                                 "Could not find object '%s' in '%s'" % 
                                 ( object_name, filename ), ctext )
                if name == 'traits':
                    self.traits.update( object.traits )
                else:
                    self.contents.append( object.contents )
                continue
            if value != '':
                if do_eval:
                    try:
                        value = eval( value )
                    except:
                        raise TraitObjectError( 
                                 "Could not evaluate: '%s'" % value, ctext )
                self.traits[ name ] = value
                line += 1
            else:
                object      = TraitObject()
                object.name = name
                self.contents.append( object )
                line = object._parse( text, line + 1, cindent ) 

#-------------------------------------------------------------------------------
#  'TraitObjectError' class:
#-------------------------------------------------------------------------------

class TraitObjectError ( Exception ):
    
    #---------------------------------------------------------------------------
    #  Initialize the object:
    #---------------------------------------------------------------------------
    
    def __init__ ( self, msg, text = None ):
        if text is None:
            self.args = msg
        else:
            self.args = '%s in line: %s' % ( msg, text )
            
