""" The main charm plugin. """


# Enthought library imports.
from enthought.envisage.api import PluginDefinition

# Plugin definition imports (extension points etc).
from enthought.envisage.workbench.workbench_plugin_definition import \
     Branding, Perspective, View, Workbench

from enthought.plugins.python_shell.python_shell_plugin_definition import \
     Namespace


# The plugin's globally unique identifier (also used as the prefix for all
# identifiers defined in this module).
ID = 'enthought.charm.plugin.charm'


###############################################################################
# Extensions.
###############################################################################

#### Branding #################################################################

branding =  Branding(
    # The about box image.
    about_image = 'about.png',

    # The application icon.
    application_icon = 'application.ico',

    # The application name.
    application_name = 'Charm'
)

#### Namespace ################################################################

namespace = Namespace(
    commands = [
        'from scipy import *'
    ]
)

#### Workbench ################################################################

workbench = Workbench(
    views = [
        View(
            class_name  = ID + '.view.package_explorer.PackageExplorer',
            name        = 'Package Explorer',
            position    = 'left'
        ),

        # fixme: COmmented out because of a bug with SciPy and pickling...
##         View(
##             class_name  = ID + '.view.scipy_explorer.SciPyExplorer',
##             name        = 'SciPy Explorer',
##             position    = 'left'
##         ),
        
        View(
            class_name  = ID + '.view.outline.Outline',
            name        = 'Outline',
            position    = 'bottom'
        )
    ],
    
    perspectives = [
        Perspective(
            id = ID + '.perspective.charm',
            name = 'Charm',

            contents = [
##                 Perspective.Item(
##                     id = ID + '.view.scipy_explorer.SciPyExplorer',
##                     position = 'left'
##                 ),
                
                Perspective.Item(
                    id = ID + '.view.package_explorer.PackageExplorer',
                    position = 'left'
##                    position = 'with',
##                     relative_to = ID + '.view.scipy_explorer.SciPyExplorer'
                ),

                Perspective.Item(
                    id = ID + '.view.outline.Outline',
                    position = 'bottom',
                    relative_to = ID + '.view.package_explorer.PackageExplorer'
                ),
                
                Perspective.Item(
                    id = 'enthought.plugins.python_shell.view.PythonShellView',
                    position = 'bottom'
                )
            ]
        )
    ],

    default_perspective = ID + '.perspective.charm'
)

###############################################################################
# The plugin definition.
###############################################################################

class CharmPluginDefinition(PluginDefinition):
    """ The main charm plugin. """
    
    # The plugin's globally unique identifier.
    id = ID

    # The name of the class that implements the plugin.
    class_name = ID + '.plugin_implementation.CharmPlugin'

    # General information about the plugin.
    name          = 'Charm'
    version       = '1.0.1'
    provider_name = 'Enthought Inc'
    provider_url  = 'www.enthought.com'
    autostart     = True

    # The Id's of the plugins that this plugin requires.
    requires = [
        'enthought.envisage.workbench',
        'enthought.plugins.python_shell'
    ]

    # The extension points offered by this plugin.
    extension_points = []

    # The contributions that this plugin makes to extension points offered by
    # either itself or other plugins.
    extensions = [branding, namespace, workbench]

#### EOF ######################################################################
