""" The outline view. """


# Enthought library imports.
from enthought.charm.ui.outline_tree import OutlineTree
from enthought.envisage.workbench.api import View
from enthought.io.api import File

# Local imports.
from charm_view import CharmView


class Outline(CharmView):
    """ The outline view. """

    ###########################################################################
    # 'View' interface.
    ###########################################################################

    def create_control(self, parent):
        """ Creates the toolkit-specific control that represents the view.

        'parent' is the toolkit-specific control that is the view's parent.

        """
        
        self._outline_tree = tree = OutlineTree(parent, show_lines=False)
        tree.on_trait_change(self._on_node_activated, 'node_activated')
        tree.on_trait_change(self._on_selection_changed, 'selection')

        # Listen for changes to Charm's current module.
        self.charm.on_trait_change(self._on_module_changed, 'module')

        def l(self, new):
            print '888888888888888888', new

        self.window.on_trait_change(l, 'active_editor')
        
        return tree.control

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _select_node(self, node):
        """ Selects a node. """
        
        # Get the module that the node is in.
        module = self._get_module(node)

        # See if the module is being edited.
        editor = self.window.get_editor_by_resource(File(module.filename))
        if editor is not None:
            # Make sure the line is visible (and maybe center it).
            editor.center_line(node.lineno, force=False)
            
            # fixme: It seems we need '-1' here otherwise the STC shows the
            # line after the class definition (even though the line number is
            # definitely the line number that the class keyword appears on!).
            editor.select_line(node.lineno - 1)

        return

    def _get_module(self, node):
        """ Returns the module that the node is in. """

        namespace = node.namespace
        while not hasattr(namespace, 'filename'):
            namespace = namespace.namespace

        return namespace
    
    #### Trait change handlers ################################################

    def _on_module_changed(self, new):
        """ Dynamic trait change handler. """

        self._outline_tree.root = new

        return

    def _on_node_activated(self, tree, trait_name, old, new):
        """ Dynamic trait change handler. """

        # Edit Charm's current module.
        workbench = self.window.workbench
        workbench.edit(File(self.charm.module.filename))

        # A klass, method, trait or attribute...
        if hasattr(new, 'lineno'):
            self._select_node(new)

        return
    
    def _on_selection_changed(self, tree, trait_name, old, new):
        """ Called when a node in the outline tree is selected. """

        if len(new) > 0:
            # A klass, method, trait or attribute...
            if hasattr(new[0], 'lineno'):
                self._select_node(new[0])
            
        return

#### EOF ######################################################################
