""" The node type for an attribute. """


# Enthought library imports.
from enthought.charm.assign import Assign
from enthought.charm.klass import Klass
from enthought.pyface.api import ImageResource
from enthought.pyface.tree.api import NodeType


class AttributeNodeType(NodeType):
    """ The node type for an attribute. """

    #### 'NodeType' interface #################################################

    # The image used to represent nodes of this type.
    image = ImageResource('attribute')

    ###########################################################################
    # 'NodeType' interface.
    ###########################################################################

    def is_type_for(self, node):
        """ Returns True if this node type recognizes a node. """

        # fixme: We should have different node types for class and module
        # attributes.
        if isinstance(node, Assign):
            if isinstance(node.namespace, Klass):
                is_type_for = not node.is_trait

            else:
                is_type_for = True

        else:
            is_type_for = False
            
        return is_type_for

    def allows_children(self, node):
        """ Does the node allow children (ie. a folder vs a file). """

        return False

    def get_text(self, node):
        """ Returns the label text for a node. """

        return node.targets[0]

##### EOF #####################################################################
