""" The node type for a method. """


# Enthought library imports.
from enthought.charm.function import Function
from enthought.charm.klass import Klass
from enthought.pyface.api import ImageResource
from enthought.pyface.tree.api import NodeType


class MethodNodeType(NodeType):
    """ The node type for a method. """

    #### 'NodeType' interface #################################################

    # The image used to represent nodes of this type.
    image = ImageResource('method')

    ###########################################################################
    # 'NodeType' interface.
    ###########################################################################

    def is_type_for(self, node):
        """ Returns True if this node type recognizes a node. """

        return isinstance(node, Function) and isinstance(node.namespace, Klass)

    def allows_children(self, node):
        """ Does the node allow children (ie. a folder vs a file). """

        return False

    def get_text(self, node):
        """ Returns the label text for a node. """

        return node.name

##### EOF #####################################################################
