""" The node type for modules. """


# Enthought library imports.
from enthought.charm.module import Module
from enthought.pyface.api import ImageResource
from enthought.pyface.tree.api import NodeType
from enthought.traits.api import List


class ModuleNodeType(NodeType):
    """ The node type for modules. """

    #### 'NodeType' interface #################################################

    # The image used to represent nodes of this type.
    image = ImageResource('module')

    #### 'ModuleNodeType' interface ###########################################

    # The names of the module's traits to show as its children.
    include = List(['attributes', 'klasses', 'functions'])

    ###########################################################################
    # 'NodeType' interface.
    ###########################################################################

    def is_type_for(self, node):
        """ Returns True if this node type recognizes a node. """

        return isinstance(node, Module)

    def allows_children(self, node):
        """ Does the node allow children (ie. a folder vs a file). """

        return len(self.include) > 0
    
    def has_children(self, node):
        """ Returns True if a node has children, otherwise False. """

        for trait_name in self.include:
            if len(getattr(node, trait_name)) > 0:
                has_children = True
                break

        else:
            has_children = False
        
        return has_children

    def get_children(self, node):
        """ Returns the children of a node. """

        children = []
        for trait_name in self.include:
            value = getattr(node, trait_name)
            
            keys = value.keys()
            keys.sort()

            for key in keys:
                children.append(value[key])
            
        return children

    def get_text(self, node):
        """ Returns the label text for a node. """

        components = node.name.split('.')
        
        return components[-1]

    def set_text(self, node, text):
        """ Sets the label text for a node. """

        components = text.split('.')

        import os

        old = node.filename
        basename, filename = os.path.split(node.filename)

        node.name = text
        node.filename = os.path.join(basename, components[-1] + '.py')

        from enthought.io import File

        o = File(old)
        n = File(node.filename)

        o.move(n)
        
        return

    def is_editable(self, node):
        """ Returns True if the node is editable, otherwise False.

        If the node is editable, its text can be set via the UI.

        """

        return True

##### EOF #####################################################################
