#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
# 
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
# 
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" A tree that displays the contents of a package/module/class etc. """


# Enthought library imports.
from enthought.pyface.tree.api import NodeManager, NodeTree, NodeTreeModel

# Local imports.
from attribute_node_type import AttributeNodeType
from function_node_type import FunctionNodeType
from klass_node_type import KlassNodeType
from module_node_type import ModuleNodeType
from package_node_type import PackageNodeType
from trait_node_type import TraitNodeType


class OutlineTree(NodeTree):
    """ A tree that displays the contents of a package/module/class etc. """

    ###########################################################################
    # 'Tree' interface.
    ###########################################################################

    #### Trait initializers ###################################################

    def _model_default(self):
        """ Initializes the model trait. """

        node_manager = NodeManager(
            node_types = [
                AttributeNodeType(),
                FunctionNodeType(),
                KlassNodeType(),
                ModuleNodeType(),
                PackageNodeType(),
                TraitNodeType()
            ]
        )

        return NodeTreeModel(node_manager=node_manager)

    #### Trait change handlers ################################################

    def _node_right_clicked_changed(self, (obj, point)):
        """ Called when the right mouse button is clicked on the tree. """

        # Overridden to ingnore the experimental implementation of context
        # menus in 'NodeType' etc.
        
        # Add the node that the right-click occurred on to the selection.
        self.select(obj)

        return None
    
##### EOF #####################################################################
