""" The node type for packages. """


# Enthought library imports.
from enthought.charm.package import Package
from enthought.pyface.api import ImageResource
from enthought.pyface.action.api import Action, Group, MenuManager
from enthought.pyface.tree.api import NodeType

# Local imports.
from action.new_package import NewPackage
from action.new_module import NewModule


class PackageNodeType(NodeType):
    """ The node type for packages. """

    #### 'NodeType' interface #################################################

    # The image used to represent nodes of this type.
    image = ImageResource('package')

    # The default actions/groups/menus available on nodes of this type.
    actions = [
        Action(name="aaa"),
        Action(name="bbb"),

        Group(
            Action(name="xxx"),
            Action(name="yyy"),
            separator=True,
            id="bazzle"
        ),

        Action(name="Do something"),
        Action(name="Do something else"),

        
        MenuManager(
            Action(name="Foo"),
            Action(name="Bar"),
        
            name = "Baz"
        )
    ]

    # The default actions/groups/menus for creating new children within nodes
    # of this type.
    new_actions = [
        NewPackage(),
        NewModule(),
    ]

    ###########################################################################
    # 'NodeType' interface.
    ###########################################################################

    def is_type_for(self, node):
        """ Returns True if this node type recognizes a node. """

        return isinstance(node, Package)

    def allows_children(self, node):
        """ Does the node allow children (ie. a folder vs a file). """

        return True

    def has_children(self, node):
        """ Returns True if a node has children, otherwise False. """

        return len(node.contents) > 0

    def get_children(self, node):
        """ Returns the children of a node. """

        return node.contents

    def get_text(self, node):
        """ Returns the label text for a node. """

        return node.name

    def set_text(self, node, text):
        """ Sets the label text for a node. """

        components = text.split('.')

        import os

        old = node.filename
        basename, filename = os.path.split(node.filename)

        node.name = text
        node.filename = os.path.join(basename, components[-1])

        from enthought.io import File

        o = File(old)
        n = File(node.filename)

        o.move(n)
        
        
        return

    def is_editable(self, node):
        """ Returns True if the node is editable, otherwise False.

        If the node is editable, its text can be set via the UI.

        """

        return True

##### EOF #####################################################################
