#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
# 
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
# 
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" A tree that displays a parsed package hierarchy. """


# Enthought library imports.
from enthought.pyface.action.api import Action
from enthought.pyface.tree.api import NodeManager, NodeTree, NodeTreeModel
from enthought.traits.api import Instance

# Local imports.
from attribute_node_type import AttributeNodeType
from function_node_type import FunctionNodeType
from klass_node_type import KlassNodeType
from method_node_type import MethodNodeType
from module_node_type import ModuleNodeType
from package_node_type import PackageNodeType
from package_tree_sorter import PackageTreeSorter
from trait_node_type import TraitNodeType


class PackageTree(NodeTree):
    """ A tree that displays a parsed package hierarchy. """

    #### 'Tree' interface #####################################################

    # The tree's sorter (None if no sorting is required).
    sorter = Instance(PackageTreeSorter, ())

    ###########################################################################
    # 'Tree' interface.
    ###########################################################################

    #### Trait initializers ###################################################

    def _model_default(self):
        """ Initializes the model trait. """
            
        node_manager = NodeManager(
            node_types = [
                AttributeNodeType(),
                FunctionNodeType(),
                KlassNodeType(include=[]),
                ModuleNodeType(include=[]),
                PackageNodeType(),
                TraitNodeType()
            ],

            system_actions = [
                Action(name='Cut'),
                Action(name='Copy'),
                Action(name='Paste'),
                Action(name='Delete'),
                Action(name='Rename'),
            ]
        )

        return NodeTreeModel(node_manager=node_manager)

    #### Trait change handlers ################################################

    def _node_right_clicked_changed(self, (obj, point)):
        """ Called when the right mouse button is clicked on the tree. """

        # Overridden to ingnore the experimental implementation of context
        # menus in 'NodeType' etc.
        
        # Add the node that the right-click occurred on to the selection.
        self.select(obj)

        return None
    
##### EOF #####################################################################
