""" Base-class for all components. """


# Enthought library imports.
from enthought.traits.api import HasTraits, Str, Trait, TraitDict


class Component(HasTraits):
    """ Base-class for all components. """

    # The unique id of the component.
    id = Str

    # The name of the component.
    name = Str

    # A short description of the component.
    description = Str
 
    # Arbitrary meta-data to further describe the component.
    metadata = Trait({}, TraitDict())

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, **kw):
        """ Creates a new component. """

        # Split the traits and the metadata!
        traits = {}
        metadata = {}
        trait_names = self.trait_names()
        for key,value in kw.items():
            if key in trait_names:
                traits[key] = value
            else:
                metadata[key] = value
        #traits, metadata = self.split_traits(kw)
        
        # Base-class constructor.
        super(Component, self).__init__(metadata=metadata, **traits)
        
        return

    def __str__(self):
        """ Returns an 'informal' representation of the object. """

        return self.name
        
#### EOF ######################################################################
