import types

import wx
from wxPython.gizmos import wxTreeListCtrl

from enthought.envisage.node_tree import NodeTree
from enthought.envisage.node_tree_model import NodeTreeModel

def on_user_namespace_changed(application, namespace):
    """ Called to display a new version of the namespace ... """
    
    dlg = NamespaceView(None, namespace)
    result = dlg.ShowModal()
    if result == wx.CANCEL:
        print 'continuing'
    
    dlg.Destroy()

    return


class NamespaceView(wx.Dialog):

    def __init__(self, parent, namespace, help_id=None):
        
        wx.Dialog.__init__(self, parent, -1, "Namespace Browser", 
            size=wx.Size(300,500), 
            style=wx.RESIZE_BORDER | wx.CAPTION | wx.DEFAULT_DIALOG_STYLE)
            
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        
        label = wx.StaticText(self, -1, 
            "Execution suspended. Click Continue button to resume execution."
        )
        sizer.Add(label, 0, wx.ALL, border=10)
        
        # Add the main panel 
        panel = self._create_viewer(self, namespace)
        sizer.Add(panel, 1, wx.EXPAND|wx.ALL, border=10)
        
        # Is the 'Help' button required?
        self._help_id = help_id
        
        # Add the buttons.
        buttons = self._create_buttons(self)
        sizer.AddSizer(buttons, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
        
        # Sizes the window so that it fits around its subwindows
        sizer.Fit(self) 
        
        self.CenterOnScreen()
        return
        
        
    def _create_viewer(self, parent, namespace):
        
        model = NodeTreeModel(name="Root")
        namespace_view=NodeTree(parent, -1, model=model)
    
        root = model.root
    
        del root.children[:]
        for name, value in namespace.items():
            if name[0] != '_':
                node = self._create_node(name, value)
                root.append(node)

        model.structure_changed(root)
        
        return namespace_view

        
    def _create_node(self, name, value):

        import types
        from enthought.envisage.node            import Node
        from enthought.traits.api                   import HasTraits
        from enthought.envisage.has_traits_node import HasTraitsNode
        
        from cp.log import Log, LogIndex, LogSuite
        from cp.layer import Column
        from proava2.node.column_node import ColumnNode
        from proava2.node.log_node import LogNode
        from proava2.node.log_index_node import LogIndexNode
        from proava2.node.log_suite_node import LogSuiteNode
        
        if name == "namespace":
              node = Node(name=name, allows_children=False)
              
        elif type(value) is types.InstanceType:
            if isinstance(value, HasTraits):
                if isinstance(value, Log):
                    node = LogNode(value)

                elif isinstance(value, LogSuite):
                    node = LogSuiteNode(value)

                elif isinstance(value, LogIndex):
                    node = LogIndexNode(value)

                elif isinstance(value, Column):
                    node = ColumnNode(value)
                    
                else:
                    node = HasTraitsNode(value, name)
                    
            else:
                node = Node(name=name, allows_children=True)
 
                for n, v in value.__dict__.items():
                    if not n.startswith('_'):
                         # fixme: This only prevents circular references to
                         # self.
                        if type(v) is types.InstanceType and v is value:
                                continue
    
                        node.append(self._create_node(n, v))
                        
        else:
            node = Node(name=name, allows_children=False)
            
    
        return node
        
        
    def _create_buttons(self, parent):
        """ Creates the buttons. """

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        #ok = wx.Button(parent, wx.ID_OK, "Continue")
        #sizer.Add(ok, 0, wx.ALL, 5)
        #wx.EVT_BUTTON(parent, wx.ID_OK, self._on_ok)
        
        cancel = wx.Button(parent, wx.ID_CANCEL, "Continue")
        sizer.Add(cancel, 0, wx.ALL, 5)

        if self._help_id:
            helpbtn = wx.Button(parent, wx.ID_HELP, "Help")
            sizer.Add(helpbtn, 0, wx.ALL, 5)

            # Wire it up.
            wx.EVT_BUTTON(parent, wx.ID_HELP, self._on_help)

        cancel.SetDefault()
        
        return sizer
        
    def _on_ok(self, event):
        """ Called when the 'OK' button is pressed. """
    
        return    

    def _on_help(self, event):
        """Called when the 'Help' button is pressed. """

        from enthought.envisage.application import get_application
        get_application().proava2.helpviewer.view_id(self._help_id)
        
        return
        
### EOF ########################################################################
