""" A file system decorator that manages script components. """


# Standard library imports.
import os

# Enthought library imports.
from enthought.envisage.file_system import LocalFileSystem
from enthought.traits.api import Trait, TraitInstance

# Local imports.
from script_component_factory import ScriptComponentFactory


class ScriptComponentFileSystem(LocalFileSystem):
    """ A file system decorator that manages script components. """

    #########################################################################
    # 'object' interface.
    #########################################################################


    #########################################################################
    # 'FileSystem' interface.
    #########################################################################

    def refresh(self):
        """ Check for external modifications to this file system. """

        self.root.refresh()

        # Trait event notification.
        self.refreshed = True
        
        return

    #########################################################################
    # 'ScriptComponentFileSystem' interface.
    #########################################################################

    def register_components(self):
        """ Registers all script components in the file system. """

        scf = ScriptComponentFactory()
        
        # Walk the file system looking for Python files.
        for root, dirs, files in os.walk(self.root.path):
            for name in files:
                if name.endswith('.py'):
                    path = os.path.join(root, name)
                    script_component = scf.from_file(path)
                    
        return

    def unregister_components(self):
        """ Unregisters all script components in the file system. """

        from enthought.component.registry import registry
        scf = ScriptComponentFactory()
        
        # Walk the file system looking for Python files.
        for root, dirs, files in os.walk(self.root.path):
            for name in files:
                if name.endswith('.py'):
                    path = os.path.join(root, name)
                    script_component = scf.from_file(path)
                    script_component.unregister()

        return
    
#### EOF ####################################################################

