# Standard library imports
import logging

# Enthought package imports
from enthought.component.function_binding import FunctionBinding
from enthought.traits.api import Bool, HasTraits, Instance, List, Property, Str
from enthought.traits.ui.api import CustomEditor, Group, Item, View
from enthought.traits.ui.menu import OKButton, CancelButton, HelpButton

# Local imports
from serial_workflow_editor import WorkflowListEditor
from exception import AbortSerialWorkflow


# Setup a logger for this module
logger = logging.getLogger(__name__)


class SerialWorkflow(HasTraits):
    """ A class that executes a list of FunctionBindings within a namespace.
    """

    name = Str('')

    operations = List(Instance(FunctionBinding))

    enabled = Bool(True)

    query = Str('')

    view = View(Group(Item(name = 'name'),
                      Group(Item(name = 'operations',
                                 editor = WorkflowListEditor(),
                                 width = 350,
                                 height = 300),
                            show_labels = False
                            )
                      ),
                title='Workflow Properties',
                buttons = [OKButton, CancelButton, HelpButton],
                resizable = True,
                help_id = 'enlib|HID_Workflow_Properties_Dlg'
                )

    _workflow = Property

    def __init__(self, name='', operations=[], query='', **traits):

        traits['name'] = name
        traits['operations'] = operations
        traits['query'] = query

        # base class constructor
        super(SerialWorkflow, self).__init__(**traits)

        return

    def __str__(self):
        return self.name

    def clone(self):

        clone = self.clone_traits(traits = ['name', 'enabled', 'query'])

        # do a manual deep clone of the operations
        clone.operations = [op.clone() for op in self.operations]

        return clone

    ###########################################################################
    # SerialWorkflow interface
    ###########################################################################

    def run(self, namespace):

        try:
            for operation in self.operations:
                if operation.enabled:
                    #logger.info('SerialWorkflow running operation: ' + operation.function.name)
                    operation.run(namespace)

        # intercept a component legitimately cancelling the workflow execution
        except AbortSerialWorkflow, msg:
            # reraise the error so that reporting shows us where in the workflow
            # the exception occured.  This is helpful for debugging.
            raise

        except Exception, msg:
            logger.exception(msg)
            raise Exception, msg

        return

    def _get__workflow(self):
        # h&ck to make traits ui give us the right thang
        return self

#### EOF ######################################################################
