""" An (extremely) simple type definition manager. """


# Enthought library imports.
from enthought.traits.api import HasPrivateTraits, Property


class TypeDefManager(HasPrivateTraits):
    """ An (extremely) simple type definition manager.

    Basically, the type definition manager is a simple repository of type
    definitions.  It provides a way to add type definitions to the registry and
    then look them up!

    """

    
    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self):
        """ Creates a new type definition manager. """

        self._type_defs = {} # { string type_id : TypeDef type_def }
        
        return

    ###########################################################################
    # 'TypeDefManager' interface.
    ###########################################################################

    #### Trait properties #####################################################
    
    def _get_type_defs(self):
        """ Returns all of the type definitions in the manager. """

        return self._type_defs.values()

    type_defs = Property(_get_type_defs)
    
    #### Methods ##############################################################
    
    def add(self, type_def):
        """ Add the specified type definition to the manager. """

        if self._type_defs.has_key(type_def.id):
            raise ValueError, "Duplicate type definition %s" % type_def.id
        
        self._type_defs[type_def.id] = type_def

        return

    def lookup(self, id):
        """ Lookup the type definition with the specified id. """

        return self._type_defs.get(id)

# The (as yet unenforced) singleton instance of the type manager.
type_def_manager = TypeDefManager()

#### EOF ######################################################################
