""" A demo plugin. """

import os

# Enthought library imports.
from enthought.envisage import Plugin

from enthought.traits.api import List

from enthought.envisage import get_application

from enthought.graph.vpl.graph_editor_plugin_definition import ActorClassLocations

from enthought.logger import logger

from enthought.graph.chips.view.chip_view import ChipView

class ChipResourcePlugin(Plugin):
    """ A python resource plugin. """

    # The shared plugin instance.
    instance = None
    
    # Paths to actions
    action_path = None
    
    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, **traits):
        """ Creates a new plugin. """

        # Base-class constructor.
        super(ChipResourcePlugin, self).__init__(**traits)

        # Set the shared instance.
        ChipResourcePlugin.instance = self
        
        return

    ###########################################################################
    # 'Plugin' interface.
    ###########################################################################

    def start(self, application):
        """ Starts the plugin.

        Can be called manually, but is usually called exactly once when the
        plugin is first required.

        """
        
        
        self._get_file_paths()

        pass
    
    def stop(self, application):
        """ Stops the plugin.

        Can be called manually, but is usually called exactly once when the
        application exits.

        """

        pass
        
        
### Protected Methods #########################################################

    def _get_file_paths(self):
        """ Get the paths to the chip files to load.
        """
        
        id = ActorClassLocations
        
        extensions = get_application().extension_registry.get_extensions(id)
        
        paths = []
        
        for actorClassLocations in extensions:
            for actorClassLocation in actorClassLocations.class_locations:
                
                # actorClassLocation specifies a path to load chips from.
                if (actorClassLocation.path is not None
                    and actorClassLocation.path != '' ):
                    
                    absolute_path = self._find_abs_path(actorClassLocation.path)
                    
                    if absolute_path is not None:
                        paths.append(absolute_path)
                        
        self.action_paths = paths
#        chip_view = ChipView.instance
#        chip_view.add_paths(paths)
         
        return
                         
    def _find_abs_path(self, relative_path):
        """ Find the absolute path for the given relative path.
        """
        
        absolute_path = None
        
        module_name = relative_path.replace('/', '.')
        module_name = module_name.replace('\\', '.')
                
        try:
            # find the module for the given path.
            module = __import__(module_name, globals(), locals(), [''])
            
            # remove init.pyc from the end
            absolute_path = os.path.split(module.__file__)[0]
        except:
            logger.exception('Unable to find absolute path for ' + relative_path)
        
        return absolute_path
                     

#### EOF ######################################################################
