""" Demo plugin definition. """


# Plugin extension-point imports.
from enthought.envisage.core.core_plugin_definition \
     import PluginDefinition

from enthought.envisage.project.project_plugin_definition \
     import ResourceType, ResourceTypes, ResourceAction, ResourceActions

from enthought.envisage.project.project_plugin_definition \
     import CookieImplementation, CookieImplementations
     
from enthought.envisage.ui.ui_plugin_definition \
    import EnabledWhen
    
from enthought.envisage.ui.ui_plugin_definition import UIViews, View


# The plugin's globally unique identifier (also used as the prefix for all
# identifiers defined in this module).
ID = "enthought.graph.chips"


###############################################################################
# Extensions.
###############################################################################

#### Resource types ###########################################################

CHIP_RESOURCE_TYPE = \
    ID + ".resource_type.chip_resource_type.ChipResourceType"

resource_types = ResourceTypes(
    resource_types = [
        ResourceType(
            class_name = CHIP_RESOURCE_TYPE,
            
            precedes = [
                "enthought.envisage.resource.file_resource_type" \
                ".FileResourceType"
            ]
        )
    ]
)

#### Cookies ##################################################################

cookies = CookieImplementations(
    implementations = [
        CookieImplementation(
            resource_type = CHIP_RESOURCE_TYPE,

            cookie_interface = \
                "enthought.envisage.project.action.open_cookie.OpenCookie",

            cookie_implementation = ID + \
                ".resource_type.chip_resource_open_cookie" \
                ".ChipResourceOpenCookie"
        ),
    ]
)

#### Views ####################################################################

ui_views = UIViews(
    views = [
        View(
            id = ID + ".view.ChipView",
            class_name = ID + ".view.ChipView",
            name = "Chips",
            position = "left"
        )
    ]
)

###############################################################################
# The plugin definition!
###############################################################################

PluginDefinition(
    # The plugin's globally unique identifier.
    id = ID,

    # The name of the class that implements the plugin.
    class_name = ID  + ".chip_resource_plugin.ChipResourcePlugin",

    # General information about the plugin.
    name          = "Chip Resource Plugin",
    version       = "1.0.0",
    provider_name = "Enthought Inc",
    provider_url  = "www.enthought.com",
    enabled       = True,
    autostart     = True,

    # The Id's of the plugins that this plugin requires.
    requires = [
        "enthought.envisage.ui",
        "enthought.envisage.project",
        "enthought.graph.vpl"
    ],

    # The extension points offered by this plugin,
    extension_points = [],
    
    # The contributions that this plugin makes to extension points offered by
    # either itself or other plugins.
    extensions = [resource_types, cookies] #, ui_views]
)

#### EOF ######################################################################
