""" A script editor. """

import os

# Enthought library imports.
from enthought.envisage.ui import Editor
from enthought.traits.api import Any, Str
from enthought.pyface.api import confirm, YES 

# Local imports.
from port_editor import PortEditor


class ChipEditor(Editor):
    """ A chip editor. """
 
    # The port editor.
    port_editor = Any
    
    # The pathname of the script being edited.
    path = Str    

    # The resource being edited.
    resource = Any
    
    ###########################################################################
    # Protected 'Window' interface.
    ###########################################################################

    def _create_contents(self, parent):
        """ Creates the window contents. """
        
        return self._create_port_editor(parent)
                        
    ###########################################################################
    # Private interface.
    ###########################################################################
 
    def _create_port_editor(self, parent):
        """ Create the port editor.
        """
          
        self.port_editor = PortEditor(resource = self.resource)
        
        self.port_editor.on_trait_change(self._on_dirty_changed, 'dirty')
        
        self._panel = self.port_editor.edit_traits(parent = parent, kind = 'panel')
        
        self.title = os.path.split(self.path)[1]
                
        return self._panel.control
        

    def _closing_changed(self, event):
        """ Handle the window closing. 
        """
        if self.port_editor.dirty:
            result = confirm(self.control, 'Do you want to save your changes?', '')
            
            if result == YES:
                self.port_editor._save()
                
        self.port_editor.on_trait_change(self._on_dirty_changed, 'dirty', remove = True)
        
        self._panel.dispose()
        
        return
        
    def _on_dirty_changed(self):
        """ Called when dirty indicator on port editor changes
        """
        
        file_name = os.path.split(self.path)[1]
        
        if self.port_editor.dirty:
            self.title = file_name + '*'
        else:
            self.title = file_name
        
        return
              
#### EOF ######################################################################
