import re

from string import capwords

from enthought.traits.api import HasTraits, Str, List, Dict

class FunctionArgumentInformation(HasTraits):
    
    # The name of the function
    function_name = Str
    
    # suggested_chip_name
    suggested_chip_name = Str
    
    # The input arguments
    input_arguments = List
    
    # The output arguments
    output_arguments = List
    
    # Default values for input arguments
    input_defaults = Dict
    
    def __init__(self, function_text, **traits):
        """ construct new FunctionArgumentInformation object """
        
        super(FunctionArgumentInformation, self).__init__(**traits)
        
        self._set_function_name_and_arguments(function_text)
        self._set_function_return_arguments(function_text)
        
        self.suggested_chip_name = self._convert_function_name(self.function_name)
        
        return
    
    
    ###########################################################################
    # Private interface.
    ###########################################################################
    
    def _set_function_name_and_arguments(self, text):
        """ set function name and the function arguments
        """
        
        # TODO this regex and others in this file are too simplistic to work in all cases.
        # In particular function definition with intermixed comments will be problematic, as well
        # as default arguments containing ')' and a ':' etc.
        doc_pat = re.compile('def(\s+)(\S+)(\s*)\((.*?)\)(\s*)\:', re.DOTALL)
        
        match = doc_pat.search(text)
        
        arguments = []
        defaults = {}
        
        name = ''
        
        if match is not None:
            
            name = match.group(2)
            
            arg_text = match.group(4)
            
            arg_arr = arg_text.split(',')
            
            value = None
            
            for arg in arg_arr:
                # argument could be assigned a value
                equal_index = arg.find('=')
                
                # strip out the value
                if equal_index != -1:
                    value = self._get_value(arg, equal_index)
                    arg = arg[:equal_index]
                    
                # arument could appear on a new line after a comment.
                newline_index = arg.find('\n')
                
                # strip out the comment.
                if newline_index != -1:
                    after_newline = arg[newline_index:]
                    after_newline = after_newline.lstrip().rstrip()
                    
                    if len(after_newline) > 0:
                        arg = after_newline
                
                arg = arg.lstrip().rstrip()
                
                if arg != '':
                    arguments.append(arg)
                    
                    if value is not None:
                        defaults[arg] = value
                        value = None
                        
        self.function_name = name
        self.input_arguments = arguments
        self.input_defaults = defaults
                            
        return
        
    def _set_function_return_arguments(self, text):
        """ Set arguments being returned by the function.
        """
        
        doc_pat = re.compile('return(\s+)([^\n]*)', re.DOTALL)
        
        match = doc_pat.search(text)
        
        arguments = []
        
        if match is not None:
            
            arg_text = match.group(2)
            
            arg_text = arg_text.replace('(', '')
            arg_text = arg_text.replace(')', '')
            
            arg_text = arg_text.replace('[', '')
            arg_text = arg_text.replace(']', '')
            
            arg_arr = arg_text.split(',')
        
            for arg in arg_arr:                
                arg = arg.lstrip().rstrip()
                
                if arg != '':
                    arguments.append(arg)
      
        self.output_arguments = arguments
            
        return
        
    def _convert_function_name(self, function_name):
        """ For display purposes, remove underscores from function name and capitalize words.
        """
        
        return capwords(function_name.replace('_', ' '))
        
    def _get_value(self, arg, equal_index):
        """ Get the default value for argument after the equal sign. 
        """
        
        value = arg[equal_index + 1:]
        
        value = value.replace("'", '')
        value = value.replace('"', '')
        
        value = value.lstrip().rstrip()
        
        return value
        
#### EOF ######################################################################
