import re

from string import capwords

from enthought.graph.function.metadata_parser import MetadataParser
from enthought.graph.function.metadata_elements import FunctionElement
from enthought.graph.function.xml_metadata_writer import XmlMetadataWriter

from function_argument_information import FunctionArgumentInformation

class XmlScriptUtil:
    """ Helper to create xml information from function element and vice versa. 
    """ 

    ###########################################################################
    # Public interface.
    ###########################################################################

    def get_function_element(self, text):
        """ Get the function element from the function text.
            
            return tuple with the function element and the function text separately.
        """
        
        function_info = self._get_xml_function_text_tuple(text)
        
        xml = function_info[0]
        
        if xml is not None:
            metadataParser = MetadataParser()
            function_element = metadataParser.elements_from_string('function', xml)
            function_element = function_element[0]
        else:
            function_element = self._create_empty_function_element()
        
        return (function_element, function_info[1])
        
    def put_xml_info_from_element(self, text, function_element):
        """ generate the xml information from the element and put it back in the function text. 
        """
        
        xmlMetadataWriter = XmlMetadataWriter()
        
        # if this the function element is empty, don't put 
        # the metadata representation in the doc string.
        if self.check_empty_function_element(function_element):
            return text
        
        function_doc_string = xmlMetadataWriter.write(function_element)
        
        return self._put_xml_info(text, function_doc_string)
        
    def check_empty_function_element(self, function_element):
        """ Check to see if the function element is newly created with no bundles defined.
        """
        
        input_args = function_element.input_args
        output_args = function_element.output_args
        
        no_input_args = input_args is None or len(input_args) == 0
        no_output_args = output_args is None or len(output_args) == 0
        
        if no_input_args and no_output_args:
            return True
            
        return False 
            
    ###########################################################################
    # Private interface.
    ###########################################################################
    
    def _get_xml_function_text_tuple(self, text):
        """ Get tuple with the xml and function text.
        
        """
        
        func_pat = re.compile( r'<function(.*?)</function>', re.DOTALL)
        
        match = func_pat.search(text)
        
        function_text = text
        
        if match is not None:
            xml = match.group()
            
            end_index = function_text.find('from enthought.graph.vpl.chip_manager import ChipManager')
            
            
            if end_index != -1:
                function_text = function_text[:end_index]
            
        else:
            xml = None
        
        return (xml, function_text)
        
    def _put_xml_info(self, text, xml_info):
        """ put the xml info back into the function text. 
        """
        
        doc_pat = re.compile( 'ChipManager\(\)\.register(.*?)(\'\'\'|""")(.*?)(\'\'\'|""")', re.DOTALL)
        
        match = doc_pat.search(text)
          
        retVal = text
        
        if match is not None:
            doc = match.group()
            
            start = text[:match.end() - 3]
            end = text[match.end() - 3:]
                
            start = start.rstrip()
            start += '\n\n'
               
            function_doc_string += '\n    '
             
            retVal = start + xml_info + end
        else:
            
            function_information = FunctionArgumentInformation(text)
            
            retVal += '\n\nfrom enthought.graph.vpl.chip_manager import ChipManager'
            retVal += '\nChipManager().register(' + function_information.function_name + ', '
            retVal += '\n    """\n' + xml_info + '\n    """\n)' 
                    
        return retVal
                
    def _create_empty_function_element(self):
        """ Create a function element with no bundles.
        """
        
        function_element = FunctionElement()
        function_element.name = 'name'
        function_element.category = 'My Chips'
        
        return function_element
            
#### EOF ######################################################################
