""" Graph exceptions. """

import traceback
import sys

class GraphError(Exception):
    """ Base class for all graph exceptions. """
    pass

class ExecutionError( GraphError ) :
    """ Exception raised when a dataflow fails to execute.
    Usually this means an actor's prefire or postfire method returned False.
    """
    pass

class InsufficientInputError( GraphError ) :
    """ Exception raised when a dataflow cannot execute because a required
    input port is not connected or has no available tokens.  I.e. prefire
    failed.
    """
    pass

class OutputError( GraphError ) :
    """ Exception raised when a dataflow actor postfire failed. """
    pass
    
class FireError( GraphError ):
    """ Exception raised the fire method content raises an exception. """
    
    def __init__( self, actor, exception ):
        self.actor = actor
        self.exception = exception

        t, v, tb = sys.exc_info()
        self.etype = t
        self.value = v
        self.tb = tb
        return
    
    def __str__( self ):
        s = ''.join(traceback.format_exception(self.etype, self.value, self.tb))
        s += "FireError: Actor '%s' caused previous traceback.\n" \
            % self.actor.full_name
        return s
            
            
#### EOF ######################################################################

