from enthought.logger import logger

from enthought.traits.api import HasTraits
from enthought.traits.api import List

from receiver import Receiver

class FifoReceiver( Receiver ) :
    """ Implments a Receiver with first-in-first-out semantics. """
    
    # The tokens currently stored in this receiver
    fifo = List
    

    def __init__(self, *args, **kw) :
        """ Create a new Receiver. """
        Receiver.__init__(self, *args, **kw)
    
    
    #### Receiver Methods ####################################################

    def clear( self ) :
        """ Clear this receiver of any contained tokens. """
        self.fifo = []

        
    def available_tokens( self ) :
        """ Return the number of tokens available for get to return."""
        return len(self.fifo)

    def has_tokens( self, n=1 ) :
        """ Return True if the receiver has n tokens for get to return. """
        return len(self.fifo) >= n
    
    def get( self, n=1 ) :
        """ Return a token or list of tokens, when n > 1, from this receiver.
        """
        if n > len(self.fifo) :
            raise ValueError, "Insufficient data in receiver"
        
        tokens = self.fifo[:n]
        del self.fifo[:n]

        if n == 1 :
            tokens = tokens[0]
            
#        logger.debug( "FifoReceiver.get: %s" % tokens )
        return tokens

        
    def has_room( self, n=1 ) :
        """ Return True if the receiver can accept n inputs. 
        This implementation uses an unbounded FIFO.
        """
        return True

    def put( self, token ) :
        """ Put a token into this recevier.
        """
#        logger.debug( "FifoReceiver.put: %s" % token )
        
        self.fifo.append( token )
        
        return    
        
    
#### EOF ######################################################################    
