""" Service that manages default values for metadata elements.  The defaults
are identified by their tag type and name value.
"""

from enthought.traits.api import HasTraits, Dict
from xml_element_names import *

from metadata_parser import MetadataParser

class MetadataDefaults( HasTraits ):
    """ Service that manages default values for metadata elements.  The defaults
    are identified by their tag type and name value.
    """
    
    # Metadata element tag names are the keys.  Values are dictionaries where
    # the key is the name attribute value and the values are NodeElement
    # instances.  Most common element tag is 'arg'.
    defaults = Dict
    
    def bind(self, element):
        """ Add the specified element as the template for elements of the same
        tag and name attribute.
        """
        tag_name = element.tag_name
        
        if hasattr( element, 'composite_name' ):
            name = element.composite_name
        else:
            name = element.name
        
        if tag_name is None or tag_name == '':
            raise ValueError, "Invalid tag_name"
            
        if name is None:
            raise ValueError, "Invalid name for %s tag: None" % tag_name

        if self.defaults.has_key( tag_name ):
           by_name = self.defaults[ tag_name ]

        else:
            by_name = {}
            self.defaults[ tag_name ] = by_name
            
        by_name[ name ] = element
        
        return

    def lookup(self, tag_name, name, exact_match = False ):
        """ Returns a new element with default values for all attributes for
        the specified tag_name and name.
        """
        # Lookup by complete name.
        by_name = self.defaults.get( tag_name, {} )
        element = by_name.get( name, None )
        
        if not exact_match:
            # For args, lookup by progressively stripping off suffixes.
            if element is None and tag_name == K_ARG_TAG:
                n = name.split('_')
                if len(n) > 1:
                    n = '_'.join(n[:-1])
                    element = self.lookup( tag_name, n )
           
             
            # If there is still no match, lookup the true default.
            if element is None:
                element = by_name.get( '', None )

        # Produce the copy to be returned.
        if element is not None:
            # The element may have a shorter name that we are looking for.
            # Copy it and replace the name (e.g. 'k' when looking for 'k_wat_sh'.)
            element = element.clone_traits( traits='all' )
            
            if tag_name == K_ARG_TAG:
                element.composite_name = name
                element.arg_name = name

            else:
                element.name = name
        
        return element

    def unbind(self, tag_name, name):
        """ Remove the specified default. """
        if self.defaults.has_key( tag_name ):
            by_name = self.defaults[ tag_name ]
            if by_name.has_key( name ):
                del by_name[ name ]
        
        return

    def load_from_filename(self, filename):
        """ from the specified filename load defaults """
        parser = MetadataParser()
        elements = parser.elements_from_file( 'default', filename )
        
        for default_element in elements:
            for arg in default_element.default_args:
                self.bind( arg )
        return
        
### EOF #######################################################################
