# Run within CPLab
from enthought.traits.api import HasTraits, Dict, List, Instance

from enthought.graph.function.metadata_elements import ArgumentElement, DefaultElement
from enthought.graph.vpl.chip_manager import ChipManager
from enthought.graph.vpl.graph_actor_folder import GraphActorFolder
from enthought.graph.function import MetadataDefaults

class NonconformantArgs( HasTraits ):

    # key is port.arg_name, value is List(ConvergePort)
    args_by_name = Dict

    cm = Any
    
    md = Any
    
    arg_attr_names = List([ #'arg_name', # 'name',
         'modifier', 'bundle', 'composite_name',
         'units', 'units_system_name', 'family_name',
         'external_type', 'internal_type', 
         'default', 
         'is_optional', 'repeat_group', ])

    arg_attr_print_names = List([ 'actor_name', 'arg_name', 'name', 'modifier', 'bundle', 'composite_name',
         'units', 'units_system_name', 'family_name',
         'external_type', 'internal_type', 
         'description', 'default', 
         'is_optional', 'repeat_group' ])

    def is_nonconformant( self, arg, arg_list ):
        arg_default = self.md.lookup('arg', arg.arg_name)
        return not arg.equals( arg_default, self.arg_attr_names)

    def process_port( self, actor, port ):
        arg = ArgumentElement()
        pname = port.name
        port.name = port.basename
        arg.set_from_object( port )
        port.name = pname
        
        name = arg.arg_name
        if name is None or name == '':
            print '\nArg with no name: actor.name=', actor.name
            arg.print_traits()
        arg_list = self.args_by_name.setdefault( name, [] )
        if self.is_nonconformant( arg, arg_list ):
            arg.actor_name = actor.name
            arg_list.append( arg )

    def process_actor( self, actor ):
        for bundle in actor.class_input_bundles:
            for port in bundle.ports:
                self.process_port( actor, port )
        for bundle in actor.class_output_bundles:
            for port in bundle.ports:
                self.process_port( actor, port )
    
    def process_folder( self, folder ):
        for item in folder.items:
            if isinstance( item, GraphActorFolder ):
                self.process_folder( item )
            else:
                self.process_actor( item.instance )
    
    def print_ports( self, out ):
        """ print string representation of port metadata """
        for attr_name in self.arg_attr_print_names:
            out.write( "%s\t" % attr_name )
        out.write('\n')
        
        for name, arg_list in self.args_by_name.items():
            if len(arg_list) > 0:
                arg_default = self.md.lookup('arg', name)
                arg_default.actor_name = 'DEFAULT'
                for attr_name in self.arg_attr_print_names:
                    out.write("%s\t" % arg_default.get(attr_name)[attr_name])
                out.write('\n')

                for arg in arg_list:
                    for attr_name in self.arg_attr_print_names:
                        out.write("%s\t" % arg.get(attr_name)[attr_name])
                    out.write('\n')

    def as_xml( self ):
        args = []
        for l in self.args_by_name.values():
            args.extend(l)
        
        args.sort(lambda x, y: cmp(x.arg_name, y.arg_name))
        de = DefaultElement( default_args=args)
        xml = de.as_xml()
        return xml


    def unique_args(self, filename):
        """ write nonconformant arguments into the named file.
        Read into excel to view; columns are tab separated.
        """
        self.args_by_name = {}
        self.md = application.service_registry.get_service(
                        'enthought.graph.function.MetadataDefaults')
        self.cm=ChipManager()
        self.process_folder( self.cm.actor_classes )
        
        out = file( filename, 'w+')
        self.print_ports( out )
        out.close()

nca=NonconformantArgs()

