# Run within CPLab
from enthought.traits.api import HasTraits, Dict, List

from enthought.graph.function.metadata_elements import ArgumentElement, DefaultElement
from enthought.graph.vpl.chip_manager import ChipManager
from enthought.graph.vpl.graph_actor_folder import GraphActorFolder

class UniqueArgFinder( HasTraits ):

    # key is port.arg_name, value is List(ConvergePort)
    args_by_name = Dict

    arg_attr_names = List([ 'arg_name', 'name', 'modifier', 'bundle', 'composite_name',
         'units', 'units_system_name', 'family_name',
         'external_type', 'internal_type', 
         'default', 
         'is_optional', 'repeat_group', ])

    arg_attr_print_names = List([ 'arg_name', 'name', 'modifier', 'bundle', 'composite_name',
         'units', 'units_system_name', 'family_name',
         'external_type', 'internal_type', 
         'description', 'default', 
         'is_optional', 'repeat_group', 'actor_name' ])

    def is_unique_arg( self, arg, arg_list ):
        for a in arg_list:
            if arg.equals(a, self.arg_attr_names):
                return False

        return True

    def process_port( self, actor, port ):
        arg = ArgumentElement()
        arg.set_from_object( port )

        name = arg.arg_name
        if name is None or name == '':
            print '\nArg with no name: actor.name=', actor.name
            arg.print_traits()
        arg_list = self.args_by_name.setdefault( name, [] )
        if self.is_unique_arg( arg, arg_list ):
            arg.actor_name = actor.name
            arg_list.append( arg )

    def process_actor( self, actor ):
        for bundle in actor.class_input_bundles:
            for port in bundle.ports:
                self.process_port( actor, port )
    
    def process_folder( self, folder ):
        for item in folder.items:
            if isinstance( item, GraphActorFolder ):
                self.process_folder( item )
            else:
                self.process_actor( item.instance )
    
    def print_ports( self ):
        for attr_name in self.arg_attr_print_names:
            print "%s\t" % attr_name,
        print
        for name, arg_list in self.args_by_name.items():
            for arg in arg_list:
                for attr_name in self.arg_attr_print_names:
                    print "%s\t" % arg.get(attr_name)[attr_name],
                print

    def as_xml( self ):
        args = []
        for l in self.args_by_name.values():
            args.extend(l)
        
        args.sort(lambda x, y: cmp(x.arg_name, y.arg_name))
        de = DefaultElement( default_args=args)
        xml = de.as_xml()
        return xml


    def print_unique_args(self):
        cm=ChipManager()
        self.process_folder( cm.actor_classes )
        print self.as_xml()

uaf=UniqueArgFinder()
