import unittest
from os.path import abspath, dirname, join

class ActorPrototypeFactoryTestCase( unittest.TestCase ):
    
    def setUp( self ):        
        self.filename = join( abspath(dirname(__file__)), "sample1.py")

        
    def test_factory( self ):
        try:
            #import to ensure that port factory is setup at import time.
            from enthought.graph.tests.port_factory_setup import PortFactorySetup        
            from enthought.graph.function import ActorPrototypeFactory
            from enthought.graph.any_type_value_port import AnyTypeValuePort
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("import error: " + str(e))

        apf = ActorPrototypeFactory()
        
        actor = apf.from_filename( self.filename )
        
        self.assertEqual( actor.name, 'Sample1')
        self.assertEqual( 4, len( actor.input_ports ) )
        self.assertEqual( 3, len( actor.class_input_bundles ) )
        self.assertEqual( 1, len( actor.output_ports ) )
        self.assertEqual( 1, len( actor.class_output_bundles ) )
        
        bundle = actor.class_input_bundles[0]
        self.assertEqual( 'm', bundle.name )
        port = bundle.ports[0]
        self.assertEqual( 'm', port.name )
        
        bundle = actor.class_input_bundles[2]
        self.assertEqual( 'shale', bundle.name )
        port = bundle.ports[0]
        self.assertEqual( 'k_wat_sh', port.name )
        self.assertEqual( 'k', port.basename )
        self.assertEqual( 'wat', port.modifier )
        self.assertEqual( 'sh', port.bundle )
        
        bundle = actor.class_input_bundles[1]
        self.assertEqual( 'sand', bundle.name )
        self.assertEqual( ['1', '2', '3'], bundle.names )
        self.assertEqual( 3, bundle.max_num )
        self.assertEqual( 1, bundle.min_num )
        port = bundle.ports[0]
        self.assertEqual( 'repeatable_ss', port.name )
        self.assertEqual( 'repeatable', port.basename )
        port = bundle.ports[1]
        self.assertEqual( 'repeatable2_ss', port.name )
        self.assertEqual( 'repeatable2', port.basename )
        
        bundle = actor.class_output_bundles[0]
        self.assertEqual( 'avg', bundle.name )
        port = bundle.ports[0]
        self.assertEqual( 'inputs_avg', port.name )
        self.assertEqual( 'inputs', port.basename )
        
        result = actor.function[0]( 'first', 'second' )
        self.assertEqual( result, ['first', 'second'])

        return
        
#### EOF ######################################################################
