import unittest
from os.path import abspath, dirname, join
import re

from enthought.traits.api import HasTraits

class BoolTester( HasTraits ):
    def __init__(self):
        from enthought.graph.function import Bool
        self.value = Bool

class MetadataParserTestCase( unittest.TestCase ) :

    def setUp( self ):
        try:
            from enthought.graph.function import Bool
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("import error: " + str(e))
    
    def tearDown( self ):
        return
        

    def test_bool( self ):
        """ test bool trait that allows strings """
        
        b = BoolTester()
        b.value = True
        self.assertEqual( True, b.value)
        b.value = 'True'
        self.assertEqual( True, b.value)
        b.value = 'true'
        self.assertEqual( True, b.value)
        b.value = 'false'
        self.assertEqual( False, b.value)
        b.value = 'False'
        self.assertEqual( False, b.value)
        b.value = False
        self.assertEqual( False, b.value)
    
        
    def test_sample1( self ):
        try:
            from enthought.graph.function import MetadataParser
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("import error: " + str(e))
            
        
        filename = join( abspath(dirname(__file__)), "sample1.py")

        # create instance of xml processor
        parser = MetadataParser()
    
        function_elements = parser.elements_from_file('function', filename)
        
        self.assertEqual( 1, len(function_elements) )
        
        fe = function_elements[0]
        self.assertEqual( fe.tag_name, 'function' )
        self.assertEqual( len(fe.input_args), 4)
        self.assertEqual( len(fe.output_args), 1)
        self.assertEqual( fe.function_name, 'sample1' )
        self.assertEqual( fe.name, 'Sample1' )
        self.assertEqual( fe.category, 'Examples' )
        self.assertEqual( fe.description, 'Sample function in Examples category' )
        self.assertEqual( fe.is_hidden, False )
        self.assertEqual( fe.history.text, 
            """2003/05/16 rjb Changed input name to match family names
            2004/11/15 mlr Convert to python
            2005/03/02 wjd Add check of input parameter count""" )
        self.assertEqual( fe.help.text, 
            """Examples / Sample 1
            Help for the sample.""")
        
        arg = fe.input_args[0]
        self.assertEqual( arg.tag_name, 'arg' )
        self.assertEqual( arg.name, 'k' )
        self.assertEqual( arg.modifier, 'wat' )
        self.assertEqual( arg.bundle, 'sh' )
        self.assertEqual( arg.description, 'Bulk modulus of pore water' )
        self.assertEqual( arg.units, 'gpa' )
        self.assertEqual( arg.family_name, 'modulus' )
        self.assertEqual( arg.units_system_name, 'KGS' )
        self.assertEqual( arg.external_type, 'str' )
        self.assertEqual( arg.internal_type, 'str' )
        self.assertEqual( arg.is_index, False)
        self.assertEqual( arg.is_optional, False)
        self.assertEqual( arg.arg_name, 'x')
        
        arg = fe.input_args[1]
        self.assertEqual( arg.tag_name, 'arg' )
        self.assertEqual( arg.name, 'm' )
        self.assertEqual( arg.modifier, '' )
        self.assertEqual( arg.bundle, 'm' )
        self.assertEqual( arg.description, 'property to be averaged' )
        self.assertEqual( arg.units, 'gpa' )
        self.assertEqual( arg.family_name, 'modulus' )
        self.assertEqual( arg.external_type, 'str' )
        self.assertEqual( arg.internal_type, 'str' )
        self.assertEqual( arg.is_index, False)
        self.assertEqual( arg.is_optional, True)
        self.assertEqual( arg.arg_name, 'm')
        
        arg = fe.output_args[0]
        self.assertEqual( arg.tag_name, 'arg' )
        self.assertEqual( arg.name, 'inputs' )
        self.assertEqual( arg.modifier, '' )
        self.assertEqual( arg.bundle, 'avg' )
        self.assertEqual( arg.description, 'Hill average of property' )
        self.assertEqual( arg.units, 'gpa' )
        self.assertEqual( arg.family_name, 'modulus' )
        self.assertEqual( arg.external_type, 'list' )
        self.assertEqual( arg.internal_type, 'list' )
        self.assertEqual( arg.is_index, False)
        self.assertEqual( arg.is_optional, False)
        self.assertEqual( arg.arg_name, 'm_avg')
        
        
    def print_function_elements( function_elements ):
        """ debugging printer..."""
        for function_element in function_elements:
            print "Function attributes:"
            function_element.print_traits()
    
            print "\nInput arguments:"
            for arg in function_element.input_args :
                arg.print_traits()
    
            print "\nOutput arguments:"
            for arg in function_element.output_args :
                arg.print_traits()
        
        return
        
#### EOF ######################################################################
