
from relation import Relation

# inspired by Ptolemy http://ptolemy.eecs.berkeley.edu/ptolemyII
# Unlike Ptolemy, our Ports are single channel ports, thus the getReceiver
# methods don't return [][]
    
class IORelation( Relation ) :
    """ IORelation manages connections between IOPorts.
    
    The container for instances of this class must be a CompositeActor. 
    """
    def __init__(self, *args, **kw) :
        """ Create a new IORelation. """
        Relation.__init__(self, *args, **kw)

    
    def deep_receivers( self, exclude=None ) :
        """ Return the receivers of all input ports linked to this relation,
        directly or indirectly except those in the specified port.
        """
        ### this appears to be unused...
        receivers = []
        ports = self.linked_destination_ports( exclude )
        for port in ports :
            if port.is_inside_linked( self ) and port.is_opaque == False :
                receivers += port.remote_receivers( self )
            
            else :
                receivers += port.receivers( self ) 

        return receivers
        
    def linked_destination_ports( self, exclude=None ) :
        """ Return a list of the input ports that we are linked to from the
        outside, and the output ports that we are linked to from
        the inside, except the port given as an argument and return
        a list of list of these ports.
        I.e., list the ports through or to which we could send data.
        If the given port is null or is not linked to this relation,
        then enumerate all the input ports.
        """
        ports = []
        for port in self.linked_ports(exclude) :
            if port.is_inside_linked(self) :
                if not port.is_input :
                    ports.append(port)
            else :
                if port.is_input :
                    ports.append(port)
                    
        return ports

    def linked_source_ports( self, exclude=None ) :
        """ List the output ports that we are linked to from the outside
		and the input ports that we are linked to from the inside excluding
		the specified port.
		I.e. list the ports from or through which we might receive data.
        """
        ports = []
        for port in self.linked_ports(exclude) :
            if port.is_inside_linked( self ) :
                if port.is_input :
                    ports.append(port)
            else :
                if not port.is_input :
                    ports.append(port)
        
        return ports

#### EOF ######################################################################
