from enthought.logger import logger

from enthought.traits.api import HasTraits, Property, Int

from contained import Contained
from schedule import Schedule

class Scheduler(Contained) :
    """ A Scheduler builds a Schedule from a CompositeNode.
	A Director will iterate the Actors in the Schedule in the order specified
	by the Schedule.
	
	Trait Properties:
		container isa CompsiteActor
		schedule isa Schedule
    """
    
    def __init__(self, *args, **kw) :
        self._schedule = None
        Contained.__init__(self, *args, **kw)
        return
        
    #### Trait Properties #####################################################
    

    def _get_schedule( self ) :
        """ return a Schedule for the container """
        if self._schedule is None :
            if self.container is None :
                raise RuntimeError, "No container to schedule"
            self._schedule = self._create_schedule()
        
        return self._schedule
    
    schedule = Property( _get_schedule )
    
    #### Protected Methods ####################################################

    def _create_schedule( self ) :
        """ Build the schedule from the container CompositeActor.
        Subclasses are expected to override this method.
        
        The default implementation returns the reachable nodes in an undefined
        order with the iterations count set to one.
        """
        items=self._container.reachable_node_list()
        return Schedule( items=items, iterations=1 )
        
#### EOF ######################################################################        
