""" A script editor. """


# Standard library imports.
import os

# Enthought library imports.
from enthought.envisage.ui import Editor
from enthought.pyface.api import PythonEditor
from enthought.traits.api import Any, Str


class ScriptEditor(Editor):
    """ A script editor. """

    # The Python editor.
    editor = Any
    
    # The pathname of the script being edited.
    path = Str

    ###########################################################################
    # Protected 'Window' interface.
    ###########################################################################

    def _create_contents(self, parent):
        """ Creates the window contents. """

        self.editor = PythonEditor(parent, path=self.path)
        self.editor.on_trait_change(self._on_dirty_changed, 'dirty')
        self.editor.on_trait_change(self._on_path_changed, 'path')
        self.editor.on_trait_change(self._on_key_pressed, 'key_pressed')
        
        return self.editor.control

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _on_dirty_changed(self, dirty):
        """ Called when the dirty status of the editor has changed. """

        if len(self.path) > 0:
            if dirty:
                self.title = os.path.basename(self.path) + '*'
                
            else:
                self.title = os.path.basename(self.path)
            
        return

    def _on_path_changed(self, path):
        """ Called when the path of the editor has changed. """

        self.path = path
        self.title = os.path.basename(path)

        return

    def _on_key_pressed(self, event):
        """ Called when a key is pressed. """

        # fixme: Errr, we might want to loosen up the coupling between key
        # bindings and actions!
        # 
        # Ctrl-S saves the file!
        if event.control_down and event.key_code == 19:
            self.editor.save()

        # Ctrl-R saves AND runs the file!
        elif event.control_down and event.key_code == 18:
            self.editor.save()

            # fixme: This should be part of the editor API!
            from enthought.envisage.ui.python_shell import PythonShellPlugin

            plugin = PythonShellPlugin.instance
            plugin.execute_command('execfile("%s")' % self.path)

        # Zoom in.
        elif event.alt_down and event.key_code == 317:
            zoom = self.editor.control.GetZoom()
            if zoom != 20:
                self.editor.control.SetZoom(zoom+1)

        elif event.alt_down and event.key_code == 319:
            zoom = self.editor.control.GetZoom()
            if zoom != -10:
                self.editor.control.SetZoom(zoom-1)

        return
    
#### EOF ######################################################################
