
import traceback
import sys

# Enthought library

from enthought.logger import logger
from enthought.traits.api import Class

from sdf_director import SdfDirector
from value_port_receiver import ValuePortReceiver
from token_proxy import TokenProxy
from proxy_receiver import ProxyReceiver

from scheduler import Scheduler
from schedule import Schedule
from topology_scheduler import TopologyScheduler
from exceptions import FireError

class SimpleSdfDirector( SdfDirector ) :
    """
    A Director that implements Synchronous DataFlow semantics but assumes the
    graph is very simple.  This means each input has zero or one upstream port,
    data rates are all one, and ports are all ValuePort instances.
    """
    
    def new_receiver( self, port ) :
        """ Return a new receiver that is compatible with the director.
        """
        # TODO: need to also handle input ports bound to naming system names
        if len(port.relations) == 0:
            source_port = port
        
        else:
            source_port = port.source_port()
            
        if isinstance( source_port.token, TokenProxy ):
            receiver = ProxyReceiver( source_port=source_port )
        else:
            receiver = ValuePortReceiver( source_port=source_port)

        #print 'SimpleSdfDirector.new_receiver', port.full_name, receiver
        return receiver
    

    def prefire( self ) :
        """ Check the input ports of the container composite actor (if there
        are any) to see whether they have enough tokens, and return true
        if they do.  If there are no input ports, then also return true.
        Otherwise, return false.  Note that this does not call prefire()
        on the contained actors.
        """
        schedule = self._get_scheduler().schedule
        rate = schedule.iterations
        
        # No checking of port input avaliability, that's an assumption this
        # implementation makes.
        
        return True
        
        
    def fire( self ) :
        """ Calculate the current schedule, if necessary, and iterate the
        contained actors in the order given by the schedule. 
        
        Iterating an actor involves calling the actor's iterate() method,
        which is equivalent to calling the actor's prefire(), fire() and
        postfire() methods in succession.  
        """
        schedule = self._get_scheduler().schedule
        self._run_schedule( schedule )

        return

    #### Protected Methods ####################################################
    
    def _run_schedule( self, schedule ) :
        """ Handles simple schedules only, i.e. no nested schedules.
        """
        try:
            for item in schedule.items :
                item.iterate( schedule.iterations )

        except Exception, ex:
            raise FireError( item, ex )

        return

        
        
        
#### EOF ######################################################################    
