
from sets import Set

from enthought.logger import logger

from topology_scheduler import TopologyScheduler
from subgraph import create_ancestor_map


def pretty_print(ancestor_map):
    logger.debug(" *** Ancestor map *** ")
    for key, items in ancestor_map.items():
        logger.debug( key.name + ": " +  ",".join([i.name for i in items]))


class SubgraphScheduler(TopologyScheduler):
    """
    Scheduler for a subgraph of a graph.  A subgraph is defined by a set of
    input actors and a set of output actors, and this scheduler only runs
    the parts of the graph necessary to generate new values on the output
    actors given changes on the input actors.
    """
    
    def __init__(self, input_actors=None, output_actors=None, **kw):
        """
        If input_actors is None, then all actors without ancestors are
        considered input actors.
        
        If output_actors is None, then treats all actors with no descendants
        as output actors.
        """
        self.input_actors = input_actors
        self.output_actors = output_actors
        logger.debug("subgraph:: init")
        TopologyScheduler.__init__(self, **kw)
        logger.debug("subgraph:: finished init")
    
    def _create_ancestor_map(self):
        """
        Returns the "ancestor map", a dict that maps each actor to its list of
        ancestor actors.  The only items on the ancestor map will be actors
        lying in the graph between self.input_actors and self.output_actors.
        """
        
        all_nodes = self._container.reachable_node_list()
        if self.input_actors is None:
            input_actors = all_nodes
        else:
            input_actors = self.input_actors
        output_actors = self.output_actors
        
        ancestor_map = create_ancestor_map(input_actors, output_actors)
        #pretty_print(ancestor_map)
        return ancestor_map


# EOF
