import unittest

from enthought.logger import logger

from enthought.graph import SdfDirector

from actors import *

        
class FanOutActorTestCase( unittest.TestCase ) :

    def setUp(self) :
        logger.debug("-------------------------------------------------------")
        self.top = CompositeActor('composite')

        self.output_actor = OutputActor('output_actor', value='a')
        self.output_actor.container = self.top

        self.fanout_actor = FanOutActor('fanout_actor')
        self.fanout_actor.container = self.top

        self.input1_actor = InputActor('input1_actor')
        self.input1_actor.container = self.top

        self.input2_actor = ListSinkActor('input2_actor')
        self.input2_actor.container = self.top

        self.top.connect( self.output_actor.get_port('out'), 
                        self.fanout_actor.get_port('in'))

        self.top.connect( self.fanout_actor.get_port('out1'), 
                        self.input1_actor.get_port('in'), 'in1')

        self.top.connect( self.fanout_actor.get_port('out2'), 
                        self.input2_actor.get_port('in'), 'in2')

        self.director = SdfDirector()
        self.director.container = self.top

        logger.debug("test execution begins...")
        
        self.top.preinitialize()
        self.top.initialize()
        return

    def tearDown( self ) :
        logger.debug("test execution ends.")
        self.top.wrapup()
        return
        
    def test_simple_token( self ) :
        self.output_actor.value = 1
        self.top.iterate(1)
        self.assertEqual( 1, self.input1_actor.value, \
            "input_actor has unexpected value: %s" % self.input1_actor.value )    
        self.assertEqual( 1, self.input2_actor.value, \
            "input_actor has unexpected value: %s" % self.input2_actor.value )    
        return

    def test_list_token( self ) :
        self.output_actor.value = [1,2,3]
        self.top.iterate(1)
        self.assertEqual( [1,2,3], self.input1_actor.value, \
            "input_actor has unexpected value: %s" % self.input1_actor.value )    
        self.assertEqual( 0, self.input1_actor._available_tokens(), \
            "input_actor has incorrect number of available tokens")
        self.assertEqual( [1,2,3], self.input2_actor.value, \
            "input_actor has unexpected value: %s" % self.input2_actor.value )    
        self.assertEqual( 0, self.input2_actor._available_tokens(), \
            "input_actor has incorrect number of available tokens")
        return

    def test_list_of_tokens( self ) :
        tokens = range(5)
        self.output_actor.value_list = tokens
        for value in tokens :
            self.top.iterate(1)
            self.assertEqual( value, self.input1_actor.value,
                "input_actor has unexpected value: %s" % self.input1_actor.value )    
            self.assertEqual( value, self.input2_actor.value,
                "input_actor has unexpected value: %s" % self.input2_actor.value )    
            self.assertEqual( range(value+1), self.input2_actor.values,
                "input_actor has unexpected values: %s" % self.input2_actor.values )    
        return


class FanOutRelationTestCase( FanOutActorTestCase ) :
    
    def setUp(self) :
        logger.debug("-------------------------------------------------------")
        self.top = CompositeActor('composite')

        self.output_actor = OutputActor('output_actor')
        self.output_actor.container = self.top

        self.input1_actor = InputActor('input1_actor')
        self.input1_actor.container = self.top

        self.input2_actor = ListSinkActor('input2_actor')
        self.input2_actor.container = self.top

        fanout_relation = self.top.connect( self.output_actor.get_port('out'), 
                                            self.input1_actor.get_port('in'),
                                            'fanout_relation')

        self.input2_actor.get_port('in').link( fanout_relation )

        self.director = SdfDirector()
        self.director.container = self.top

        logger.debug("test execution begins...")
        
        self.top.preinitialize()
        self.top.initialize()
        return
                
    
### EOF ######################################################################
