import unittest

from enthought.logger import logger

from enthought.graph import SdfDirector
from enthought.graph import FifoReceiver
from enthought.graph import FifoHistoryReceiver
from enthought.graph import FifoAutoHistoryReceiver

from actors import *
from two_node_execution_test_case import TwoNodeExecution

class TransparentCompositeActorWithNoop( TwoNodeExecution ) :

    def setUp(self) :
        logger.debug("-------------------------------------------------------")

        self.setUp_construct_dataflow()
        
        logger.debug("test execution begins TransparentCompositeActorWithNoop...")
        
        self.top.preinitialize()
        self.top.initialize()
        return

    def setUp_construct_dataflow( self ) :
        self.top = CompositeActor('top')

        self.output_actor = OutputActor('output_actor', value='a')
        self.output_actor.container = self.top

        self.ca = TransparentCompositeActor('transparent_composite')
        self.ca.container = self.top

        self.input_actor = InputActor('input_actor')
        self.input_actor.container = self.top

        self.noop_actor = NoopActor('noop_actor')
        self.noop_actor.container = self.ca
        relation = self.ca.get_relation("cout")
        relation.container = None
        
        
        self.ca.connect( self.ca.get_port( "cin"), 
                        self.noop_actor.get_port("in" ))
        self.ca.connect( self.noop_actor.get_port("out" ),
                        self.ca.get_port( "cout"))
        
        self.top.connect( self.output_actor.get_port('out'), 
                        self.ca.get_port('cin'))

        self.top.connect( self.ca.get_port('cout'), 
                        self.input_actor.get_port('in'))

        # based on the selection of SdfDirector with FifoReceiver...
        self.expected_available_tokens = 1
        
        self.director = SdfDirector( receiver_class = FifoAutoHistoryReceiver )
        self.director.container = self.top
        
        return
        
class TransparentCompositeActorWithNoopTestCase( 
    TransparentCompositeActorWithNoop, unittest.TestCase ) :

    def __init__( self, *args, **kw ) :
        TransparentCompositeActorWithNoop.__init__( self, *args, **kw )
        unittest.TestCase.__init__( self, *args, **kw )
        return


    
### EOF ######################################################################
