from enthought.logger import logger


from enthought.graph import AtomicActor, CompositeActor
from enthought.util.numerix import array, concatenate

def build_array(a, num_repeat, incrmnt=0.0) :
    """Build a longer array by concatenating a to itself num_repeat times"""
    b = a
    if num_repeat > 0 :
        boost = incrmnt * len(a)
        for n in range(0, num_repeat) :
            b = concatenate((b,a+(n+1)*boost), 1)

    return b
        
class BiotCoeffRhobActor( AtomicActor ) :
    """ Actor that emulates the 'biot_coeff_rhob properties' chip
    from the TSD demo for the Matlab version of converge.
    It has the following inputs:

        - k_wat_ss_in
        - rho_wat_ss_in
        - kg_ss_in
        - rhog_ss_in
        - rhob_ss_in
        - vp_ss_in
        - vs_ss_in
        
    Its outputs are:
        
        - biot_ss_out
        - biotmax_ss_out
        
    Computations for this chip are performed in function exec_script().
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports for set #1
        pin_k_wat = self.get_port("k_wat_ss_in")
        pin_rho_wat = self.get_port("rho_wat_ss_in")
        pin_kg_ss = self.get_port("kg_ss_in")
        pin_rhog_ss = self.get_port("rhog_ss_in")
        pin_rhob_ss = self.get_port("rhob_ss_in")
        pin_vp_ss = self.get_port("vp_ss_in")
        pin_vs_ss = self.get_port("vs_ss_in")

#        # get the input ports for set #2
#        pin_k_watsh = self.get_port("k_wat_sh_in")
#        pin_rho_watsh = self.get_port("rho_wat_sh_in")
#        pin_kg_sh = self.get_port("kg_sh_in")
#        pin_rhog_sh = self.get_port("rhog_sh_in")
#        pin_rhob_sh = self.get_port("rhob_sh_in")
#        pin_vp_sh = self.get_port("vp_sh_in")
#        pin_vs_sh = self.get_port("vs_sh_in")

        # get tokens from input ports for set #1
        k_wat = pin_k_wat.get()
        rho_wat = pin_rho_wat.get()
        kg_ss = pin_kg_ss.get()
        rhog_ss = pin_rhog_ss.get()
        rhob_ss = pin_rhob_ss.get()
        vp_ss = pin_vp_ss.get()
        vs_ss = pin_vs_ss.get()

#        # get tokens from input ports for set #2
#        k_watsh = pin_k_watsh.get()
#        rho_watsh = pin_rho_watsh.get()
#        kg_sh = pin_kg_sh.get()
#        rhog_sh = pin_rhog_sh.get()
#        rhob_sh = pin_rhob_sh.get()
#        vp_sh = pin_vp_sh.get()
#        vs_sh = pin_vs_sh.get()

        # execute the script
        self.value = self._exec_script(k_wat, rho_wat, kg_ss, rhog_ss, rhob_ss,
                                       vp_ss, vs_ss )
#                                       , k_watsh, rho_watsh, kg_sh,
#                                       rhog_sh, rhob_sh, vp_sh, vs_sh)

        # display tokens for set #1
        #logger.debug("%s fire inputs get %s for k_wat" % (self.name,k_wat))
        #logger.debug("%s fire inputs get %s for rho_wat" % (self.name, rho_wat))
        #logger.debug("%s fire inputs get %s for kg_ss" % (self.name, kg_ss))
        #logger.debug("%s fire inputs get %s for rhog_ss" % (self.name, rhog_ss))
        #logger.debug("%s fire inputs get %s for rhob_ss" % (self.name, rhob_ss))
        #logger.debug("%s fire inputs get %s for vp_ss" % (self.name, vp_ss))
        #logger.debug("%s fire inputs get %s for vs_ss" % (self.name, vs_ss))

        #logger.debug("%s fire input total is %s" % (self.name, self.value))

        # get output ports and put values
        pout_biot_ss = self.get_port("biot_ss_out")
        pout_biotmax_ss = self.get_port("biotmax_ss_out")
#        pout_biot_sh = self.get_port("biot_sh_out")
#        pout_biotmax_sh = self.get_port("biotmax_sh_out")
        pout_biot_ss.put(self.value[0])
        pout_biotmax_ss.put(self.value[1])

        return

    def _exec_script(self, k_wat, rho_wat, kg_ss, rhog_ss, rhob_ss,
                     vp_ss, vs_ss ):
#                     , k_watsh, rho_watsh, kg_sh,
#                     rhog_sh, rhob_sh, vp_sh, vs_sh) :
        # call function that implements the rock physics
        try:
            from cp.rockphysics.converge.elasticity.biot_coeff_rhob import biot_coeff_rhob
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("ImportError: " + str(e))
        output = biot_coeff_rhob(k_wat, rho_wat, kg_ss, rhog_ss, rhob_ss,
                                 vp_ss, vs_ss )
        return output

    def _create_ports(self) :
        # create input ports for set #1
        self.new_port("k_wat_ss_in", True )
        self.new_port("rho_wat_ss_in", True )
        self.new_port("kg_ss_in", True )
        self.new_port("rhog_ss_in", True )
        self.new_port("rhob_ss_in", True )
        self.new_port("vp_ss_in", True )
        self.new_port("vs_ss_in", True )

#        # create input ports for set #2
#        self.new_port("k_wat_sh_in", True )
#        self.new_port("rho_wat_sh_in", True )
#        self.new_port("kg_sh_in", True )
#        self.new_port("rhog_sh_in", True )
#        self.new_port("rhob_sh_in", True )
#        self.new_port("vp_sh_in", True )
#        self.new_port("vs_sh_in", True )

        # create output ports for set #1
        self.new_port("biot_ss_out", False )
        self.new_port("biotmax_ss_out", False )

#        # create output ports for set #2
#        self.new_port("biot_sh_out", False )
#        self.new_port("biotmax_sh_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        # available tokens for input ports for set #1
        p1a = self.get_port("k_wat_in")
        p2a = self.get_port("rho_wat_in")
        p3a = self.get_port("kg_ss_in")
        p4a = self.get_port("rhog_ss_in")
        p5a = self.get_port("rhob_ss_in")
        p6a = self.get_port("vp_ss_in")
        p7a = self.get_port("vs_ss_in")

#        # available tokens for input ports for set #2
#        p1b = self.get_port("k_watsh_in")
#        p2b = self.get_port("rho_watsh_in")
#        p3b = self.get_port("kg_sh_in")
#        p4b = self.get_port("rhog_sh_in")
#        p5b = self.get_port("rhob_sh_in")
#        p6b = self.get_port("vp_sh_in")
#        p7b = self.get_port("vs_sh_in")

        # receivers for input ports for set #1
        r1a = p1a.receivers()
        r2a = p2a.receivers()
        r3a = p3a.receivers()
        r4a = p4a.receivers()
        r5a = p5a.receivers()
        r6a = p6a.receivers()
        r7a = p7a.receivers()

#        # receivers for input ports for set #2
#        r1b = p1b.receivers()
#        r2b = p2b.receivers()
#        r3b = p3b.receivers()
#        r4b = p4b.receivers()
#        r5b = p5b.receivers()
#        r6b = p6b.receivers()
#        r7b = p7b.receivers()

        # availiable tokens for input ports for set #1
        n  = r1a[0].available_tokens()
        n += r2a[0].available_tokens()
        n += r3a[0].available_tokens()
        n += r4a[0].available_tokens()
        n += r5a[0].available_tokens()
        n += r6a[0].available_tokens()
        n += r7a[0].available_tokens()

#        # availiable tokens for input ports for set #2
#        n  = r1b[0].available_tokens()
#        n += r2b[0].available_tokens()
#        n += r3b[0].available_tokens()
#        n += r4b[0].available_tokens()
#        n += r5b[0].available_tokens()
#        n += r6b[0].available_tokens()
#        n += r7b[0].available_tokens()
        return n/(7*2)

class BrinePropertiesActor( AtomicActor ) :
    """ Actor that emulates the 'brine properties' chip
    from the TSD demo for the Matlab version of converge.
    It has the following inputs:
        
        - ppore_in
        - tform_in
        - salw_in
    
    Its outputs are:

        - rho_wat_out
        - k_wat_out
        
    Computations for this chip are performed in function exec_script().
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports
        pin_ppore = self.get_port("ppore_in")
        pin_tform = self.get_port("tform_in")
        pin_salw  = self.get_port("salw_in")

        # get tokens from input ports
        ppore  = pin_ppore.get()
        tform = pin_tform.get()
        salw = pin_salw.get()

        # execute script
        self.value = self._exec_script(ppore, tform, salw)

        # display tokens
        #logger.debug( "%s fire inputs get %s for ppore" % (self.name, ppore) )
        #logger.debug( "%s fire inputs get %s for tform" % (self.name, tform) )
        #logger.debug( "%s fire inputs get %s for salw" % (self.name, salw) )
        #logger.debug( "%s fire input total is %s" % (self.name, self.value) )

        # get output ports and put values
        pout_rhowat = self.get_port("rho_wat_out")
        pout_kwat = self.get_port("k_wat_out")
        pout_rhowat.put(self.value[0])
        pout_kwat.put(self.value[1])

        return

    def _exec_script(self, ppore, tform, salw) :
        # call function that implements the rock physics
        try:
            from cp.rockphysics.converge.fluids.brine_properties import brine_properties
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("ImportError: " + str(e))
        output = brine_properties(ppore, tform, salw)
        return output

    def _create_ports(self) :
        self.new_port("ppore_in", True )
        self.new_port("tform_in", True )
        self.new_port("salw_in", True )
        self.new_port("rho_wat_out", False )
        self.new_port("k_wat_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        p1 = self.get_port("ppore_in")
        p2 = self.get_port("tform_in")
        p3 = self.get_port("salw_in")
        r1 = p1.receivers()
        r2 = p2.receivers()
        r3 = p3.receivers()
        n  = r1[0].available_tokens()
        n += r2[0].available_tokens()
        n += r3[0].available_tokens()
        return n/3

class TempPressLinearActor( AtomicActor ) :
    """ Actor that emulates the 'temp press linear' chip
    from the TSD demo for the Matlab version of converge.
    It has the following inputs:
        
        - watdth_in
        - oftemp_in
        - tgrad_in
        - pgrad_in
        - depth_in

    Its outputs are:

        - tform_out
        - ppore_out

    Computations for this chip are performed in function exec_script().
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports
        pin_watdth = self.get_port("watdth_in")
        pin_oftemp = self.get_port("oftemp_in")
        pin_tgrad  = self.get_port("tgrad_in")
        pin_pgrad  = self.get_port("pgrad_in")
        pin_depth  = self.get_port("depth_in")

        # get tokens from input ports
        watdth = pin_watdth.get()
        oftemp = pin_oftemp.get()
        tgrad  = pin_tgrad.get()
        pgrad  = pin_pgrad.get()
        depth  = pin_depth.get()

        # execute the script
        self.value = self._exec_script(watdth, oftemp, tgrad, pgrad, depth)

        #logger.debug( "%s fire inputs get %s" % (self.name, self.value) )

        # get output ports and put values
        pout_tform = self.get_port("tform_out")
        pout_ppore = self.get_port("ppore_out")
        pout_tform.put(self.value[0])
        pout_ppore.put(self.value[1])

        return

    def _exec_script(self, watdth, oftemp, tgrad, pgrad, depth) :
        # call function that implements the rock physics
        try:
            from cp.rockphysics.converge.fluids.temp_press_linear import temp_press_linear
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("ImportError: " + str(e))
        output = temp_press_linear(watdth, oftemp, tgrad, pgrad, depth)
        return output

    def _create_ports(self) :
        self.new_port("watdth_in", True )
        self.new_port("oftemp_in", True )
        self.new_port("tgrad_in", True )
        self.new_port("pgrad_in", True )
        self.new_port("depth_in", True )
        self.new_port("tform_out", False )
        self.new_port("ppore_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        p1 = self.get_port("watdth_in")
        p2 = self.get_port("oftemp_in")
        p3 = self.get_port("tgrad_in")
        p4 = self.get_port("pgrad_in")
        p5 = self.get_port("depth_in")
        r1 = p1.receivers()
        r2 = p2.receivers()
        r3 = p3.receivers()
        r4 = p4.receivers()
        r5 = p5.receivers()
        n  = r1[0].available_tokens()
        n += r2[0].available_tokens()
        n += r3[0].available_tokens()
        n += r4[0].available_tokens()
        n += r5[0].available_tokens()
        return n/5

class TsdDataInActor( AtomicActor ) :
    """ Actor with all of the output ports necessary to provide the external
    inputs necessary for the tsd demo test cases (e.g., TestTsdNoSubgroup):
        
        - watdth_out
        - oftemp_out
        - tgrad_out
        - pgrad_out
        - depth_out
        - salw_ss_out
        - salw_sh_out
        - vp_ss_out
        - vp_sh_out
        - kg_ss_out
        - rhog_ss_out
        - rhob_ss_out
        - kg_sh_out
        - rhog_sh_out
        - rhob_sh_out
        - gg_ss_out
        - phit_out
        - xsh_out

    if value_list is not None, the first item is removed from the list and
    assigned to value before value is output.
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        if kw.has_key('value') :
            self.value = kw['value']
        
        self.value = range(0, 18)
        self.value_list = None
        self._create_ports()
        return

    def build_input_arrays(self, num_repeats, increment) :

        self.outputArrays = {}
        self.outputArrays['watdth_out']  = 1362.1512
        self.outputArrays['oftemp_out']  = 1.6667
        self.outputArrays['tgrad_out']   = 1.1
        self.outputArrays['pgrad_out']   = 0.465
        self.outputArrays['depth_out']   = \
                build_array(array((1981.2, 1981.8096, 1982.4192, 1983.0288, 1983.6384)),
                    num_repeats, increment)
        self.outputArrays['salw_ss_out'] = 30000.0
        self.outputArrays['salw_sh_out'] = 15000.0
        self.outputArrays['vp_ss_out']   = \
                build_array(array((3.0, 3.0, 3.107, 2.922, 3.095)), num_repeats)
        self.outputArrays['vp_sh_out']   = \
                build_array(array((2.7, 2.7, 2.8,   2.6,   2.65)), num_repeats)
        self.outputArrays['kg_ss_out']   = \
                build_array(array((37.0, 37.0, 36.0, 38.0, 40.0)), num_repeats)
        self.outputArrays['rhog_ss_out'] = \
                build_array(array((2.65, 2.65, 2.64, 2.63, 2.66)), num_repeats)
        self.outputArrays['rhob_ss_out'] = \
                build_array(array((2.3, 2.3, 2.349, 2.355, 2.448)), num_repeats)
        self.outputArrays['kg_sh_out']   = \
                build_array(array((25.0, 25.0, 27.0, 26.0, 38.0)), num_repeats)
        self.outputArrays['rhog_sh_out'] = \
                build_array(array((2.6, 2.6, 2.582, 2.568, 2.305)), num_repeats)
        self.outputArrays['rhob_sh_out'] = \
                build_array(array((2.5, 2.5, 2.478, 2.492, 2.506)), num_repeats)
        self.outputArrays['gg_ss_out']   = \
                build_array(array((44.0, 44.0, 42.0, 41.0, 43.0)), num_repeats)
        self.outputArrays['phit_out']    = \
                build_array(array((0.15, 0.15, 0.16, 0.145, 0.18)), num_repeats)
        self.outputArrays['xsh_out']     = \
                build_array(array((0.6876, 0.6880, 0.6430, 0.6520, 0.6380)),
                    num_repeats)

        return

    def fire(self) :

        for key in self.outputArrays :
            port = self.get_port(key)
            port.put(self.outputArrays[key])

        return

    def _create_ports(self) :
        self.new_port("watdth_out", False )
        self.new_port("oftemp_out", False )
        self.new_port("tgrad_out", False )
        self.new_port("pgrad_out", False )
        self.new_port("depth_out", False )
        self.new_port("salw_ss_out", False )
        self.new_port("salw_sh_out", False )
        self.new_port("vp_ss_out", False )
        self.new_port("vp_sh_out", False )
        self.new_port("kg_ss_out", False )
        self.new_port("rhog_ss_out", False )
        self.new_port("rhob_ss_out", False )
        self.new_port("kg_sh_out", False )
        self.new_port("rhog_sh_out", False )
        self.new_port("rhob_sh_out", False )
        self.new_port("gg_ss_out", False )
        self.new_port("phit_out", False )
        self.new_port("xsh_out", False )
        return

class TsdModelRhobActor( AtomicActor ) :
    """ Actor that emulates the 'tsdmodel rhob' chip
    from the TSD demo for the Matlab version of converge.
    It has the following inputs:
        
        - vp_ss_in
        - vs_ss_in
        - rhob_ss_in
        - rhog_ss_in
        - kg_ss_in
        - gg_ss_in
        - vp_sh_in
        - vs_sh_in
        - rhob_sh_in
        - rhog_sh_in
        - kg_sh_in
        - phit_in
        - xsh_in
        - rho_wat_in
        - rho_watsh_in (optional)
        
    Its outputs are:
        
        - vp_wet_out
        - vs_wet_out
        - rhob_wet_out
        - vp_sm_out
        - vs_sm_out
        - phi_sm_out
        - rhob_sm_out
        - phie_sm_out
        - rhog_sm_out
        - kg_sm_out
        - vp_hs_out
        - vs_hs_out
        - phi_hs_out
        - xlam_out
        - xdis_out
        - phi_ss_out
        - phi_sh_out
        
    Computations for this chip are performed in function exec_script().
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports
        pin_vp_ss = self.get_port("vp_ss_in")
        pin_vs_ss = self.get_port("vs_ss_in")
        pin_rhob_ss = self.get_port("rhob_ss_in")
        pin_rhog_ss = self.get_port("rhog_ss_in")
        pin_kg_ss = self.get_port("kg_ss_in")
        pin_gg_ss = self.get_port("gg_ss_in")
        pin_vp_sh = self.get_port("vp_sh_in")
        pin_vs_sh = self.get_port("vs_sh_in")
        pin_rhob_sh = self.get_port("rhob_sh_in")
        pin_rhog_sh = self.get_port("rhog_sh_in")
        pin_kg_sh = self.get_port("kg_sh_in")
        pin_phit = self.get_port("phit_in")
        pin_xsh = self.get_port("xsh_in")
        pin_rho_watss = self.get_port("rho_wat_ss_in")
        pin_rho_watsh = self.get_port("rho_wat_sh_in")

        # get tokens from input ports
        vp_ss = pin_vp_ss.get()
        vs_ss = pin_vs_ss.get()
        rhob_ss = pin_rhob_ss.get()
        rhog_ss = pin_rhog_ss.get()
        kg_ss = pin_kg_ss.get()
        gg_ss = pin_gg_ss.get()
        vp_sh = pin_vp_sh.get()
        vs_sh = pin_vs_sh.get()
        rhob_sh = pin_rhob_sh.get()
        rhog_sh = pin_rhog_sh.get()
        kg_sh = pin_kg_sh.get()
        phit = pin_phit.get()
        xsh = pin_xsh.get()
        rho_watss = pin_rho_watss.get()
        rho_watsh = pin_rho_watsh.get()

        # execute the script
        self.value = self._exec_script(vp_ss, vs_ss, rhob_ss, rhog_ss, kg_ss,
                                       gg_ss, vp_sh, vs_sh, rhob_sh, rhog_sh,
                                       kg_sh, phit, xsh, rho_watss, rho_watsh)

        # display tokens
        #logger.debug( "%s fire inputs get %s for vp_ss" % (self.name, vp_ss) )
        #logger.debug( "%s fire inputs get %s for vs_ss" % (self.name, vs_ss) )
        #logger.debug( "%s fire inputs get %s for rhob_ss" % (self.name,\
        #rhob_ss) )
        #logger.debug( "%s fire inputs get %s for rhog_ss" % (self.name,\
        #rhog_ss) )
        #logger.debug( "%s fire inputs get %s for kg_ss" % (self.name, kg_ss) )
        #logger.debug( "%s fire inputs get %s for gg_ss" % (self.name, gg_ss) )
        #logger.debug( "%s fire inputs get %s for vp_sh" % (self.name, vp_sh) )
        #logger.debug( "%s fire inputs get %s for vs_sh" % (self.name, vs_sh) )
        #logger.debug( "%s fire inputs get %s for rhob_sh" % (self.name,\
        #rhob_sh) )
        #logger.debug( "%s fire inputs get %s for rhog_sh" % (self.name,\
        #rhog_sh) )
        #logger.debug( "%s fire inputs get %s for kg_sh" % (self.name, kg_sh) )
        #logger.debug( "%s fire inputs get %s for phit" % (self.name, phit) )
        #logger.debug( "%s fire inputs get %s for xsh" % (self.name, xsh) )
        #logger.debug( "%s fire inputs get %s for rho_watss" % (self.name,\
        #rho_watss) )
        #logger.debug( "%s fire inputs get %s for rho_watsh" % (self.name,\
        #rho_watsh) )

        #logger.debug( "%s fire input total is %s" % (self.name, self.value) )

        # get output ports
        pout_vp_wet = self.get_port("vp_wet_out")
        pout_vs_wet = self.get_port("vs_wet_out")
        pout_rhob_wet = self.get_port("rhob_wet_out")
        pout_vp_sm = self.get_port("vp_sm_out")
        pout_vs_sm = self.get_port("vs_sm_out")
        pout_phi_sm = self.get_port("phi_sm_out")
        pout_rhob_sm = self.get_port("rhob_sm_out")
        pout_phie_sm = self.get_port("phie_sm_out")
        pout_rhog_sm = self.get_port("rhog_sm_out")
        pout_kg_sm = self.get_port("kg_sm_out")
        pout_vp_hs = self.get_port("vp_hs_out")
        pout_vs_hs = self.get_port("vs_hs_out")
        pout_phi_hs = self.get_port("phi_hs_out")
        pout_xlam = self.get_port("xlam_out")
        pout_xdis = self.get_port("xdis_out")
        pout_phi_ss = self.get_port("phi_sm_out")
        pout_phi_sh = self.get_port("phi_sh_out")

        # put values
        pout_vp_wet.put(self.value[0])
        pout_vs_wet.put(self.value[1])
        pout_rhob_wet.put(self.value[2])
        pout_vp_sm.put(self.value[3])
        pout_vs_sm.put(self.value[4])
        pout_phi_sm.put(self.value[5])
        pout_rhob_sm.put(self.value[6])
        pout_phie_sm.put(self.value[7])
        pout_rhog_sm.put(self.value[8])
        pout_kg_sm.put(self.value[9])
        pout_vp_hs.put(self.value[10])
        pout_vs_hs.put(self.value[11])
        pout_phi_hs.put(self.value[12])
        pout_xlam.put(self.value[13])
        pout_xdis.put(self.value[14])
        pout_phi_ss.put(self.value[15])
        pout_phi_sh.put(self.value[16])

        return

    def _exec_script(self, vpss, vsss, rhobss, rhogss, kgss, ggss, vpsh, vssh,
                     rhobsh, rhogsh, kgsh, phit, xsh, rhowat, rhowatsh):
        # call function that implements the rock physics
        try:
            from cp.rockphysics.converge.models.thomas_stieber_dvorkin.tsd_model_rhob import tsd_model_rhob
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("ImportError: " + str(e))
            
        output = tsd_model_rhob(vpss, vsss, rhobss, rhogss, kgss, ggss, vpsh,
                               vssh, rhobsh, rhogsh, kgsh, phit, xsh, rhowat,
                               rhowatsh)
        return output

    def _create_ports(self) :

        # create input ports
        self.new_port("vp_ss_in", True )
        self.new_port("vs_ss_in", True )
        self.new_port("rhob_ss_in", True )
        self.new_port("rhog_ss_in", True )
        self.new_port("kg_ss_in", True )
        self.new_port("gg_ss_in", True )
        self.new_port("vp_sh_in", True )
        self.new_port("vs_sh_in", True )
        self.new_port("rhob_sh_in", True )
        self.new_port("rhog_sh_in", True )
        self.new_port("kg_sh_in", True )
        self.new_port("phit_in", True )
        self.new_port("xsh_in", True )
        self.new_port("rho_wat_ss_in", True )
        self.new_port("rho_wat_sh_in", True )

        # create output ports
        self.new_port("vp_wet_out", False )
        self.new_port("vs_wet_out", False )
        self.new_port("rhob_wet_out", False )
        self.new_port("vp_sm_out", False )
        self.new_port("vs_sm_out", False )
        self.new_port("phi_sm_out", False )
        self.new_port("rhob_sm_out", False )
        self.new_port("phie_sm_out", False )
        self.new_port("rhog_sm_out", False )
        self.new_port("kg_sm_out", False )
        self.new_port("vp_hs_out", False )
        self.new_port("vs_hs_out", False )
        self.new_port("phi_hs_out", False )
        self.new_port("xlam_out", False )
        self.new_port("xdis_out", False )
        self.new_port("phi_ss_out", False )
        self.new_port("phi_sh_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        p1 = self.get_port("vp_ss_in")
        p2 = self.get_port("vs_ss_in")
        p3 = self.get_port("rhob_ss_in")
        p4 = self.get_port("rhog_ss_in")
        p5 = self.get_port("kg_ss_in")
        p6 = self.get_port("gg_ss_in")
        p7 = self.get_port("vp_sh_in")
        p8 = self.get_port("vs_sh_in")
        p9 = self.get_port("rhob_sh_in")
        p10 = self.get_port("rhog_sh_in")
        p11 = self.get_port("kg_sh_in")
        p12 = self.get_port("phit_in")
        p13 = self.get_port("xsh_in")
        p14 = self.get_port("rho_wat_in")
        p15 = self.get_port("rho_watsh_in")

        r1 = p1.receivers()
        r2 = p2.receivers()
        r3 = p3.receivers()
        r4 = p4.receivers()
        r5 = p5.receivers()
        r6 = p6.receivers()
        r7 = p7.receivers()
        r8 = p8.receivers()
        r9 = p9.receivers()
        r10 = p10.receivers()
        r11 = p11.receivers()
        r12 = p12.receivers()
        r13 = p13.receivers()
        r14 = p14.receivers()
        r15 = p15.receivers()
        n  = r1[0].available_tokens()
        n += r2[0].available_tokens()
        n += r3[0].available_tokens()
        n += r4[0].available_tokens()
        n += r5[0].available_tokens()
        n += r6[0].available_tokens()
        n += r7[0].available_tokens()
        n += r8[0].available_tokens()
        n += r9[0].available_tokens()
        n += r10[0].available_tokens()
        n += r11[0].available_tokens()
        n += r12[0].available_tokens()
        n += r13[0].available_tokens()
        n += r14[0].available_tokens()
        n += r15[0].available_tokens()
        return n/15

class VsWetTrendsActor( AtomicActor ) :
    """ Actor that emulates the 'vs wet trends' chip
    from the TSD demo for the Matlab version of converge.
    It has the following inputs:
        
        - vp_ss_in
        - vp_sh_in (optional)
        
    Its outputs are:
        
        - vs_ss_out
        - vs_sh_out

    Computations for this chip are performed in function exec_script().
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports
        pin_vp_ss = self.get_port("vp_ss_in")
        pin_vp_sh = self.get_port("vp_sh_in")

        # get tokens from input ports
        vp_ss  = pin_vp_ss.get()
        vp_sh = pin_vp_sh.get()

        # execute the script
        self.value = self._exec_script(vp_ss, vp_sh)

        # display tokens
        #logger.debug( "%s fire inputs get %s for vp_ss" % (self.name, vp_ss) )
        #logger.debug( "%s fire inputs get %s for vp_sh" % (self.name, vp_sh) )
        #logger.debug( "%s fire input total is %s" % (self.name, self.value) )

        # get output ports and put values
        pout_vs_ss = self.get_port("vs_ss_out")
        pout_vs_sh = self.get_port("vs_sh_out")
        pout_vs_ss.put(self.value[0])
        pout_vs_sh.put(self.value[1])

        return

    def _exec_script(self, vpss, vpsh):
        # call function that implements the rock physics
        try:
            from cp.rockphysics.converge.shear.vs_wet_trends import vs_wet_trends
        except ImportError, e:
            import testoob.testing
            testoob.testing.skip("ImportError: " + str(e))
        output = vs_wet_trends(vpss, vpsh)
        return output

    def _create_ports(self) :
        self.new_port("vp_ss_in", True )
        self.new_port("vp_sh_in", True )
        self.new_port("vs_ss_out", False )
        self.new_port("vs_sh_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        p1 = self.get_port("vp_ss_in")
        p2 = self.get_port("vp_sh_in")
        r1 = p1.receivers()
        r2 = p2.receivers()
        n  = r1[0].available_tokens()
        n += r2[0].available_tokens()
        return n/2

class TsdSubgroup1Actor ( CompositeActor) :
    """ CompositeActor that forms a subgroup contained within
    the TSD demo. This subgroup contains the vs_wet_trends,
    biot_coeff_rhob, and tsd_model_rhob chips.
    The subgroup has the following inputs:
        
        - ca_k_wat_ss_in
        - ca_rho_wat_ss_in
        - ca_kg_ss_in
        - ca_rhog_ss_in
        - ca_rhob_ss_in
        - ca_vp_ss_in
        - ca_gg_ss_in
        - ca_k_wat_sh_in
        - ca_rho_wat_sh_in
        - ca_kg_sh_in
        - ca_rhog_sh_in
        - ca_rhob_sh_in
        - ca_vp_sh_in
        - ca_phit_in
        - ca_xsh_in
        
    Its outputs are:
        
        - ca_biot_ss_out
        - ca_biotmax_ss_out
        - ca_biot_sh_out
        - ca_biotmax_sh_out
        - ca_vp_wet_out
        - ca_vs_wet_out
        - ca_rhob_wet_out
        - ca_vp_sm_out
        - ca_vs_sm_out
        - ca_phi_sm_out
        - ca_rhob_sm_out
        - ca_phie_sm_out
        - ca_rhog_sm_out
        - ca_kg_sm_out
        - ca_vp_hs_out
        - ca_vs_hs_out
        - ca_phi_hs_out
        - ca_xlam_out
        - ca_xdis_out
        - ca_phi_ss_out
        - ca_phi_sh_out
    """
    def __init__(self, *args, **kw) :
        CompositeActor.__init__(self, *args, **kw)
        self.opaque=False
        self._create_ports()
        return

    def fire(self) :
        
        # some test cases will add a Director to this actor 
        # making it not transparent
        if not self.is_opaque :
            raise RuntimeError, \
                "TsdSubgroupActor composite actor fire has been called."
        
        #logger.debug( "******* TsdSubgroupActor '%s' fire" % self.name)
        return CompositeActor.fire(self)

    def _create_ports(self) :
        # input ports
        self.new_port("ca_vp_ss_in", True)
        self.new_port("ca_vp_sh_in", True)
        self.new_port("ca_rhob_ss_in", True)
        self.new_port("ca_rhog_ss_in", True)
        self.new_port("ca_kg_ss_in", True)
        self.new_port("ca_gg_ss_in", True)
        self.new_port("ca_rhob_sh_in", True)
        self.new_port("ca_rhog_sh_in", True)
        self.new_port("ca_kg_sh_in", True)
        self.new_port("ca_phit_in", True)
        self.new_port("ca_xsh_in", True)
        self.new_port("ca_k_wat_ss_in", True)
        self.new_port("ca_rho_wat_ss_in", True)
        self.new_port("ca_k_wat_sh_in", True)
        self.new_port("ca_rho_wat_sh_in", True)

        # output ports
        self.new_port("ca_biot_ss_out", False)
        self.new_port("ca_biotmax_ss_out", False)
        self.new_port("ca_biot_sh_out", False)
        self.new_port("ca_biotmax_sh_out", False)
        self.new_port("ca_vp_wet_out", False)
        self.new_port("ca_vs_wet_out", False)
        self.new_port("ca_rhob_wet_out", False)
        self.new_port("ca_vp_sm_out", False)
        self.new_port("ca_vs_sm_out", False)
        self.new_port("ca_phi_sm_out", False)
        self.new_port("ca_rhob_sm_out", False)
        self.new_port("ca_phie_sm_out", False)
        self.new_port("ca_rhog_sm_out", False)
        self.new_port("ca_kg_sm_out", False)
        self.new_port("ca_vp_hs_out", False)
        self.new_port("ca_vs_hs_out", False)
        self.new_port("ca_phi_hs_out", False)
        self.new_port("ca_xlam_out", False)
        self.new_port("ca_xdis_out", False)
        self.new_port("ca_phi_ss_out", False)
        self.new_port("ca_phi_sh_out", False)

        return

    def _available_tokens(self, port_name) :
        p = self.get_port(port_name)
        n = p.available_tokens()
        return n


class TsdSubgroup2Actor ( CompositeActor) :
    """ CompositeActor that forms a subgroup contained within
    the TSD demo. This subgroup contains the biot_coeff_rhob
    and tsd_model_rhob chips. The subgroup has the following inputs:
        
        - ca_k_wat_ss_in
        - ca_rho_wat_ss_in
        - ca_kg_ss_in
        - ca_rhog_ss_in
        - ca_rhob_ss_in
        - ca_vp_ss_in
        - ca_vs_ss_in
        - ca_gg_ss_in
        - ca_k_wat_sh_in
        - ca_rho_wat_sh_in
        - ca_kg_sh_in
        - ca_rhog_sh_in
        - ca_rhob_sh_in
        - ca_vp_sh_in
        - ca_vs_sh_in
        - ca_phit_in
        - ca_xsh_in

    Its outputs are:
        
        - ca_biot_ss_out
        - ca_biotmax_ss_out
        - ca_biot_sh_out
        - ca_biotmax_sh_out
        - ca_vp_wet_out
        - ca_vs_wet_out
        - ca_rhob_wet_out
        - ca_vp_sm_out
        - ca_vs_sm_out
        - ca_phi_sm_out
        - ca_rhob_sm_out
        - ca_phie_sm_out
        - ca_rhog_sm_out
        - ca_kg_sm_out
        - ca_vp_hs_out
        - ca_vs_hs_out
        - ca_phi_hs_out
        - ca_xlam_out
        - ca_xdis_out
        - ca_phi_ss_out
        - ca_phi_sh_out
    """
    def __init__(self, *args, **kw) :
        CompositeActor.__init__(self, *args, **kw)
        self.opaque=False
        self._create_ports()
        return

    def fire(self) :
        
        # some test cases will add a Director to this actor 
        # making it not transparent
        if not self.is_opaque :
            raise RuntimeError, \
                "TsdSubgroupActor composite actor fire has been called."
        
        #logger.debug( "******* TsdSubgroupActor '%s' fire" % self.name)
        return CompositeActor.fire(self)

    def _create_ports(self) :
        # input ports
        self.new_port("ca_vp_ss_in", True)
        self.new_port("ca_vs_ss_in", True)
        self.new_port("ca_vp_sh_in", True)
        self.new_port("ca_vs_sh_in", True)
        self.new_port("ca_rhob_ss_in", True)
        self.new_port("ca_rhog_ss_in", True)
        self.new_port("ca_kg_ss_in", True)
        self.new_port("ca_gg_ss_in", True)
        self.new_port("ca_rhob_sh_in", True)
        self.new_port("ca_rhog_sh_in", True)
        self.new_port("ca_kg_sh_in", True)
        self.new_port("ca_phit_in", True)
        self.new_port("ca_xsh_in", True)
        self.new_port("ca_k_wat_ss_in", True)
        self.new_port("ca_rho_wat_ss_in", True)
        self.new_port("ca_k_wat_sh_in", True)
        self.new_port("ca_rho_wat_sh_in", True)

        # output ports
        self.new_port("ca_biot_ss_out", False)
        self.new_port("ca_biotmax_ss_out", False)
        self.new_port("ca_biot_sh_out", False)
        self.new_port("ca_biotmax_sh_out", False)
        self.new_port("ca_vp_wet_out", False)
        self.new_port("ca_vs_wet_out", False)
        self.new_port("ca_rhob_wet_out", False)
        self.new_port("ca_vp_sm_out", False)
        self.new_port("ca_vs_sm_out", False)
        self.new_port("ca_phi_sm_out", False)
        self.new_port("ca_rhob_sm_out", False)
        self.new_port("ca_phie_sm_out", False)
        self.new_port("ca_rhog_sm_out", False)
        self.new_port("ca_kg_sm_out", False)
        self.new_port("ca_vp_hs_out", False)
        self.new_port("ca_vs_hs_out", False)
        self.new_port("ca_phi_hs_out", False)
        self.new_port("ca_xlam_out", False)
        self.new_port("ca_xdis_out", False)
        self.new_port("ca_phi_ss_out", False)
        self.new_port("ca_phi_sh_out", False)

        return

    def _available_tokens(self, port_name) :
        p = self.get_port(port_name)
        n = p.available_tokens()
        return n

class TsdSubgroup3Actor ( TsdSubgroup2Actor ) :
    """Inherits exact behavior from TsdSubgroup2Actor."""
    pass

class TsdLevel3Actor ( CompositeActor ) :
    def __init__(self, *args, **kw) :
        CompositeActor.__init__(self, *args, **kw)
        self.opaque=False
        self._create_ports()
        return

    def fire(self) :
        
        # some test cases will add a Director to this actor 
        # making it not transparent
        if not self.is_opaque :
            raise RuntimeError, \
                "TsdLevel3Actor composite actor fire has been called."
        
        #logger.debug( "******* TsdLevel3Actor '%s' fire" % self.name)
        return CompositeActor.fire(self)

    def _create_ports(self) :
        # input ports
        self.new_port("ca_level_3_in", True)

        # output ports
        self.new_port("ca_level_3_out", False)

        return

    def _available_tokens(self, port_name) :
        p = self.get_port(port_name)
        n = p.available_tokens()
        return n

class TsdMultiplyBy2Actor ( AtomicActor ) :
    """ Actor that emulates a multiplier chip, when the
    data on the input port is multiplied by the scalar 2.
    It has the following input:
        
        - tsd_mult_in
        
    Its output is:
        
        - tsd_mult_out
    """
    def __init__(self, *args, **kw) :
        AtomicActor.__init__(self, *args, **kw)
        self._create_ports()
        return

    def fire(self) :

        # get the input ports
        pin = self.get_port("tsd_mult_in")

        # get tokens from input ports
        in_sequence = pin.get()

        # execute the script
        self.value = self._exec_script(in_sequence)

        # display tokens
        #logger.debug( "%s fire inputs get %s for in_sequence" % (self.name, in_sequence) )
        #logger.debug( "%s fire input output is %s" % (self.name, self.value) )

        # get output ports and put values
        pout = self.get_port("tsd_mult_out")
        pout.put(self.value[0])

        return

    def _exec_script(self, in_sequence) :
        # kludge up an output tuple
        output  = 2 * in_sequence
        return (output,)

    def _create_ports(self) :
        self.new_port("tsd_mult_in", True )
        self.new_port("tsd_mult_out", False )
        return

    def _available_tokens(self) :
        #TODO: ?
        p1 = self.get_port("tsd_mult_in")
        r1 = p1.receivers()
        n  = r1[0].available_tokens()
        return n
    
### EOF ######################################################################
