import unittest

from enthought.logger import logger

import time
from enthought.util.numerix import array
from enthought.util.testingx import assert_array_almost_equal

from tsd_no_subgroup_test_case     import TsdNoSubgroup
from tsd_with_subgroup_1_test_case import TsdWithSubgroup1
from tsd_with_subgroup_2_test_case import TsdWithSubgroup2

num_execs = 10

class TsdTimeNoSubgroup ( TsdNoSubgroup ) :
            
    def benchmark_time_computation( self ) :
        logger.debug("----> check_time_computation")
        
        # set increment between adjacent depth indices
        increment = .6096

        for num_repeats in [0, 49, 99, 149, 199] :

            # set input data
            self.tsd_data_in_actor.build_input_arrays(num_repeats, increment)

            # start time
            start_time = time.clock()

            elapsed_time = 0.0

            for doexec in range(num_execs) :
                # don't need to set value for input, that's done in TsdDataInActor
                self.top.iterate(1)
        
                # get end and elapsed time
                end_time = time.clock()
                elapsed_time += end_time - start_time

            seq_length = len(self.tsd_data_in_actor.outputArrays['depth_out'])
            logger.info("Running TestTsdTimeNoSubgroup %d times, seq length %d took %.3f sec" % \
                  (num_execs, seq_length, elapsed_time))

        return

class TsdTimeNoSubgroupTestCase ( TsdTimeNoSubgroup, unittest.TestCase ) :
    pass

class TsdTimeSubgroup1 ( TsdWithSubgroup1 ) :
            
    def benchmark_time_computation( self ) :
        logger.debug("----> check_time_computation")
        
        # set increment between adjacent depth indices
        increment = .6096

        for num_repeats in [0, 49, 99, 149, 199] :

            # set input data
            self.tsd_data_in_actor.build_input_arrays(num_repeats, increment)

            # start time
            start_time = time.clock()

            elapsed_time = 0.0

            for doexec in range(num_execs) :
                # don't need to set value for input, that's done in TsdDataInActor
                self.top.iterate(1)
        
                # get end and elapsed time
                end_time = time.clock()
                elapsed_time += end_time - start_time

            seq_length = len(self.tsd_data_in_actor.outputArrays['depth_out'])
            logger.info("Running TestTsdTimeSubgroup1 %d times, seq length %d took %.3f sec" % \
                  (num_execs, seq_length, elapsed_time))

        return

class TsdTimeSubgroup1TestCase ( TsdTimeSubgroup1, unittest.TestCase ) :
    pass

class TsdTimeSubgroup2 ( TsdWithSubgroup2 ) :
            
    def benchmark_time_computation( self ) :
        logger.debug("----> check_time_computation")
        
        # set increment between adjacent depth indices
        increment = .6096

        for num_repeats in [0, 49, 99, 149, 199] :

            # set input data
            self.tsd_data_in_actor.build_input_arrays(num_repeats, increment)

            # start time
            start_time = time.clock()

            elapsed_time = 0.0

            for doexec in range(num_execs) :
                # don't need to set value for input, that's done in TsdDataInActor
                self.top.iterate(1)
        
                # get end and elapsed time
                end_time = time.clock()
                elapsed_time += end_time - start_time

            seq_length = len(self.tsd_data_in_actor.outputArrays['depth_out'])
            logger.info("Running TestTsdTimeSubgroup2 %d times, seq length %d took %.3f sec" % \
                  (num_execs, seq_length, elapsed_time))

        return

class TsdTimeSubgroup2TestCase ( TsdTimeSubgroup2, unittest.TestCase ) :
    pass
        
class TsdTimeDirect :

    def build_input_arrays(self, num_repeat, increment) :

          # build the input arrays
        self.watdth = 1362.1512
        self.oftemp = 1.6667
        self.tgrad  = 1.1
        self.pgrad  = 0.465
        self.depth  = build_array(array((1981.2, 1981.8096, 1982.4192, 1983.0288, 1983.6384)),
                num_repeat, increment)
        self.salw_ss = 30000.0
        self.salw_sh = 15000.0
        self.vp_ss  = build_array(array((3.0, 3.0, 3.107, 2.922, 3.095)),
                num_repeat)
        self.vp_sh  = build_array(array((2.7, 2.7, 2.8,   2.6,   2.65)),
                num_repeat)
        self.kg_ss  = build_array(array((37.0, 37.0, 36.0, 38.0, 40.0)),
                num_repeat)
        self.rhog_ss = build_array(array((2.65, 2.65, 2.64, 2.63, 2.66)),
                num_repeat)
        self.rhob_ss = build_array(array((2.3, 2.3, 2.349, 2.355, 2.448)),
                num_repeat)
        self.kg_sh = build_array(array((25.0, 25.0, 27.0, 26.0, 38.0)),
                num_repeat)
        self.rhog_sh = build_array(array((2.6, 2.6, 2.582, 2.568, 2.305)),
                num_repeat)
        self.rhob_sh = build_array(array((2.5, 2.5, 2.478, 2.492, 2.506)),
                num_repeat)
        self.gg_ss = build_array(array((44.0, 44.0, 42.0, 41.0, 43.0)),
                num_repeat)
        self.phit = build_array(array((0.15, 0.15, 0.16, 0.145, 0.18)),
                num_repeat)
        self.xsh = build_array(array((0.6876, 0.6880, 0.6430, 0.6520, 0.6380)),
                num_repeat)
        return

    def benchmark_time_computation( self ) :
        logger.debug("----> check_time_computation")

        # set increment between adjacent depth indices
        increment = .6096

        for num_repeats in [0, 49, 99, 149, 199] :

            # set input data
            self.build_input_arrays(num_repeats, increment)

            # start time
            start_time = time.clock()

            elapsed_time = 0.0
            for doexec in range(num_execs) :
                # call functions that implements the rock physics
                (tform, ppore) = temp_press_linear(self.watdth, self.oftemp,
                    self.tgrad, self.pgrad, self.depth)
        
                (rho_wat_ss, k_wat_ss) = brine_properties(ppore, tform,
                    self.salw_ss)
        
                (rho_wat_sh, k_wat_sh) = brine_properties(ppore, tform,
                    self.salw_sh)
        
                (vs_ss, vs_sh) = vs_wet_trends(self.vp_ss, self.vp_sh)
        
                (biot_ss, biotmax_ss, biot_sh, biotmax_sh) =\
                    biot_coeff_rhob(k_wat_ss, rho_wat_ss, self.kg_ss, self.rhog_ss,
                    self.rhob_ss, self.vp_ss, vs_ss, k_wat_sh, rho_wat_sh,
                    self.kg_sh, self.rhog_sh, self.rhob_sh, self.vp_sh, vs_sh)
        
                (vp_wet, vs_wet, rhob_wet, vp_sm, vs_sm, phi_sm, rhob_sm, phie_sm,
                    rhog_sm, kg_sm, vp_hs, vs_hs, phi_hs, xlam, xdis, phi_ss,
                    phi_sh) = tsd_model_rhob(self.vp_ss, vs_ss, self.rhob_ss,
                    self.rhog_ss, self.kg_ss, self.gg_ss, self.vp_sh, vs_sh,
                    self.rhob_sh, self.rhog_sh, self.kg_sh, self.phit, self.xsh,
                    rho_wat_ss, rho_wat_sh)
        
                # get end and elapsed time
                end_time = time.clock()
                elapsed_time += end_time - start_time

            logger.info("Running TestTsdTimeDirect %d times, seq length %d took %.3f sec" % \
                  (num_execs, len(self.depth), elapsed_time))

        return

class TsdTimeDirectTestCase( TsdTimeDirect, unittest.TestCase ) :
    pass

### EOF ######################################################################
