import unittest

from enthought.logger import logger

from enthought.graph import SdfDirector

from actors import *

# TwoNodeExecution is defined this way so that it can be imported
# and subclassed without unittest running its test each time it is imported.
        
class TwoNodeExecution :

    def __init__( self, *args, **kw ) :
        # based on the selection of SdfDirector with FifoReceiver...
        self.expected_available_tokens = 0
        
        return
        
        
    def setUp(self) :
        logger.debug("-------------------------------------------------------")
        self.top = CompositeActor('top')

        self.output_actor = OutputActor('output_actor', value='a')
        self.output_actor.container = self.top

        self.input_actor = InputActor('input_actor')
        self.input_actor.container = self.top

        self.top.connect( self.output_actor.get_port('out'), 
                        self.input_actor.get_port('in'))

        self.director = SdfDirector()
        self.director.container = self.top

        logger.debug("test execution begins...")
        
        self.top.preinitialize()
        self.top.initialize()
        return

    def tearDown( self ) :
        logger.debug("test execution ends.")
        self.top.wrapup()
        return
        
    def test_simple_token( self ) :
        self.output_actor.value = 1
        self.top.iterate(1)
        self.assertEqual( 1, self.input_actor.value, \
            "input_actor has unexpected value: %s" % self.input_actor.value )    
        return

    def test_list_token( self ) :
        self.output_actor.value = [1,2,3]
        self.top.iterate(1)
        self.assertEqual( [1,2,3], self.input_actor.value,
            "input_actor has unexpected value: %s" % self.input_actor.value )    
        self.assertEqual( self.expected_available_tokens,
            self.input_actor._available_tokens(),
            "input_actor has incorrect number of available tokens")
        return

    def test_multiple_iterations( self ) :
        self.output_actor.value_list = [1,2,3]
        self.top.iterate(2)
        
        self.assertEqual( 2, self.input_actor.value,
            "input_actor has unexpected value: %s" % self.input_actor.value )    
        self.assertEqual( self.expected_available_tokens,
            self.input_actor._available_tokens(),
            "input_actor has incorrect number of available tokens")
        return
        
    def test_multiple_iterations_list( self ) :
        self.output_actor.value_list = [[1,2,3], [4,5,6]]
        self.top.iterate(2)
        self.assertEqual( [4,5,6], self.input_actor.value, \
            "input_actor has unexpected value: %s" % self.input_actor.value )    
        self.assertEqual( self.expected_available_tokens,
            self.input_actor._available_tokens(),
            "input_actor has incorrect number of available tokens")
        return


class TwoNodeExecutionTestCase( TwoNodeExecution, unittest.TestCase ) :
    
    def __init__( self, *args, **kw ) :
        TwoNodeExecution.__init__( self, *args, **kw )
        unittest.TestCase.__init__( self, *args, **kw )
        return
    
### EOF ######################################################################
