""" TokenProxy implements proxy behavior for ValuePort tokens. """

from enthought.traits.api import HasTraits, Property, Trait, Str

class TokenProxy( HasTraits ):
    """ TokenProxy implements proxy behavior for ValuePort tokens. """

    # Value can be set via a name from the workspace naming system or an object.
    # It will always return the object named rather than the name.
    value = Property

    # When value has been set via a name, the name property can be used to
    # retrieve that name.
    name = Property

    # Name of a Token in the workspace naming system.
    _name = Trait( None, Str, None )

    def _set_value( self, value ):
        """ Change the object value will return: value is either a workspace
        naming system name or an object. """
        old = self._name or self._obj

        if isinstance(value, basestring):
            self._name = value
            self._obj = None

        else:
            self._obj = value
            self._name = None

        self.trait_property_changed( 'value', old, self.value )

        return

    def _get_value( self ):
        """ Returns the Log object specified via _set_value. """
        if self._obj is None:
            if self._name is not None:
                obj = self._workspace.lookup( self._name )
                if isinstance( obj, TokenProxy ):
                    obj = obj.value
                self._obj = obj

        return self._obj

    def _get_name( self ):
        """ Returns the name of the proxied object if set via a name or None.
        """
        return self._name

#### EOF ######################################################################
