from sets import Set

from enthought.logger import logger

from scheduler import Scheduler
from schedule import Schedule

class TopologyScheduler( Scheduler ) :

	def __init__(self, *args, **kw) :
		Scheduler.__init__(self, *args, **kw)
		
		return
		
    #### Protected Methods ####################################################

	def _create_schedule( self ) :
		""" Reachable nodes in the CompositeActor containing this scheduler
		are scheduled to run according to the topological ordering based on
		the input dependencies of the Actors.
		
		Current implementation assumes that all data rates are one.
		"""
		
		ordered_actors = [] 

		# determine ancestors of each Actor
		ancestor_map = self._create_ancestor_map()
		
		# map is unordered, create ordered_actors list topology order
		# actors with no ancestors are first, then their decendents and so on.
		one_satisfied = True
		while one_satisfied and len(ancestor_map) > 0	:
			one_satisfied = False
			for actor, ancestors in ancestor_map.items() :
				if len(ancestors) == 0 :
					# input dependencies satisfied
					ordered_actors.append( actor )
					del ancestor_map[ actor ]
					one_satisfied = True
					# remove actor from input dependencies of other actors
					for ancestors in ancestor_map.values() :
						ancestors.discard( actor )
		
		if len(ancestor_map) > 0 :
			raise RuntimeError, "Topology Schedule cannot be found. "\
				+ "Unsatisfied ancestors for actors: %s" \
				% [ actor.full_name for actor in ancestor_map.keys() ]
		
		del ancestor_map
		
		return Schedule( items=ordered_actors, iterations=1 )
		
	def _create_ancestor_map( self ) :
		""" return a dictionary with an entry for each reachable actor.
		key: Actor, value: Set( Actor )
		Each Actor in the set is an ancestor of key Actor.
		"""
		ancestor_map = {}
		
		for actor in self._container.reachable_node_list() :
			# make sure actor has one
			ancestors = self._get_ancestor_set( ancestor_map, actor )

			# make sure dependents know about actor
			for dependent in actor.dependents() :
				dep_ancestors = self._get_ancestor_set( ancestor_map, dependent )
				dep_ancestors.add( actor )
		
		return ancestor_map
		
	def _get_ancestor_set( self, ancestor_map, actor ) :
		""" From the ancestor_map return the Set of ancestors for the actor.
		The set is created and added to the ancestor_map if necessary.
		"""
		set = None

		if not ancestor_map.has_key( actor ) :
			set = Set()
			ancestor_map[ actor ] = set
		else :
			set = ancestor_map[ actor ]

		return set


		
        
#### EOF ######################################################################        	
