""" Typing information for IOPorts, use as a mixin. """

from enthought.traits.api import HasTraits, Any, Str
from enthought.units.unit_traits import unit_family_trait

class TypedPort( HasTraits ):
    """ Typing information for IOPorts, use as a mixin. 
    
    Subclass from both IOPort and TypedPort to defined a typed port class.
    These subclasses convert data recevied that is of external_type into
    internal_type when the containing actor calls get.  Likewise, the port
    converts internal_typed data put on the port by the actor into external_type
    data before forwarding on to the destination receivers.
    """
    
    # name of the required datatype at the dataflow level.
    external_type = 'object'

    # name of the required datatype required by the Actor containing the port.
    internal_type = 'object'

    # Units: e.g. km/s.
    units = Any        # FIXME: when there is a propert trait type for units
    
    # Units family name: e.g. pvelocity.
    family_name = unit_family_trait
    
    # Units system name: e.g. KGS
    units_system_name = Str('KGS')

### EOF #######################################################################
