from enthought.logger import logger

from enthought.traits.api import HasTraits
from enthought.traits.api import Instance

from value_port import ValuePort
from receiver import Receiver

class ValuePortReceiver( Receiver ) :
    """ Implments a Receiver with no input buffer and where get retrieves
    the returned value directly from the sending port which is required to
    be an instance of ValuePort.

    Use with input ports linked to a ValuePort output port.
    """
    
    source_port = Instance( ValuePort )
    
    #### Receiver Methods ####################################################

    def clear( self ) :
        """ Clear this receiver of any contained tokens. """
        pass
        
    def available_tokens( self ) :
        """ Return the number of tokens available for get to return."""
        return 1

    def has_tokens( self, n=1 ) :
        """ Return True if the receiver has n tokens for get to return. """
        return True
    
    def get( self, n=1 ) :
        """ Return a token or list of tokens, when n > 1, from this receiver.
        """
        if n > 1 :
            raise ValueError, \
                "ValuePortReceiver only provides single tokens %s" \
                % self._container.full_name

        # fetch the value and unit convert if needed
        port = self._container

        value = self._get_value()
        if value is not None and hasattr(value, 'units') \
            and value.units != port.units:
                value = value.change_unit_system(port.units_system_name)

        return value

    def _get_value(self):
        """ Return the current value with no error checking or units conversion
        """
        return self.source_port.token
        
    def has_room( self, n=1 ) :
        """ Return True if the receiver can accept n inputs. 
        This implementation uses an unbounded FIFO.
        """
        return True

    def put( self, token ) :
        """ Put a token into this recevier.
        """
        raise ValueError, "Attempted put on ValuePortReceiver: %s" \
            % self._container.full_name
        
        return    
        
    
#### EOF ######################################################################    
