""" utility class for building bundles """

noise_words = ( '_pert', '_reg', '_' )

suffix_map  = ( 
    ( '_ss',     'sand'         ),
    ( '_sh',     'shale'        ),
    ( '_wat',    'water'        ),
    ( '_hs',     'hard shale'   ),
    ( '_sm',     'sandy member' ),
    ( '_bh',     'borehole'     ),
    ( '_watsh',  'shale water'  ),
    ( '_f',      'fluid'        ),
    ( '_sub',    'mixed'        ),
    ( '_wet',    'wet'          ),
    ( '_lith',   'lith'         ),
    ( '_nom',    'nom'          ),
    ( 'phit',    'total'        ),
    ( 'xsh',     'total'        ),
    ( 'xlam',    'total'        ),
    ( 'xdis',    'total'        ),
    ( 'ppore',   'env'          ),
    ( 'tform',   'env'          ),
    ( '_repeat', 'repeat'       ),
)    

class BundleBuilder:
    """ utility class for building bundles
    """
    
    def build_bundles ( self, items ):
        """ A bundle is a tuple containing a label and list of port names collected
        together under that common label.
        The label is derived from the suffix on the port names, thus all ports with
        the same suffix are collected under the same label.
        This function takes the list of input items, port names, and returns
        a list of bundles.
        """
        
        bundle_dict = {}
        bundles    = []
        
        for index, name in enumerate( items ):
            name = self.remove_noise( name )
            
            for suffix, bundle_name in suffix_map:
                if self.ends_with( name, suffix ):
                    if suffix == '_repeat':
                        name = name[:-7]
                    break
            else:
                bundle_name = name
                
            if bundle_name not in bundle_dict:
                bundle_dict[ bundle_name ] = bundle = [ bundle_name, [] ]
                bundles.append( bundle )
            
            bundle_dict[ bundle_name ][1].append( ( name, index ) )
                
        return bundle_dict

    def remove_noise ( self, string ):
        """ Removes any noise words from a specified string.
        """
        while True:
            for word in noise_words:
                if self.ends_with( string, word ):
                    string = string[ : -len( word ) ]
                    break
            else:
                return string
                
    def ends_with ( self, string, suffix ):
        """ Returns whether one strings ends with another string.
        """
        return (suffix == string[ -len( suffix ): ])