""" Constant value Log actor """

from enthought.util.numerix import array

from enthought.graph \
    import AtomicActor
    
from enthought.traits.api import Any, Instance, Str, This, Tuple

from converge_actor \
    import ConvergeActor, \
            ConvergeOutputBundle, ConvergeOutputPort, \
            ConvergeInputBundle, ConvergeInputPort

from expression_contact_controller import ExpressionContactController

# input bundle that will hold all input ports
INPUT_BUNDLE = 'inputs'

# input port that will cause new input ports to be created
ADD_INPUT_NAME = 'new input'


class ExpressionActor( ConvergeActor ):
    """ An actor that outputs a log with a constant value.
    The name is used as the value.
    """
    
    chip_name = Str('Expression')

    help = """Expression
        In a namespace that contains the input port values, the chip's name
        is evaluated as an expression.  The result of that expressin is the
        chip's output value."""
    
    class_input_bundles = [
        ConvergeInputBundle(
            name  = INPUT_BUNDLE,
            ports = [
                ConvergeInputPort(
                    name        = ADD_INPUT_NAME,
                    description = 'connect to this port to create a new input',
                    units       = 'KM/S',
                    index       = 0 )            
            ] ),
        ]
    
    class_output_bundles = [
        ConvergeOutputBundle(
            name  = 'out',
            ports = [
                ConvergeOutputPort(
                    name        = 'out',
                    description = 'the result of evaluating the name as an expression',
                    units       = 'KM/S',
                    index       = 0 )            
            ] ),
        ]
    
    
    # one bundle with out output, the constant log
    instance_output_bundles = Any( () )
    
    # one bundle with one input, the log index
    instance_input_bundles = Any( () )

    #fixme: the model shouldn't know about the controller
    contact_controller = Instance( ExpressionContactController )

    # Redefine the function trait to be the fire method.
    function = Tuple(This.fire,)
    

    
    #### Object Methods #######################################################
    
    def __init__ ( self, *args, **traits ):
        """ Create a new ConstantLog object """
        args = args or ( self.__class__.__name__, )
        super( ExpressionActor, self ).__init__( *args, **traits )
        
        #fixme: the model shouldn't know about the controller
        self.contact_controller = ExpressionContactController()
        
        port = self.get_port(ADD_INPUT_NAME)
        port.is_optional = True
        
        return

    def add_input_port( self, name ):
        """ create a new input port """
        bundle = self.instance_input_bundles[0]

        ports = bundle.ports
        
        port = self._create_input_port( name, name, ports, self.input_ports )

        # set the ports back in the bundle so that it recognizes the changes
        bundle.ports = ports
        
        return port
        
    #### Actor methods ########################################################
    def fire ( self ):
        """ Runs the function associated with the actor """
        
        locals = {}
        
        
        for port in self.input_port_list() :
            if port.has_tokens() :
                value = port.get()
            else:
                value = None
                
            locals[port.name] = value
        
        value = eval(self.name, globals(), locals )
        
        port = self.get_port( 'out' )
        port.put( value )
        
        return

    #### Protected methods ####################################################
        
    def _create_input_port(self, name, description, bundle_ports, graph_ports):
        """ add the name as a port... """
        
        position = len( graph_ports )
        
        # create the meta port...
        port_meta = ConvergeInputPort(
                                name        = name,
                                description = description,
                                units       = 'None',
                                index       = position,
                                is_optional = False,
                                internal_type = 'Any',
                                external_type = 'Any',
                                actor         = self
                                )
        
        # add the meta to the bundle
        bundle_ports.append( port_meta )
        
        # create the actual port
        port = self.new_port( port_meta.name, True, port_meta )

        graph_ports.append( port )
        
        return port
        
# Prototype instance of this class to be loaded by the plugin          
expression_actor_prototype = ExpressionActor()
    
#### EOF ######################################################################
