"""  Defines the contact controller for ExpressionActor """

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------

from string import lower

from enthought.enable.om \
    import OMContactController, OMLink, OMContact

from enthought.graph.vpl.graph_contact_controller  \
    import GraphContactController, connect_ports

from enthought.units.unit_system import UnitSystem

from converge_actor import ConvergeInputPort

# FIXME: circular imports.  Need controller factory to break
# from expression_actor import INPUT_BUNDLE, ADD_INPUT_NAME

# input bundle that will hold all input ports
INPUT_BUNDLE = 'inputs'

# input port that will cause new input ports to be created
ADD_INPUT_NAME = 'new input'


class ExpressionContactController ( GraphContactController ):
        
### Protected method overrides ################################################
    def _connect_ports( self, output_actor, output_bundle, opl, 
                              input_actor, input_bundle, ipl, canvas ):
        """ connect as many ports as possible from output to input """
        
        # Remember, only the input controller makes decisions and
        # this one only allows connections to the special input bundle / port.
        if input_bundle.name not in [ INPUT_BUNDLE, ADD_INPUT_NAME ]:
            return 0
            
        connections = 0

        # use the instance_input_bundles because there is only one
        # that contains all the input ports
        instance_input_bundle = input_actor.instance_input_bundles[0]

        # determine if there is one or more names in the output bundle that
        # don't yet exist in the input bundle and thus could be connected.
        in_port_names = [ in_port.name 
                        for in_port in instance_input_bundle.ports ]
            
        output_ports = output_bundle.ports
        for output_port in output_ports :
            if output_port.name not in in_port_names:
                # create the port
                port = input_actor.add_input_port( output_port.name )
                
                # connect it
                connections += 1
                connect_ports( canvas.data, opl[ output_port.index ], port)

                #output names should be unique, but just in case...
                in_port_names.append( output_port.name )

        return connections
                                

    def _connectable(self, output, input):
        """ return connectability of output and input.
        protected method called by connectable after it has 
        worked out which is which
        This controller is always representing the input side.
        This controller will allow anything to be connected to the 'special'
        input port as long as there is not already an input port with the
        same name.
        """
        
        is_connectable = 0

        # Remember, only the input controller makes decisions and
        # this one only allows connections to the special input bundle / port.
        if lower(input.label) in [ INPUT_BUNDLE, ADD_INPUT_NAME ]:
            # attempt to connect to the special port,

            output_actor, output_bundle, output_dir = output.data
            input_actor,  input_bundle,  input_dir  = input.data
    
            # Get the list of output and input ports to check for allowed 
            # connections.  The contact has no reference to the bundle other
            # than the name.  Names are unique thus, only one bundle will
            # match; thus the [0].
###            output_bundle = [ b for b in output_actor.output_bundles
###                               if lower(output.label) == lower(b.name) ][0]
    
            # use the instance_input_bundles because there is only one
            # that contains all the input ports
###            input_bundle = input_actor.instance_input_bundles[0]
    
            # determine if there is one or more names in the output bundle that
            # don't yet exist in the input bundle and thus could be connected.
            in_port_names = [ lower(in_port.name) for in_port in input_bundle.ports ]
            
            for out_port in output_bundle.ports:
                if lower(out_port.name) not in in_port_names:
                    is_connectable = 2
                    break
            
            else :
                # for every output port the actor already has an input port
                # with the same name
                is_connectable = 0

        return is_connectable
    
        
#### EOF ######################################################################    
