#-------------------------------------------------------------------------------
#  
#  Defines classes for editing the bundles/ports of a Graph Actor class.  
#  
#  Written by: David C. Morrill
#  
#  Date: 04/01/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------
    
from converge_actor \
    import ConvergeActorTraits, ConvergeInputBundle, ConvergeOutputBundle, \
           ConvergeInputPort, ConvergeOutputPort

from enthought.traits.api \
    import Instance
    
from enthought.traits.ui.api \
    import View, Item, Handler, TreeEditor, TreeNode
    
from enthought.traits.ui.menu \
    import Menu, Action, Separator
    
from enthought.traits.ui.wx.tree_editor \
    import CutAction, CopyAction, PasteAction, RenameAction
    
#-------------------------------------------------------------------------------
#  Views associated with various tree nodes:  
#-------------------------------------------------------------------------------
        
no_view            = View(                         kind = 'subpanel' )
actor_view         = View( 'name',                 kind = 'subpanel' )
input_bundle_view  = View( 'name',                 kind = 'subpanel' )
output_bundle_view = View( 'name', '_', 'visible', kind = 'subpanel' )
input_port_view    = View( 'name',                 kind = 'subpanel' )
output_port_view   = View( 'name', '_', 'visible', kind = 'subpanel' )

#-------------------------------------------------------------------------------
#  Node menues:  
#-------------------------------------------------------------------------------

bundle_menu = Menu(
    CutAction,
    CopyAction,
    PasteAction,
    Separator(),
    Action( name         = 'Delete',
            action       = 'editor._menu_delete_node',
            enabled_when = 'handler.can_delete_bundle(object)' ),
    Separator(),
    RenameAction,
    name = 'popup'
)

#-------------------------------------------------------------------------------
#  Chip tree editor:  
#-------------------------------------------------------------------------------
        
chip_editor = TreeEditor( 
    nodes = [
        TreeNode( node_for   = [ ConvergeActorTraits ],
                  auto_open  = True,
                  copy       = False,
                  delete     = False,
                  children   = '',
                  view       = actor_view,
                  label      = 'name' ),
        TreeNode( node_for   = [ ConvergeActorTraits ],
                  auto_open  = True,
                  copy       = False,
                  children   = 'instance_input_bundles',
                  label      = '=Inputs',
                  view       = no_view,
                  add        = [ ConvergeInputBundle ] ),
        TreeNode( node_for   = [ ConvergeActorTraits ],
                  auto_open  = True,
                  copy       = False,
                  children   = 'instance_output_bundles',
                  label      = '=Outputs',
                  view       = no_view,
                  add        = [ ConvergeOutputBundle ] ),
        TreeNode( node_for   = [ ConvergeInputBundle ],
                  name       = 'Connection Point',
                  #auto_close = True,
                  copy       = False,
                  delete     = False,
                  children   = 'ports',
                  label      = 'name',
                  view       = input_bundle_view,
                  menu       = bundle_menu,
                  move       = [ ConvergeInputPort ] ),
        TreeNode( node_for   = [ ConvergeOutputBundle ],
                  name       = 'Connection Point',
                  #auto_close = True,
                  copy       = False,
                  delete     = False,
                  children   = 'ports',
                  label      = 'name',
                  view       = output_bundle_view,
                  menu       = bundle_menu,
                  move       = [ ConvergeOutputPort ] ),
        TreeNode( node_for   = [ ConvergeInputPort ],
                  view       = input_port_view,
                  label      = 'name' ),
        TreeNode( node_for   = [ ConvergeOutputPort ],
                  view       = output_port_view,
                  label      = 'name' )
    ]
)
    
#-------------------------------------------------------------------------------
#  'GraphBundleEditor' class:  
#-------------------------------------------------------------------------------
        
class GraphBundleEditor ( Handler ):
    
    #---------------------------------------------------------------------------
    #  Trait definitions:  
    #---------------------------------------------------------------------------
        
    # Actor being edited:
    actor = Instance( ConvergeActorTraits )
    
    #---------------------------------------------------------------------------
    #  Trait editor views:  
    #---------------------------------------------------------------------------
        
    # Define the View to use:
    traits_view = View( [ Item( name      = 'actor', 
                                id        = 'actor',
                                editor    = chip_editor, 
                                resizable = True ), '|<>' ],
                        title     = 'Converge Connection Editor',
                        id        = 'enthought.graph.vpl.graph_bundle_editor',
                        kind      = 'livemodal',
                        help      = True,
                        help_id   = "enlib|HID_Converge_Connection_Editor_dlg",
                        resizable = True,
                        width     = .4,
                        height    = .4 )
                        
    #---------------------------------------------------------------------------
    #  Returns whether or not a bundle can be deleted or not:  
    #---------------------------------------------------------------------------
    
    def can_delete_bundle ( self, bundle ):
        return (len( bundle.ports ) == 0)

