#-------------------------------------------------------------------------------
#  
#  Defines the GraphCanvasToolbar class that manages the toolbar for an graph  
#  canvas.
#  
#  Written by: David C. Morrill
#  
#  Date: 05/19/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------

from enthought.traits.api \
    import HasPrivateTraits, Instance, Any
    
from enthought.traits.ui.menu \
    import Action, Separator, ToolBar
    
from enthought.enable.om \
    import OMMenuController
    
from enthought.pyface.image_resource \
    import ImageResource

from graph_canvas \
    import GraphCanvas
    
from graph_canvas_controller \
    import graph_canvas_controller
    
from converge_actor \
    import ConvergeCompositeActor
    
#-------------------------------------------------------------------------------
#  'GraphCanvasToolbar' class:  
#-------------------------------------------------------------------------------

class GraphCanvasToolbar ( HasPrivateTraits ):
    
    #---------------------------------------------------------------------------
    #  Trait definitions:  
    #---------------------------------------------------------------------------
        
    # Undo action:
    undo      = Instance( Action, 
                          { 'name':    'Undo',
                            'action':  'on_undo',
                            'enabled': False,
                            'image':   ImageResource( 'undo16.png' ) } )
                  
    # Redo action:                            
    redo      = Instance( Action, 
                          { 'name':    'Redo',
                            'action':  'on_redo',
                            'enabled': False,
                            'image':   ImageResource( 'redo16.png' ) } )
                            
    # Run the current dataflow represented by the graph:
    run       = Instance( Action, 
                          { 'name':    'Run',
                            'action':  'run',
                            'image':   ImageResource( 'run16.png' ) } )
                            
    # Edit the selected component action:                            
    edit      = Instance( Action, 
                          { 'name':    'Edit',
                            'action':  'on_edit',
                            'enabled': False,
                            'image':   ImageResource( 'edit16.png' ) } )
                  
    # Delete selected components action:                            
    delete    = Instance( Action,
                          { 'name':    'Delete',
                            'action':  'delete_action',
                            'enabled': False,
                            'image':   ImageResource( 'delete16.png' ) } )
                               
    # Create a composite component from the selected components:                            
    collapse  = Instance( Action,
                          { 'name':    'Condense',
                            'action':  'on_collapse_composite',
                            'enabled': False,
                            'image':   ImageResource( 'collapse16.png' ) } )
                                   
    # Expand the selected composite components action:                            
    expand    = Instance( Action,
                          { 'name':    'Expand',
                            'action':  'on_expand_composite',
                            'enabled': False,
                            'image':   ImageResource( 'expand16.png' ) } )
                                 
    # Show all components in their collapsed state action:                            
    collapsed = Instance( Action,
                          { 'name':    'Min',
                            'action':  'on_collapse_view',
                            'enabled': False,
                            'image':   ImageResource( 'collapsed16.png' ) } )
                                    
    # Show all components in their normal state action:                            
    normal    = Instance( Action,
                          { 'name':    'Normal',
                            'action':  'on_normal_view',
                            'enabled': False,
                            'image':   ImageResource( 'normal16.png' ) } )
                                 
    # Show all components in their expanded state action:                            
    expanded  = Instance( Action,
                          { 'name':    'Max',
                            'action':  'on_expand_view',
                            'enabled': False,
                            'image':   ImageResource( 'expanded16.png' ) } )
                                   
    # Left align all selected components action:                            
    left      = Instance( Action,
                          { 'name':    'Left',
                            'action':  'on_align_left',
                            'enabled': False,
                            'image':   ImageResource( 'align_left16.png' ) } )
                               
    # Horizontally center all selected components action:                        
    hcenter   = Instance( Action,
                          { 'name':    'Center',
                            'action':  'on_align_hcenter',
                            'enabled': False,
                            'image':   ImageResource( 'align_hcenter16.png' ) })
                                 
    # Right align all selected components action:                            
    right     = Instance( Action,
                          { 'name':    'Right',
                            'action':  'on_align_right',
                            'enabled': False,
                            'image':   ImageResource( 'align_right16.png' ) } )
                                
    # Top align all selected components action:                            
    top       = Instance( Action, 
                          { 'name':    'Top',
                            'action':  'on_align_top',
                            'enabled': False,
                            'image':   ImageResource( 'align_top16.png' ) } )
                              
    # Vertically center all selected components action:                            
    vcenter   = Instance( Action,
                          { 'name':    'Center',
                            'action':  'on_align_vcenter',
                            'enabled': False,
                            'image':   ImageResource( 'align_vcenter16.png' ) } )
                                   
    # Bottom align all selected components action:                            
    bottom    = Instance( Action,
                          { 'name':    'Bottom',
                            'action':  'on_align_bottom',
                            'enabled': False,
                            'image':   ImageResource( 'align_bottom16.png' ) } )
                                 
    # Horizontally distribute all selected components action:                            
    hdist     = Instance( Action,
                          { 'name':    'Horz.',
                            'action':  'on_distribute_horizontally',
                            'enabled': False,
                            'image':   ImageResource( 'hdist16.png' ) } )
                                
    # Vertically distribute all selected components action:                            
    vdist     = Instance( Action,
                          { 'name':    'Vert.',
                            'action':  'on_distribute_vertically',
                            'enabled': False,
                            'image':   ImageResource( 'vdist16.png' ) } )
                            
    # The canvas this is the toolbar for:                            
    canvas    = Instance( GraphCanvas )
    
    # The toolbar control:
    control   = Any
    
    #---------------------------------------------------------------------------
    #  Initializes the toolbar for a specified window:  
    #---------------------------------------------------------------------------
        
    def __init__ ( self, parent = None, **traits ):
        super( GraphCanvasToolbar, self ).__init__( **traits )
        canvas  = self.canvas
        toolbar = ToolBar( self.undo,
                           self.redo,
                           Separator(),
                           self.run,
                           self.edit,
                           Separator(),
                           self.delete,
                           Separator(),
                           self.collapse,
                           self.expand,
                           Separator(),
                           self.collapsed,
                           self.normal,
                           self.expanded,
                           Separator(),
                           self.left,
                           self.hcenter,
                           self.right,
                           self.top,
                           self.vcenter,
                           self.bottom,
                           self.hdist,
                           self.vdist,
                           image_size = ( 16, 16 ) )    
        canvas.toolbar_controller = OMMenuController( 
                                         controller = graph_canvas_controller,
                                         data       = ( canvas, ) )
        self.control = toolbar.create_tool_bar( parent,
                                                canvas.toolbar_controller ) 
        
    #---------------------------------------------------------------------------
    #  Handles the canvas being changed:  
    #---------------------------------------------------------------------------
    
    def _canvas_changed ( self, old, new ):
        """ Handles the canvas being changed.
        """
        if old is not None:     
             old.on_trait_change( self._on_selection_modified, 
                                  'selection_modified', remove = True )
             old.on_trait_change( self._on_canvas_updated, 'updated', 
                                  remove = True )
             
             history = old.history
             history.on_trait_change( self._on_undoable, 'undoable',
                                      remove = True )
             history.on_trait_change( self._on_redoable, 'redoable' ,
                                      remove = True )
        
        if new is not None:
            new.on_trait_change( self._on_selection_modified, 
                                    'selection_modified' )
            new.on_trait_change( self._on_canvas_updated, 'updated' )
            
            history = new.history
            history.on_trait_change( self._on_undoable, 'undoable' )
            history.on_trait_change( self._on_redoable, 'redoable' )
            if old is not None:
                new.toolbar_controller      = old.toolbar_controller
                new.toolbar_controller.data = ( new, )
                                
    #---------------------------------------------------------------------------
    #  Handles the canvas selection being changed:  
    #---------------------------------------------------------------------------
                                                  
    def _on_selection_modified ( self ):
        """ Handles the canvas selection being changed.
        """
        selection  = self.canvas.selection
        n          = len( selection )
        nge1       = (n >= 1)
        nge2       = (n >= 2)
        neq1       = (n == 1)
        
        composites = False
        for component in selection:
            if isinstance( component.data, ConvergeCompositeActor ):
                composites = True
                break
                
        self.delete.enabled = nge1
        self.left.enabled   = self.hcenter.enabled = self.right.enabled    = \
                              self.top.enabled     = self.vcenter.enabled  = \
                              self.bottom.enabled  = self.hdist.enabled    = \
                              self.vdist.enabled   = self.collapse.enabled = \
                              nge2
        self.edit.enabled   = neq1
        self.expand.enabled = composites
        
    #---------------------------------------------------------------------------
    #  Handles the canvas being updated:  
    #---------------------------------------------------------------------------
    
    def _on_canvas_updated ( self ):
        """ Handles the canvas being updated.
        """
        self.collapsed.enabled = self.normal.enabled = self.expanded.enabled = \
            (len( self.canvas.components ) > 0)
        
    #---------------------------------------------------------------------------
    #  Handles the canvas undo history having undoable actions:  
    #---------------------------------------------------------------------------
    
    def _on_undoable ( self, enabled ):
        """ Handles the canvas undo history having undoable actions.
        """
        self.undo.enabled = enabled
        
    #---------------------------------------------------------------------------
    #  Handles the canvas undo history having redoable actions:  
    #---------------------------------------------------------------------------
    
    def _on_redoable ( self, enabled ):
        """ Handles the canvas undo history having uredoable actions.
        """
        self.redo.enabled = enabled
                
