#-------------------------------------------------------------------------------
#  
#  Defines the GraphCommentController subclass of the GraphComponentController
#  class for managing 'comment chips'.
#  
#  Written by: David C. Morrill
#  
#  Date: 06/28/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------
    
from enthought.traits.ui.api \
    import View, Item, TextEditor

from graph_component_controller \
    import GraphComponentController
    
#-------------------------------------------------------------------------------
#  View of the comment actor:   
#-------------------------------------------------------------------------------
        
comment_view = View( [ Item( 'comment',
                             style     = 'custom',
                             resizable = True,
                             editor    = TextEditor( multi_line = True ) ),
                       '|<>' ],
                     title  = 'Comment',
                     width  = 0.2,
                     height = 0.125,
                     undo   = False,
                     apply  = False,
                     revert = False,
                     help   = False,
                     kind   = 'modal',
                     id     = 'enthought.graph.vpl.comment_editor' )
    
#-------------------------------------------------------------------------------
#  'GraphCommentController' class:  
#-------------------------------------------------------------------------------

class GraphCommentController ( GraphComponentController ):
        
    #---------------------------------------------------------------------------
    #  Edits the traits of the specified component:  
    #---------------------------------------------------------------------------
    
    def edit_component ( self, component ):
        """ Edits the traits of the specified component.
        """
        ui = component.data.edit_traits( view = comment_view )
        if not ui.result:
            return ( None, None )
            
        self.update_component( component )
        return ( ui.history, [] )
        
    #---------------------------------------------------------------------------
    #  Returns the current expansion state of a specified component:
    #---------------------------------------------------------------------------
                
    def get_expansion_state ( self, component ):
        """ Returns the current expansion state of a specified component.
        """
        expansion = component._expansion
        if expansion is None:
            return 1
        return expansion
                
    #---------------------------------------------------------------------------
    #  Sets the current expansion state of the component:  
    #---------------------------------------------------------------------------
                        
    def set_expansion_state ( self, component, expansion, update = True ):
        """ Sets the current expansion state of the component.
        """
        component._expansion = expansion

    #---------------------------------------------------------------------------
    #  Builds a component using the contents of its associated actor:  
    #---------------------------------------------------------------------------
                    
    def build_component ( self, component ):
        """ Builds a component using the contents of its associated actor.
        """
        # Label the chip using the actor comment value:
        component.inner_component.text = component.data.comment
        
        # Layout the contents of the chip:
        component.contact_manager.layout_contacts()
        
# Standard comment controller:
graph_comment_controller = GraphCommentController()
        
