#-------------------------------------------------------------------------------
#  
#  Defines the OMComponent subclass used to define a "chip" associated with a
#  graph Actor object.
#  
#  Written by: David C. Morrill
#  
#  Date: 11/09/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------

from enthought.enable.om \
    import OMComponent
    
#-------------------------------------------------------------------------------
#  'GraphComponent' class:  
#-------------------------------------------------------------------------------
        
class GraphComponent ( OMComponent ):
    
    #---------------------------------------------------------------------------
    #  Event handlers:  
    #---------------------------------------------------------------------------
        
    def _data_changed ( self, old, new ):
        if old is not None:
            old.on_trait_change( self._update_component, 'changed',
                                 remove = True )
        if new is not None:
            new.on_trait_change( self._update_component, 'changed' )

    #---------------------------------------------------------------------------
    #  Updates the component after a structure change to the underlying actor:  
    #---------------------------------------------------------------------------
    
    def _update_component ( self ):
        """ Updates the component after a structure change to the underlying actor.
        """
        self.controller.update_component( self )
        
    #---------------------------------------------------------------------------
    #  Restores the states of a previously pickled canvas:  
    #---------------------------------------------------------------------------
                
    def __setstate__ ( self, state ):
        """ Restores the states of a previously pickled canvas.
        """
        super( OMComponent, self ).__setstate__( state )
        
        if self.data is not None:
            self.data.on_trait_change( self._update_component, 'changed' )
            
