#-------------------------------------------------------------------------------
#  
#  An enthought.enable.om package based visual editor for the 
#  enthought.graph package.  
#  
#  Written by: David C. Morrill
#  
#  Date: 03/01/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------

import wx

import graph_canvas_controller as gcc

from enthought.traits.api \
    import Instance, Property

from enthought.traits.ui.api \
    import UI
                                                    
from enthought.envisage.ui \
    import Editor

from enthought.traits.ui.api \
     import UndoHistory
                             
from enthought.graph \
    import CompositeActor, SimpleSdfDirector
    
from graph_canvas \
    import GraphCanvas

from graph_canvas_controller \
    import graph_canvas_controller, graph_canvas_style

#-------------------------------------------------------------------------------
#  'GraphEditor' class:  
#-------------------------------------------------------------------------------

class GraphEditor ( Editor ):
    """ A graph editor. """
    
    #---------------------------------------------------------------------------
    #  Trait definitions:  
    #---------------------------------------------------------------------------

    # The dataflow canvas we are editing:
    canvas = Instance( GraphCanvas )
    #is_mdi = False
    # The Traits UI for the entire editor (all pages):
    ui = Instance( UI )
    
    resource = Property
    
    #---------------------------------------------------------------------------
    #  Trait properties:  
    #---------------------------------------------------------------------------

    def _get_resource( self ):
        """ Get the resource being edited.
        """
        return self.canvas

    #---------------------------------------------------------------------------
    #  Protected 'Window' interface:
    #  Creates the window contents.
    #---------------------------------------------------------------------------

    def _create_contents ( self, parent ):
        """ Creates the window contents. 
        """

        # Create the underlying Graph model objects:
        actor = CompositeActor( 'Composite Actor' )
        SimpleSdfDirector( container = actor )        
        
        canvas = self.canvas
        if canvas is None:
            # Create an Enable OM canvas:
            self.canvas = canvas = GraphCanvas( 
                                     bounds      = ( 0, 0, 1536, 1536 ),
                                     min_width   = 1536, 
                                     min_height  = 1536,
                                     grid_width  = 64,
                                     grid_height = 64,
                                     controller  = graph_canvas_controller,
                                     style       = graph_canvas_style,
                                     data        = actor )
       
        # Make sure these attributes are set correctly in all cases:
        canvas.set( history      = UndoHistory(),
                    key_bindings = gcc.graph_canvas_key_bindings )
                                
        # Create the workflow graph/input/output table view:
        self.ui = ui = canvas.edit_traits( parent = parent,
                                           kind   = 'panel' )
                                 
        # Make sure the canvas and input/output tables share same undo history:
        ui.history = canvas.history
        
        # Indicate that the canvas has been updated:
        canvas.updated = True
                
        # Return the UI's control as the 'editor' widget:
        return ui.control

#-- Event handlers -------------------------------------------------------------

    #---------------------------------------------------------------------------
    #  Handles the graph editor closing:  
    #---------------------------------------------------------------------------

    def _closing_changed ( self ):
        """ Handles the graph editor closing.
        """
        self.ui.dispose()
        
    #---------------------------------------------------------------------------
    #  Handles the graph editor being closed:  
    #---------------------------------------------------------------------------

    def _closed_changed ( self ):
        """ Handles the graph editor being closed.
        """
        from enthought.graph.vpl.graph_editor_plugin import GraphEditorPlugin
        
        GraphEditorPlugin.instance.close_graph_editor( self )
