#-------------------------------------------------------------------------------
#  
#  Defines the RepeatContactController subclass of the Enable 'om' package 
#  OMContactController class used for managing the creation of new 'repeat'
#  sections on a chip with repeatable inputs or outputs.
#
#  Written by: David C. Morrill
#  
#  Date: 05/03/2005
#  
#  (c) Copyright 2005 by Enthought, Inc.
#  
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:  
#-------------------------------------------------------------------------------

from enthought.enable.om \
    import OMContactController, OMContactStyle
    
from converge_actor \
    import ConvergeRepeatBundle
    
from graph_helper \
    import new_port_name

#-------------------------------------------------------------------------------
#  'RepeatContactController' class:
#-------------------------------------------------------------------------------

class RepeatContactController ( OMContactController ):
    
    #---------------------------------------------------------------------------
    #  Returns whether or not a Contact can accept a specified Link object:  
    #---------------------------------------------------------------------------

    def can_accept_link ( self, contact, link ):
        """ Returns whether or not a Contact can accept a specified Link object.
        """
        return False

    #---------------------------------------------------------------------------
    #  Returns whether or not a Contact can accept a specified Contact object:  
    #---------------------------------------------------------------------------
    
    def can_accept_contact ( self, contact, other_contact ):
        """ Returns whether or not a Contact can accept a specified Contact 
            object.
        """
        return False
        
    #---------------------------------------------------------------------------
    #  Returns whether or not a Contact is allowed to begin a drag Link 
    #  operation:  
    #---------------------------------------------------------------------------
                
    def can_drag_link ( self, contact ):
        """ Returns whether or not a Contact is allowed to begin a drag Link 
            operation.
        """
        return False
        
    #---------------------------------------------------------------------------
    #  Begins a drag link operation by returning a Link for the specified
    #  Contact:
    #---------------------------------------------------------------------------
                
    #def begin_drag_link ( self, contact ):
    #    """ Begins a drag link operation by returning a Link for the specified 
    #        Contact.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Ends a drag link operation by completing the link between the two 
    #  specified contacts:
    #---------------------------------------------------------------------------
    
    #def end_drag_link ( self, contact1, contact2 ):
    #    """ Ends a drag link operation by completing the link between the two
    #        specified contacts.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Handles the completion of an OMDragContact dragging operation:  
    #---------------------------------------------------------------------------
                
    #def drag_contact_done ( self, drag_contact, old, event ):
    #    pass
        
    #---------------------------------------------------------------------------
    #  Begins a drag operation when the 'control' key is pressed:  
    #---------------------------------------------------------------------------
                
    #def begin_control_drag ( self, contact, event ):
    #    """ Begins a drag operation when the 'control' key is pressed.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Begins a drag operation when the 'shift' key is pressed:  
    #---------------------------------------------------------------------------
                
    #def begin_shift_drag ( self, contact, event ):
    #    """ Begins a drag operation when the 'shift' key is pressed.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Begins a drag operation when the 'alt' key is pressed:  
    #---------------------------------------------------------------------------
                
    #def begin_alt_drag ( self, contact, event ):
    #    """ Begins a drag operation when the 'alt' key is pressed.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Returns the name of the image to use for a specified Contact state:  
    #---------------------------------------------------------------------------
                
    def get_contact_image ( self, contact, state ):
        """ Returns the name of the image to use for a specified Contact state.
        """
        return '=contact_add_0_6'
        
    #---------------------------------------------------------------------------
    #  Edits the properties of a specified contact:  
    #---------------------------------------------------------------------------
                
    #def edit_contact ( self, contact ):
    #    """ Edits the properties of a specified contact.
    #    """
    #    pass
        
    #---------------------------------------------------------------------------
    #  Handles a contact being clicked:  
    #---------------------------------------------------------------------------
                
    def clicked_contact ( self, contact, event ):
        """ Handles a contact being clicked.
        """
        actor, bundle, is_input = contact.data
        actor.input_ports.extend( self._create_repeat_bundle( actor,
                                      actor.instance_input_bundles, -2, True ) )
        bundles = actor.instance_output_bundles
        if ((len( bundles ) > 0) and 
            isinstance( bundles[-1], ConvergeRepeatBundle )):
            actor.output_ports.extend( self._create_repeat_bundle( actor, 
                                           bundles, -1, False ) )
        component = contact.container
        component.controller.update_component( component )
        
    #---------------------------------------------------------------------------
    #  Display the contact's context menu:  
    #---------------------------------------------------------------------------
    
    #def popup_menu ( self, contact, event ):
    #    pass

    #---------------------------------------------------------------------------
    #  Returns whether two contacts are connectable or not:  
    #---------------------------------------------------------------------------
        
    def connectable ( self, contact1, contact2 ):
        """  Returns whether two contacts are connectable or not:  """
        return False
    
#-- Private methods ------------------------------------------------------------

    #---------------------------------------------------------------------------
    #  Creates a copy of a specified 'repeat' bundle:  
    #---------------------------------------------------------------------------

    def _create_repeat_bundle ( self, actor, bundles, index, is_input ):
        """ Creates a copy of a specified 'repeat' bundle.
        """
        bundle        = bundles[ index ]
        copied_bundle = ConvergeRepeatBundle( 
                            name      = self._bundle_name_for( bundle.name ),
                            deletable = is_input )

        n             = len( bundle.ports )

        copied_bundle.ports = []
        for port in bundle.ports:
            new_port = port.clone_traits( traits='all' )
            new_port.index += n
            copied_bundle.ports.append( new_port )

        if index == -1:
            bundles.append( copied_bundle )
        else:
            bundles.insert( index + 1, copied_bundle )
        
        # Create the corresponding actor input or output ports and return them:
        ports = copied_bundle.ports[:]
        ports.sort( lambda l, r: cmp( l.index, r.index ) )
        return [ actor.new_port( new_port_name(), is_input, port )
                 for port in ports ]
        
    #---------------------------------------------------------------------------
    #  Returns a new bundle name based on the most recent old bundle name:  
    #---------------------------------------------------------------------------
                
    def _bundle_name_for ( self, name ):
        """ Returns a new bundle name based on the most recent old bundle name.
        """
        col = name.rfind( ' ' ) + 1
        if col > 0:
            try:
                return '%s%d' % ( name[ :col ], int( name[ col: ] ) + 1 )
            except:
                pass
        return name + ' 2'
        
# Standard repeat contact controller:
repeat_contact_controller = RepeatContactController()

