""" A monitor that detects changes to a composite actor. """


# Enthought library imports.
from enthought.envisage.resource import ResourceNodeMonitor


class CompositeActorMonitor(ResourceNodeMonitor):
    """ A monitor that detects changes to a composite actor. """

    ###########################################################################
    # 'NodeMonitor' interface.
    ###########################################################################
    
    def start(self):
        """ Start listening to changes to the object. """

        actor = self.node.obj

        actor.on_trait_change(self.name_changed, 'name')
        actor.on_trait_change(self._on_nodes_changed, 'nodes')
        actor.on_trait_change(self._on_nodes_changed, 'nodes_items')

        return

    def stop(self):
        """ Stop listening to changes to the object. """

        actor = self.node.obj

        actor.on_trait_change(self.name_changed, 'name', remove=True)
        actor.on_trait_change(self._on_nodes_changed, 'nodes', remove=True)
        actor.on_trait_change(self._on_nodes_changed,'nodes_items',remove=True)
        
        return
    
    ###########################################################################
    # Private interface.
    ###########################################################################

    #### Trait event handlers #################################################
    
    def _on_nodes_changed(self):
        """ Called when the nodes in the actor have changed. """

        self.fire_structure_changed()

        return
    
##### EOF #####################################################################
