""" The resource type for composite actors. """


# Enthought library imports.
from enthought.graph import CompositeActor
from enthought.envisage.resource import ResourceContextAdapterFactory
from enthought.envisage.resource import ResourceType
from enthought.traits.api import Instance

# Local imports.
from composite_actor_context_adapter import CompositeActorContextAdapter
from composite_actor_node_type import CompositeActorNodeType


class CompositeActorResourceType(ResourceType):
    """ The resource type for composite actors. """

    #### 'ResourceType' interface #############################################

    # A trait that describes the kind of object that the type represents.
    type = Instance(CompositeActor)

    ###########################################################################
    # 'ResourceType' interface.
    ###########################################################################

    #### Initializers #########################################################

    def _node_type_default(self):
        """ Initializes the node type. """

        return CompositeActorNodeType(resource_type=self)

    def _context_adapter_factory_default(self):
        """ Initializes the context adapter factory. """

        factory = ResourceContextAdapterFactory(
            adaptee_class = CompositeActor,
            adapter_class = CompositeActorContextAdapter,
            resource_type = self
        )

        return factory

    #### Methods ##############################################################

    def get_name(self, obj):
        """ Returns the name of a resource of this type. """

        return obj.name
    
    def set_name(self, obj, name):
        """ Sets the name of a resource of this type. """

        obj.name = name

        return
    
##### EOF #####################################################################
