""" A monitor that detects changes to a converge actor. """


# Enthought library imports.
from enthought.envisage.resource import ResourceNodeMonitor


class ConvergeActorMonitor(ResourceNodeMonitor):
    """ A monitor that detects changes to a converge actor. """

    ###########################################################################
    # 'NodeMonitor' interface.
    ###########################################################################
    
    def start(self):
        """ Start listening to changes to the object. """

        actor = self.node.obj

        actor.on_trait_change(self.name_changed, 'name')

        actor.on_trait_change(
            self._on_unbound_input_ports_changed, 'unbound_input_ports'
        )

        actor.on_trait_change(
            self._on_unbound_input_ports_changed, 'unbound_input_ports_items'
        )

        actor.on_trait_change(
            self._on_output_ports_changed, 'output_ports'
        )

        actor.on_trait_change(
            self._on_output_ports_changed, 'output_ports_items'
        )

        for port in actor.ports:
            port.on_trait_change(self._on_port_token_changed, 'token')
            
        return

    def stop(self):
        """ Stop listening to changes to the object. """

        actor = self.node.obj

        actor.on_trait_change(
            self.name_changed, 'name', remove=True
        )

        actor.on_trait_change(
            self._on_unbound_input_ports_changed, 'unbound_input_ports',
            remove=True
        )

        actor.on_trait_change(
            self._on_unbound_input_ports_changed, 'unbound_input_ports_items',
            remove=True
        )

        actor.on_trait_change(
            self._on_output_ports_changed, 'output_ports',
            remove=True,
        )

        actor.on_trait_change(
            self._on_output_ports_changed, 'output_ports_items',
            remove=True
        )

        for port in actor.ports:
            port.on_trait_change(
                self._on_port_token_changed, 'token', remove=True
            )
        
        return
    
    ###########################################################################
    # Private interface.
    ###########################################################################

    #### Trait event handlers #################################################
    
    def _on_unbound_input_ports_changed(self):
        """ Called when a log in the suite has been added/removed/renamed. """

        self.fire_structure_changed()

        return

    def _on_output_ports_changed(self):
        """ Called when a log in the suite has been added/removed/renamed. """

        self.fire_structure_changed()

        return

    def _on_port_token_changed(self):
        """ Called when a token on a port has changed. """

        self.fire_structure_changed()

        return
    
##### EOF #####################################################################
