""" The resource type for converge actors. """


# Enthought library imports.
from enthought.graph.vpl.converge_actor import ConvergeActor
from enthought.envisage.resource import ResourceContextAdapterFactory
from enthought.envisage.resource import ResourceType
from enthought.traits.api import Instance

# Local imports.
from converge_actor_context_adapter import ConvergeActorContextAdapter
from converge_actor_node_type import ConvergeActorNodeType


class ConvergeActorResourceType(ResourceType):
    """ The resource type for converge actors. """

    #### 'ResourceType' interface #############################################

    # A trait that describes the kind of object that the type represents.
    type = Instance(ConvergeActor)

    ###########################################################################
    # 'ResourceType' interface.
    ###########################################################################

    #### Initializers #########################################################

    def _node_type_default(self):
        """ Initializes the node type. """

        return ConvergeActorNodeType(resource_type=self)

    def _context_adapter_factory_default(self):
        """ Initializes the context adapter factory. """

        factory = ResourceContextAdapterFactory(
            adaptee_class = ConvergeActor,
            adapter_class = ConvergeActorContextAdapter,
            resource_type = self
        )

        return factory

    #### Methods ##############################################################

    def get_name(self, obj):
        """ Returns the name of a resource of this type. """

        return obj.name
    
    def set_name(self, obj, name):
        """ Sets the name of a resource of this type. """

        obj.name = name

        return
    
##### EOF #####################################################################
