""" The resource type for dataflow graphs. """


# Enthought library imports.
from enthought.enable.om.om_canvas import OMCanvas
from enthought.envisage.resource import ResourceContextAdapterFactory
from enthought.envisage.resource import ResourceType
from enthought.traits.api import Instance

# Local imports.
from dataflow_context_adapter import DataflowContextAdapter
from dataflow_node_type import DataflowNodeType


class DataflowResourceType(ResourceType):
    """ The resource type for dataflow graphs. """

    #### 'ResourceType' interface #############################################

    # A trait that describes the kind of object that the type represents.
    type = Instance(OMCanvas)

    ###########################################################################
    # 'ResourceType' interface.
    ###########################################################################

    #### Initializers #########################################################

    def _node_type_default(self):
        """ Initializes the node type. """

        return DataflowNodeType(resource_type=self)

    def _context_adapter_factory_default(self):
        """ Initializes the context adapter factory. """

        factory = ResourceContextAdapterFactory(
            adaptee_class = OMCanvas,
            adapter_class = DataflowContextAdapter,
            resource_type = self
        )

        return factory

    #### Methods ##############################################################
    
    def get_name(self, obj):
        """ Returns the name of an object. """

        return obj.data.name

    def set_name(self, obj, name):
        """ Sets the name of an object. """

        obj.data.name = name

        return
    
##### EOF #####################################################################
