""" A wizard that creates a new (and empty) VPL graph. """


# Standard library imports.
import cPickle
from os.path import dirname, join

# Enthought library imports.
from enthought.graph.vpl import graph_factory
from enthought.traits.api import Instance

# Local imports.
from enthought.envisage.project.wizard import NewNamedResourcePage
from enthought.envisage.project.wizard import NewResourceWizard
from enthought.envisage.project.wizard import ResourceFactory


class GraphFactory(ResourceFactory):
    """ A factory that creates VPL graph resources. """
    
    ###########################################################################
    # 'ResourceFactory' interface.
    ###########################################################################

    def create_resource(self):
        """ Creates the resource. """

        factory = graph_factory.GraphFactory()
        
        self.parent_folder.obj.bind(self.name, factory.create(self.name))

        return self.parent_folder.obj.lookup_binding(self.name)


class NewGraphWizard(NewResourceWizard):
    """ A wizard that creates a new (and empty) VPL graph. """

    #### 'Window' interface ###################################################

    # The window title.
    title = 'New Dataflow Model'

    #### 'NewGraphWizard' interface ##########################################

    # The resource factory that the wizard is configuring.
    factory = Instance(GraphFactory, ())
    
    ###########################################################################
    # 'NewResourceWizard' interface.
    ###########################################################################

    def reset(self):
        """ Reset the wizard to the initial state. """
        
        # The pages in the wizard.
        self.pages = [
            NewNamedResourcePage(
                text   = 'Select the parent folder',
                prefix = 'New Model',
                obj    = self.factory
            )
        ]

        return
    
#### EOF ######################################################################
