""" A test plugin definition. """


# Plugin extension-point imports.
from enthought.envisage.core.core_plugin_definition import PluginDefinition

# Enthought plugin definition imports.
from enthought.envisage.workbench.workbench_plugin_definition import \
     Action, Menu, Perspective, View, Workbench


# The plugin's globally unique identifier (also used as the prefix for all
# identifiers defined in this module).
ID = "enthought.plugins.test"


###############################################################################
# Extensions.
###############################################################################

workbench = Workbench(
    perspectives = [
        Perspective(
            id       = ID + ".perspective.foo_perspective.FooPerspective",
            name     = "Foo",
            contents = [
                Perspective.Item(
                    id = "enthought.plugins.test.view.blue_view.BlueView",
                    position = "left"
                ),

                Perspective.Item(
                    id = "enthought.plugins.test.view.red_view.RedView",
                    position = "left"
                ),

                Perspective.Item(
                    id = "enthought.plugins.test.view.yellow_view.YellowView",
                    position = "bottom"
                ),

                Perspective.Item(
                    id = "enthought.plugins.python_shell.view.PythonShellView",
                    position = "bottom"
                ),
            ]
            
        ),
        
        Perspective(
            id       = ID + ".perspective.bar_perspective.BarPerspective",
            name     = "Bar",
            contents = [
                Perspective.Item(
                    id = "enthought.plugins.test.view.green_view.GreenView",
                    position = "left"
                ),
                
                Perspective.Item(
                    id = "enthought.plugins.test.view.black_view.BlackView",
                    position = "left"
                ),

                Perspective.Item(
                    id = "enthought.plugins.test.view.yellow_view.YellowView",
                    position = "bottom"
                ),

                Perspective.Item(
                    id = "enthought.plugins.python_shell.view.PythonShellView",
                    position = "top"
                ),
            ]
        )
    ],

    default_perspective = ID + ".perspective.bar_perspective.BarPerspective",
    
    views = [
        View(
            class_name = ID + ".view.black_view.BlackView",
            name       = "Black",
            position   = "bottom"
        ),
        
        View(
            class_name = ID + ".view.blue_view.BlueView",
            name       = "Blue",
            position   = "left"
        ),
        
        View(
            class_name = ID + ".view.green_view.GreenView",
            name       = "Green",
            position   = "left"
        ),

        View(
            class_name = ID + ".view.red_view.RedView",
            name       = "Red",
            position   = "left"
        ),
        
        View(
            class_name = ID + ".view.yellow_view.YellowView",
            name       = "Yellow",
            position   = "bottom"
        )
    ],

    menus = [
        Menu(id="Tools", name="&Tools", before="Help"),
    ],

    actions = [
        Action(
            function_name = "enthought.util.refresh.refresh",
            accelerator   = "Ctrl+Shift+R",
            name          = "Refresh",
            description   = "Refresh all Python modules", 
            menu_bar_path = "Tools:end",
        )
    ]
)

###############################################################################
# The plugin definition!
###############################################################################

PluginDefinition(
    # The plugin's globally unique identifier.
    id = ID,

    # General information about the plugin.
    name          = "Test Plugin",
    version       = "1.0.0",
    provider_name = "Enthought Inc",
    provider_url  = "www.enthought.com",

    # The Id's of the plugins that this plugin requires.
    requires = [
        "enthought.envisage.workbench",
    ],

    # The extension points offered by this plugin,
    extension_points = [],
    
    # The contributions that this plugin makes to extension points offered by
    # either itself or other plugins.
    extensions = [workbench]
)

#### EOF ######################################################################
