#-----------------------------------------------------------------------------
#
#  Copyright (c) 2005-2006 by Enthought, Inc.
#  All rights reserved.
#
#  Author: Greg Rogers
#
#-----------------------------------------------------------------------------

""" A string trait that must be a valid python expression. """
import parser
import types

from enthought.traits.api        import Trait, TraitFactory, TraitHandler
from enthought.traits.ui.api     import TextEditor
from enthought.traits.trait_base import strx

# Validation via Handler
class ExpressionTraitHandler(TraitHandler):
    """ Validation handler for ExpressionTrait objects. """
    def validate ( self, object, name, value ):
        try:
            if value is not None:
                value = strx( value )

                if value.strip() != '':
                    ast = parser.expr(value)
                    
        except:
            self.error( object, name, self.repr( value ) )

        return value
    
    def info(self):
        msg = "a valid python expression or None or an empty string"
        return msg


def expression_trait_factory_function(value='', **metadata):
    return Trait( value, ExpressionTraitHandler(), 
                  editor=TextEditor() )

# The factory for ExpressionTrait objects.
# An ExpressionTrait is a string trait that must be a valid python expression.
ExpressionTrait = TraitFactory( expression_trait_factory_function )

### EOF