#-----------------------------------------------------------------------------
#
#  Copyright (c) 2006 by Enthought, Inc.
#  All rights reserved.
#
#  Author: Greg Rogers
#
#-----------------------------------------------------------------------------

""" An Expression with units specification that when evaluated returns
a Quantity object.
"""

from enthought.traits.api import Instance

from enthought.units.api import MetaQuantity, Quantity

from expression import Expression


class QuantityExpression(MetaQuantity, Expression):
    """ An Expression with units specification that when evaluated returns
    a Quantity object.
    """

    ###########################################################################
    # Expression methods
    ###########################################################################
    
    def _post_eval(self, value, globals, locals):
        """ Protected method called just after eval actually evaluates the
        expression.  This method further validates and/or transforms the
        raw evaluation value.  That value is returned as the eval result.
        If value does not pass the validation requirements or cannot be
        transformed to do so, an exception is raised.
        
        This implementation validates that value is a Quantity with the correct
        family_name and units, or transforms it if possible.
        """
        if not isinstance(value, Quantity):
            value = Quantity( value, units=self.units, 
                              family_name=self.family_name,
                              name=self.name )

        #ToDo: ensure that value is compatible with meta_quantity
        return value
    

    ###########################################################################
    # HasTraits interface
    ###########################################################################
    
    def trait_view(self, name=None, view_element=None):
        """ Returns a View """
        if (name or view_element) != None:
            return super(MetaQuantity, self).trait_view( name=name, 
                                                view_element=view_element )
        
        from enthought.python.ui.quantity_expression_view \
            import QuantityExpressionView
        return QuantityExpressionView()

### EOF
