""" QuantityProxy allows specification of a Quantity via a naming system name
or by actual value.
"""

from enthought.traits.api import \
    HasTraits, Any, Dict, Instance, Property, Str, Trait

from enthought.envisage import get_application
from enthought.graph import TokenProxy

from enthought.units.unit import unit as Unit
from enthought.units.unit_parser import unit_parser
from enthought.units.quantity import Quantity

class QuantityProxy( TokenProxy ):
    """ QuantityProxy allows specification of a Quantity via a naming system
    name, or an actual value.

    The name is used to retrieve the Quantity from the workspace naming system.
    """

    #### TokenProxy interface #################################################
    # A name from the workspace naming system or a Quantity value.
    value = Property

    # Cached reference to the Quantity from the workspace
    _qty = Instance(Quantity)

    #### Protected methods ####################################################

    def _set_value( self, value ):
        """ Change the Quantity that value will return via name or Quantity. """
        old = self._name or self._qty

        #print 'QuantityProxy.set_qty', 'type(value)', type(value), 'value', value
        self._name = self._qty = None

        if isinstance(value, basestring):
            self._name = value

        elif isinstance( value, Quantity ):
            self._qty = value

        else:
            # return without firing event
            return

        self.trait_property_changed( 'value', old, self.value )

        return


    def _get_value( self ):
        """ Returns the Quantity object specified via _set_value. """
        if self._qty is None:
            if self._name is not None:
                workspace = get_application().service_registry.get_service(
                                        'enthought.envisage.project.IWorkspace')
                obj = workspace.lookup( self._name )
                if not isinstance( obj, Quantity):
                    if isinstance( obj, QuantityProxy ):
                        obj = obj.value
                self._qty = obj

        return self._qty

    #### Picklin' #############################################################

    def __getstate__(self):
        """ return dictionary of attributes to pickle. """
        attributes = {}
        attributes.update(self.__dict__)

        # Make sure cached objects are not pickled
        if self._name is not None:
            del attributes['_qty']

        return attributes

### EOF #######################################################################
