""" Value port for ports that produce or consume Scalar objects but present
float objects to their containing actor.
"""

from enthought.traits.api import Instance

from enthought.units.scalar import Scalar
from enthought.resource_type.scalar_proxy import ScalarProxy

from enthought.graph import AnyTypeValuePort

class ScalarFloatValuePort( AnyTypeValuePort ):
    """ Value port for ports that produce or consume Scalar objects but present
    float objects to their containing actor.
    
    Use of this port class is determined by ConvergeActor at instantiation time.
    
    The Scalar object bound to token is reused on each fire call. Any receiver 
    of that object should call scalar.clone() if they want a stable copy.
    """

    external_type = 'enthought.units.scalar.Scalar'
    internal_type = 'float'
    
    def __init__( self, *args, **traits ):
        super(ScalarFloatValuePort, self).__init__( *args, **traits )
        
        # create the scalar object that will be reused on each firing
        value = traits.get('value', 0.5)
        if type(value) != float:
            value = 0.5

        self.reset_token(value)
    
        return
        
    def get( self, n=1 ):
        """ Return token from local receiver. 
        
        Token is expected to be a Scalar, but containing actor wants an array.
        """
        obj = self._get_local_receiver().get(n)
        
        #print 'ScalarFloatValuePort.get', self.full_name, 'type(obj)', type(obj)
        if not isinstance( obj, Scalar ):
            raise ValueError, "Port token is not a Scalar: %s %s" % (self.name, obj)
        
        return obj.data
            
    def get_outside( self, n=1 ):
        """ Return token from local receiver as is without reducing to array.
        """
        obj = self._get_local_receiver().get(n)
        #print 'get_outside local_receiver',self._local_receiver,' returns', obj
        return obj
        
    def put( self, token ):
        """ Convert the token to a Scalar and store in the token attribute.
        """
        #print 'ScalarFloatValuePort.put', self.full_name, 'type',type(token), 'token',token
        float_value = float(token)
        self.token.data = float_value
        return
            
    def reset_token(self, value=0.0):
        """ Reset token to the default value. """
        scalar = Scalar( value, name=self.name, 
                            units=self.units, family_name=self.family_name )

        if self.is_input:
            self.token = ScalarProxy()
            self.token.value = scalar
            
        else:
            self.token = scalar

        return