""" Represents a shared library for the Resource Sharing plugin for Envisage.

    (Copyright 2006 by Enthought, Inc.)
"""


# Third-party package imports
import pysvn
import os
import logging

# Enthought library imports
from enthought.traits.api import HasTraits, Str, Any

# Local imports


logger = logging.getLogger(__name__)


############################################################################
# Library class
############################################################################
class Library(HasTraits):
    """ Represents a single shared library available to the application.
    """

    # A descriptive name and a locator string (URL in SVN) for the library
    name = Str
    location = Str

    # The library manager that controls this library
    lib_manager = Any

    ########################################################################
    # Special methods
    ########################################################################

    def __init__(self, name, location, manager):
        """ A library should always have a name and a location from birth. 
        """
        self.name = name
        if location[-1] == '/':
            self.location = location[:-1]
        else:
            self.location = location
        self.lib_manager = manager
        return



    ########################################################################
    # Library API.  If the class is re-implemented to use a different back
    # end, the names and signatures of these methods should not change.
    ########################################################################

    def get_shared_resource(self, resource, destination):
        """ Get a shared resource from this library.

            Get file or folder 'resource' from the shared library and store 
            it in folder 'destination'
        """

        libraryloc = self.location + resource

        print libraryloc

        self.lib_manager.svn_client.checkout(libraryloc, destination)

        return


    def add_new_resource(self, resource, comment=""):
        """ Add the file or folder 'resource' to the shared library.

            The 'resource' argument should contain the pathname.
        """
        self.lib_manager.svn_client.add(resource)

        if comment == "":
            comment = "Initial checkin of" + resource

        self.lib_manager.svn_client.checkin(resource, comment)
        logger.info("Added " + resource + " to shared library")

        return


    def longname(self):
        """ Return the name used for dropdown lists and the browser. """

        return self.name + "   (" + self.location + ")"


    def make_marker(self, path):
        """ Create an empty working directory with the name of the library """

        self.lib_manager.svn_client.checkout(self.location, path, recurse=False)

        return

#### EOF ######################################################################
