""" The set of Actions called from Navigator object context menus."""

# General imports
import wx
import os
import pickle

# Enthought library imports.
from enthought.envisage.ui import WorkbenchAction
from enthought.envisage.project import Project
from enthought.traits.api import Str, HasTraits, List, Dict, Instance, Trait
from enthought.traits.ui.api import View, Item, Group, EnumEditor
from enthought.traits.ui.message import message, error
from enthought.envisage import get_application

# Local imports
from enthought.sharing.plugin import start_wait, end_wait
from library_browser import LibraryBrowser
from library_selector import LibrarySelector
from enthought.sharing.plugin.sharing_plugin import SharingPlugin


class TestChildClass(HasTraits):
    string_attr = Str
    list_attr = List(['a','b','c'])

    traits_view = View(Item('string_attr'),
                       Item('list_attr', style='text'))

    
class TestClass(HasTraits):
    name = Str
    tcc1 = Trait(TestChildClass)
    tcc2 = Trait(TestChildClass)

    def __init__(self):
        self.tcc1 = TestChildClass()
        self.tcc2 = TestChildClass()

        return

    traits_view = View(Item('name'),
                       Item('tcc1', style='custom'),
                       Item('tcc2', style='custom'))



class ObjectAddAction(WorkbenchAction):
    """ """

    def perform( self ):
        """ 
        """
        selection = self.window.selection
        binding = selection[0]
        folder = binding.obj

        sample = TestClass()
        sample.edit_traits(kind='livemodal')

        folder.create_resource(sample.name, sample)

        folder.refresh()
        
        return


class CopyObjectLibraryAction(WorkbenchAction):

    def perform( self ):
        """ Copy all objects in the shared library to the selected project(s)
        """
        selection = self.window.selection

        lm = get_application().get_service('enthought.sharing.ILibraryManager')
        ls = LibrarySelector()

        libname = ls.get_library(files=False)

        if libname != None:
            lm.set_current_object_library(libname)

            start_wait()

            hidden_library_loc = os.path.join(lm.object_buffer, libname)

            try:
                lm.current_object_library.get_shared_resource("",
                                                              hidden_library_loc) 
            except:
                message("Could not open " + lm.current_object_library.name)
                return
            
            
                
            # copy the library to the appropriately named shared folder under
            # each project (creating folders if needed)
            for binding in selection:
                project = binding.obj

                local_object_folder = os.path.join(project.path, "Shared Objects")
                if not os.path.isdir(local_object_folder):
                    project.create_folder("Shared Objects")

                dest_context = project.lookup("Shared Objects")

                visible_library_loc = os.path.join(dest_context.path, libname)

                if not os.path.isdir(visible_library_loc):
                    dest_context.create_folder(libname)

                lib_context = dest_context.lookup(libname)

                for itemname in os.listdir(hidden_library_loc):
                    itemloc = os.path.join(hidden_library_loc, itemname)
                    if not os.path.isdir(itemloc):
                        continue

                    itemfilename = os.path.join(itemloc, itemname+".pcl")
                    if not os.path.isfile(itemfilename):
                        continue

                    itemfile = open(itemfilename, 'rb')
                    newobject = pickle.load(itemfile)
                    itemfilename.close()
                    lib_context.create_resource(itemname, newobject)
 
            end_wait()

        return



class AddObjectAction(WorkbenchAction):
    """ Menu action to add the selected object(s) to the appropriate library.
    """
    def perform( self ):
        """ Add the object to the library it came from.
        """
        selection = self.window.selection

        lm = get_application().get_service('enthought.sharing.ILibraryManager')

        start_wait()

        for binding in selection:
            resource = binding.obj
        
            library = lm.get_object_library(binding)
            hidden_library_loc = os.path.join(lm.object_buffer, library.name)
            if not os.path.isdir(hidden_library_loc):
                library.make_marker(hidden_library_loc)

            resource_loc = os.path.join(hidden_library_loc, binding.name)

            if not os.path.isdir(resource_loc):
                os.mkdir(resource_loc)

            resource_filename = binding.name + ".pcl"
            savename = os.path.join(resource_loc, resource_filename)
           
            # FIXME: Also needs to check library itself!
            if os.path.isfile(savename):
                end_wait()
                if not error("Object '" + binding.name + 
                             "' already exists in library.  Update?"):
                    return
                start_wait()

            savefile = file(savename, 'wb')
            pickle.dump(resource, savefile)
            savefile.close()

            library.add_new_resource(resource_loc)

        end_wait()

        return


class BrowseObjectLibraryAction(WorkbenchAction):
    """ Menu action to build a browser for an object library """
    def perform(self):

        # Prompt for the library to be displayed
        lm = get_application().get_service('enthought.sharing.ILibraryManager')
        ls = LibrarySelector()

        selection = self.window.selection
        binding = selection[0]
        project = binding.obj

        libname = ls.get_library(files=False)

        if libname != None:
            lm.set_current_object_library(libname)

            # Construct and display a browser for the selected library
            start_wait()

            try:
                lb = LibraryBrowser(lm.current_object_library, project, objects=True)
            except:
                message("Unable to open " + libname)
                lb = None

            end_wait()

            if lb != None:
                lb.edit_traits()

        return
#### EOF ######################################################################
