""" The set of Actions called from Navigator object context menus."""

# General imports
import wx
import os

# Enthought library imports.
from enthought.envisage.project import Project
from enthought.traits.api import Str, HasTraits, List, Dict
from enthought.traits.ui.api import View, Item, Group, EnumEditor
from enthought.traits.ui.message import message
from enthought.envisage import get_application



class LibrarySelector(HasTraits):
    """ Prompts the user to select one of the available libraries. """

    # Attributes for the user-selected library and the list it is
    # selected from.
    _libname = Str
    _liblist = List
    _libdict = Dict

    # The view to be used to prompt for a library name
    library_list_view = View(Item(name='_libname',
                                  editor = EnumEditor(name='_liblist'),
                                  label='Select Library'),
                             width = 500,
                             title = 'Library Selector',
                             buttons = ['OK', 'Cancel'])

    def get_library(self, files=True):
        lm = get_application().get_service('enthought.sharing.ILibraryManager')
        if files:
            libdict = lm.available_libraries
        else:
            libdict = lm.available_object_libraries

        if len(libdict) == 0:
            message("There are no libraries currently registered.")
            return None
        else:
            for key in libdict.keys():
                nextlib = libdict[key]
                longname = nextlib.longname()
                self._liblist.append(longname)
                self._libdict[longname] = key

            if files:
                self._libname = lm.current_library.longname()
            else:
                self._libname = lm.current_object_library.longname()

            if self.edit_traits(view='library_list_view', 
                                 kind='livemodal').result \
               and self._libname != "":
                return self._libdict[self._libname]
            else:
                return None

        return

#### EOF ######################################################################
