""" The set of Actions called from the 'Sharing' submenu of 'Tools'.
"""

# Enthought library imports.
from enthought.envisage.ui import WorkbenchAction
from enthought.envisage.project import Project
from enthought.traits.api import Str, List, Instance, Dict, Button
from enthought.traits.ui.api import View, Group, Item, EnumEditor
from enthought.traits.ui.message import message
from enthought.envisage import get_application

from enthought.sharing import Library, LibraryManager
from library_selector import LibrarySelector


############################################################################
# Sharing Menu action classes
############################################################################

class ManageLibrariesAction(WorkbenchAction):

    libmanager = Instance(LibraryManager)
    libname = Str
    liblist = List
    libdict = Dict
    current_lib_key = Str
    add_new = Button
    remove = Button

    newlibname = Str
    newlibloc = Str

    traits_view = View(Group(Group(Item(name = 'liblist', 
                                        style='readonly', 
                                        height=75),
                                   show_labels = False,
                                   label = 'Shared File Libraries'),
                             Group(Item(name = 'add_new'),
                                   Item(name = 'remove', 
                                        enabled_when="len(liblist) > 0"),
                                   orientation = 'horizontal',
                                   show_labels = False)),
                       title = "Library Administration",
                       width = 500,
                       buttons = ['OK', 'Cancel'])


    add_library_view = View(Item(name='newlibname', label='Name'), 
                            Item(name='newlibloc', label='Location'),
                            kind = 'livemodal',
                            buttons = ['OK', 'Cancel'],
                            title = "Register new library",
                            width = 300)


    def _add_new_fired(self):
        if self.edit_traits(view='add_library_view', kind='livemodal').result:
            if self.libmanager.is_library(self.newlibloc):
                newlib = Library(self.newlibname, self.newlibloc, self.libmanager)
                self.libdict[self.newlibname] = newlib
                self.liblist.append(newlib.longname())
                self.libname = newlib.longname()
                self.current_lib_key = newlib.name
            else:
                errmsg = self.newlibloc + " is not a shared library"
                message(errmsg)

            self.newlibname = ""
            self.newlibloc = ""

        return


    def _remove_fired(self):
        ls = LibrarySelector()
        libname = ls.get_library()
        if libname != None:
            self.liblist.remove(self.libdict[libname].longname())
            del self.libdict[libname]
            if libname == self.current_lib_key:
                if len(self.libdict) > 0:
                    self.current_lib_key = self.libdict.keys()[0]
                else:
                    self.current_lib_key = None

        return


    def __init__(self, **kw):
        self.libmanager = get_application().get_service('enthought.sharing.ILibraryManager')
        self.libdict = self.libmanager.available_libraries

        for key in self.libdict.keys():
            nextlib = self.libmanager.available_libraries[key]
            longname = nextlib.longname()
            self.liblist.append(longname)


    def perform(self):
        self.libmanager = get_application().get_service('enthought.sharing.ILibraryManager')
        if self.libmanager.current_library != None:
            self.libname = self.libmanager.current_library.longname()
            self.current_lib_key = self.libmanager.current_library.name

        if self.edit_traits(kind='livemodal').result:
            self.libmanager.available_libraries = self.libdict
            self.libmanager.current_library = self.libdict[self.current_lib_key]

        return


class ManageObjectLibrariesAction(WorkbenchAction):

    libmanager = Instance(LibraryManager)
    libname = Str
    liblist = List
    libdict = Dict
    current_lib_key = Str
    add_new = Button
    remove = Button

    newlibname = Str
    newlibloc = Str

    traits_view = View(Group(Group(Item(name = 'liblist', 
                                        style='readonly', 
                                        height=75),
                                   show_labels = False,
                                   label = 'Shared Object Libraries'),
                             Group(Item(name = 'add_new'),
                                   Item(name = 'remove', 
                                        enabled_when="len(liblist) > 0"),
                                   orientation = 'horizontal',
                                   show_labels = False)),
                       title = "Library Administration",
                       width = 500,
                       buttons = ['OK', 'Cancel'])


    add_library_view = View(Item(name='newlibname', label='Name'), 
                            Item(name='newlibloc', label='Location'),
                            kind = 'livemodal',
                            buttons = ['OK', 'Cancel'],
                            title = "Register new library",
                            width = 300)


    def _add_new_fired(self):
        if self.edit_traits(view='add_library_view', kind='livemodal').result:
            if self.libmanager.is_library(self.newlibloc):
                newlib = Library(self.newlibname, self.newlibloc, self.libmanager)
                self.libdict[self.newlibname] = newlib
                self.liblist.append(newlib.longname())
                self.libname = newlib.longname()
                self.current_lib_key = newlib.name
            else:
                errmsg = self.newlibloc + " is not a shared library"
                message(errmsg)

            self.newlibname = ""
            self.newlibloc = ""

        return


    def _remove_fired(self):
        ls = LibrarySelector()
        libname = ls.get_library()
        if libname != None:
            self.liblist.remove(self.libdict[libname].longname())
            del self.libdict[libname]
            if libname == self.current_lib_key:
                if len(self.libdict) > 0:
                    self.current_lib_key = self.libdict.keys()[0]
                else:
                    self.current_lib_key = None

        return


    def __init__(self, **kw):
        self.libmanager = get_application().get_service('enthought.sharing.ILibraryManager')
        self.libdict = self.libmanager.available_object_libraries

        for key in self.libdict.keys():
            nextlib = self.libmanager.available_object_libraries[key]
            longname = nextlib.longname()
            self.liblist.append(longname)


    def perform(self):
        self.libmanager = get_application().get_service('enthought.sharing.ILibraryManager')
        if self.libmanager.current_object_library != None:
            self.libname = self.libmanager.current_object_library.longname()
            self.current_lib_key = self.libmanager.current_object_library.name

        if self.edit_traits(kind='livemodal').result:
            self.libmanager.available_object_libraries = self.libdict
            self.libmanager.current_object_library = \
                 self.libdict[self.current_lib_key]

        return

#### EOF ######################################################################
