""" Plugin code for Resource Sharing for Envisage.

    (Copyright 2006 by Enthought, Inc.)
"""

# General imports
import os
import pickle

# Enthought library imports.
from enthought.envisage import Plugin
from enthought.traits.api import Instance


# Local imports
from enthought.sharing import Library, LibraryManager


############################################################################
# SharingPlugin class
############################################################################
class SharingPlugin(Plugin):
    """ Creates and registers a LibraryManager for the application session.
    """

    library_manager = Instance(LibraryManager)
    library_manager_id = 'enthought.sharing.ILibraryManager'

    ###########################################################################
    # 'Plugin' interface.
    ###########################################################################

    def start(self, application):
        """ Starts the plugin.

        Can be called manually, but is usually called exactly once when the
        plugin is first required.
        This version creates a single Library Manager and registers it with
        Envisage.

        """

        library_manager_file = os.path.join(self.state_location,
                                            'sharing_library_manager')

        if os.path.isfile(library_manager_file):
            f = file(library_manager_file, 'rb')
            self.library_manager = pickle.load(f)
            self.library_manager.reset()
            f.close()
        else:
            self.library_manager = LibraryManager()
 
        hidden_shared_object_folder = os.path.join(self.state_location,
                                                   ".shared_objects")
        if not os.path.isdir(hidden_shared_object_folder):
            os.mkdir(hidden_shared_object_folder)

        self.library_manager.object_buffer = hidden_shared_object_folder

        self.register_service(self.library_manager_id, self.library_manager)

        return

    
    def stop(self, application):
        """ Stops the plugin.

        Can be called manually, but is usually called exactly once when the
        application exits.

        """

        self.save_preferences()

        self.library_manager.mothball()

        library_manager_file = self.state_location + '\sharing_library_manager'
        f = file(library_manager_file, 'wb')
        pickle.dump(self.library_manager, f)
        f.close()
        
        return

#### EOF ######################################################################
