""" Definition file for the Resource Sharing plugin for Envisage. 

    (Copyright 2006 by Enthought, Inc.)
"""

import enthought.envisage

# Enthought library imports.
from enthought.traits.api import Bool

# Plugin definition imports.
from enthought.envisage.core.core_plugin_definition \
    import PluginDefinition, Preferences

from enthought.envisage.ui.ui_plugin_definition \
    import Action, Group, Menu, UIActions, UIViews, View

from enthought.envisage.project.project_plugin_definition \
    import ResourceActions, ResourceAction, PROJECT_RESOURCE_TYPE,\
           FILE_RESOURCE_TYPE, FOLDER_RESOURCE_TYPE, INSTANCE_RESOURCE_TYPE

# Local imports


###############################################################################
# Extensions.
###############################################################################

#### Menus/Actions ############################################################


# Sharing menu option to manage the library registry.
manage_libraries_action = Action(
    id = "enthought.sharing.plugin.sharing_menu_actions.ManageLibraries",
    class_name = \
        "enthought.sharing.plugin.sharing_menu_actions.ManageLibrariesAction",
    name = "Manage Shared File Libraries",
    description = "Add new file library to list of available shared resources",
    tooltip = "Add new file library to list of available shared resources",
    menu_bar_path = "ToolsMenu/",
    tool_bar_path = "",
)

manage_object_libraries_action = Action(
    id = "enthought.sharing.plugin.sharing_menu_actions.ManageObjectLibraries",
    class_name = \
        "enthought.sharing.plugin.sharing_menu_actions.ManageObjectLibrariesAction",
    name = "Manage Shared Object Libraries",
    description = "Add new object library to list of available shared resources",
    tooltip = "Add new object library to list of available shared resources",
    menu_bar_path = "ToolsMenu/",
    tool_bar_path = "",
)

# Add the Manage Libraries action to the UIActions extension point.
ui_actions = UIActions(
    actions = [
               manage_libraries_action,
               manage_object_libraries_action,
              ]
)


# Add context menu actions to files in the Navigator.
resource_actions = ResourceActions(
    actions = [
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.CheckoutAction",
            name = "Copy Shared Library",
            resource_type = PROJECT_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.BrowseAction",
            name = "Browse Shared Library",
            resource_type = PROJECT_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.experimental_actions.CopyObjectLibraryAction",
            name = "Copy Object Library",
            resource_type = PROJECT_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.experimental_actions.BrowseObjectLibraryAction",
            name = "Browse Object Library",
            resource_type = PROJECT_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.experimental_actions.ObjectAddAction",
            name = "(Experimental)",
            resource_type = FOLDER_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.PostAction",
            name = "Post Changes",
            resource_type = FILE_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.UpdateAction",
            name = "Update from Library",
            resource_type = FILE_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.AddAction",
            name = "Add to Library",
            resource_type = FILE_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.context_menu_actions.ShowLogAction",
            name = "Show Change Log",
            resource_type = FILE_RESOURCE_TYPE,
            path = "additions",),
        ResourceAction(
            class_name = \
                "enthought.sharing.plugin.experimental_actions.AddObjectAction",
            name = "Add to Shared Library",
            resource_type = INSTANCE_RESOURCE_TYPE,
            path = "additions",),
    ]
)



###############################################################################
# The plugin definition!
###############################################################################

PluginDefinition(
    # The plugin's globally unique identifier.
    id = "enthought.sharing",

    # The name of the class that implements the plugin.
    class_name = "enthought.sharing.plugin.sharing_plugin.SharingPlugin",

    # General information about the plugin.
    name          = "Simple Sharing Plugin",
    version       = "1.0.0",
    provider_name = "Enthought Inc",
    provider_url  = "www.enthought.com",
    enabled       = True,
    autostart     = True,

    requires = [
        "enthought.envisage.ui",
        "enthought.envisage.ui.preference",
        "enthought.envisage.project",
    ],

    # The extension points offered by this plugin,
    extension_points = [],
    
    # The contributions that this plugin makes to extension points offered by
    # either itself or other plugins.
    extensions = [resource_actions, ui_actions]
)

#### EOF ######################################################################
