""" Example plugin definition. """


# Enthought library imports.
from enthought.traits.api import Bool

# Plugin definition imports.
from enthought.envisage.core.core_plugin_definition \
     import PluginDefinition

from enthought.envisage.ui.ui_plugin_definition \
     import Action, Group, Menu, UIActions, UIViews, View


###############################################################################
# Extensions.
###############################################################################

#### Menus/Actions ############################################################

file_menu = Menu( 
    id     = "FileMenu",
    path   = "FileGroup",
            
    groups = [
        Group(id = "AnExampleGroup"),
        Group(id = "AnotherGroup"),
    ]
)


sub_menu = Menu( 
    id     = "SubMenu",
    name   = "Sub",
    path   = "FileMenu/AnExampleGroup",
            
    groups = [
        Group(id = "MainGroup"),
        Group(id = "RadioGroup"),
    ]
)

sharing_menu = Menu(
    id = "SharingMenu",
    name = "&Sharing",
    path = "FileGroup",

    groups = [
        Group(id = "MyGroup"),
    ]
)

do_it_action = Action(
    id            = "action.example_action.DoItAction",
    class_name    = "action.example_action.ExampleAction",
    name          = "Do It!",
    image         = "action/images/do_it",
    description   = "An action's description can appear in the status bar",
    tooltip       = "A simple example action",
    menu_bar_path = "FileMenu/SubMenu/MainGroup",
    tool_bar_path = "additions",
    style         = "push",
)

higher_action = Action(
    id            = "action.example_action.HigherAction",
    class_name    = "action.example_action.ExampleAction",
    name          = "Higher",
    image         = "action/images/higher",
    description   = "An action's description can appear in the status bar",
    tooltip       = "A simple example action",
    menu_bar_path = "FileMenu/SubMenu/RadioGroup",
    tool_bar_path = "RadioGroup",
    style         = "radio",
    checked       = True
)

lower_action = Action(
    id            = "action.example_action.LowerAction",
    class_name    = "action.example_action.ExampleAction",
    name          = "Lower",
    image         = "action/images/lower",
    description   = "An action's description can appear in the status bar",
    tooltip       = "A simple example action",
    menu_bar_path = "FileMenu/SubMenu/RadioGroup",
    tool_bar_path = "RadioGroup",
    style         = "radio",
)

overdrive_action = Action(
    id            = "action.example_action.OverdriveAction",
    class_name    = "action.example_action.ExampleAction",
    name          = "Overdrive",
    image         = "action/images/overdrive",
    description   = "An action's description can appear in the status bar",
    tooltip       = "A simple example action",
    menu_bar_path = "FileMenu/SubMenu/",
    tool_bar_path = "additions",
    style         = "toggle",
)

ui_actions = UIActions(
    menus = [
        sub_menu, file_menu
    ],

    actions = [
        do_it_action, higher_action, lower_action, overdrive_action
    ]
)

#### Views ####################################################################

ui_views = UIViews(
    views = [
        View(
            name       = "Stuff View",
            image      = "images/example_view.png",
            id         = "view.ExampleView",
            class_name = "view.ExampleView"
        )
    ]
)

###############################################################################
# The plugin definition!
###############################################################################

PluginDefinition(
    # The plugin's globally unique identifier.
    id = "enthought_envisage_example",

    # The name of the class that implements the plugin.
    class_name = "example_plugin.ExamplePlugin",

    # General information about the plugin.
    name          = "Envisage Example Plugin",
    version       = "1.0.0",
    provider_name = "Enthought Inc",
    provider_url  = "www.enthought.com",
    enabled       = True,
    autostart     = True,

    # The Id's of the plugins that this plugin requires.
    requires = [
        "enthought.envisage.ui",
    ],

    # The extension points offered by this plugin,
    extension_points = [],
    
    # The contributions that this plugin makes to extension points offered by
    # either itself or other plugins.
    extensions = [ui_actions, ui_views]
)

#### EOF ######################################################################
