from setuptools import setup, find_packages


# Function to convert simple ETS project names and versions to a requirements
# spec that works for both development builds and stable builds.  Allows
# a caller to specify a max version, which is intended to work along with
# Enthought's standard versioning scheme -- see the following write up:
#    https://svn.enthought.com/enthought/wiki/EnthoughtVersionNumbers
def etsdep(p, min, max=None, literal=False):
    require = '%s >=%s.dev' % (p, min)
    if max is not None:
        if literal is False:
            require = '%s, <%sa' % (require, max)
        else:
            require = '%s, <%s' % (require, max)
    return require


# Declare our ETS project dependencies.
ENVISAGE = etsdep('enthought.envisage', '2.0.1b1', '3.0')
#GOTCHA -- all import enclosed in try...except blocks so not needed
NAMING = etsdep('enthought.naming', '2.0.1b1', '3.0')
PYFACE = etsdep('enthought.pyface', '2.0.1b1', '3.0')
TRAITS_UI = etsdep('enthought.traits[ui]', '2.0.1b1', '3.0')
TRAITSUIWX = etsdep('enthought.traits.ui.wx', '2.0.1b1', '3.0')
UTIL_TRAITS = etsdep('enthought.util[traits]', '2.0.1b2', '3.0')


setup(
    author = 'Enthought, Inc',
    author_email = 'info@enthought.com',
    dependency_links = [
        'http://code.enthought.com/enstaller/eggs/source',
        ],
    description = "Physical 'units' package",
    extras_require = {
        "plugin": [
            ENVISAGE,
            NAMING,
            PYFACE,
            ],

        # All non-ets dependencies should be in this extra to ensure users can
        # decide whether to require them or not.
        'nonets': [
            # "wx",  # wx does not build as an egg cleanly on all platforms.
            ],
        },
    include_package_data = True,
    install_requires = [
        TRAITS_UI,
        TRAITSUIWX,
        UTIL_TRAITS,
        ],
    license = 'BSD',
    name = 'enthought.units',
    namespace_packages = [
        "enthought",
        ],
    packages = find_packages(),
    tests_require = [
        'nose >= 0.9',
        ],
    test_suite = 'nose.collector',
    url = 'http://code.enthought.com/ets',
    version = '2.0.1b1',
    zip_safe = False,
    )

